/**
 * \file
 * \brief A driver loop in C as an example on how to do processing of hrit files. 
 */

/*
 * Copyright 2010, 2011 Stijn Nevens (RMIB)
 *
 * This file is part of libxrit
 *
 * libxrit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libxrit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libxrit.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Project headers */

#include <config.h>
#include "xritlib.h" /* the thing we are driving here */

/* Standard library headers */

#include <stdarg.h> /* handle variable argument list */
#include <stdlib.h>

/* bats library headers */

#ifdef HAVE_RSLIB  /* For use when the full RMIB library is present. The header files here
                      actually tell the extract script what to take from the lib. Also, this allows
                      testing whether a problem is due to the small library or a real problem. */

#include "image_io.h" /* for IP_ImageWrite */
#include "comline.h" /* command line input */

#else              /* For use when the small RMIB library is to be used */

#include "rmibsmalllib.h"

#endif

int main(int argc, char **argv)
{
    int returnValue = 0; 

    CL_Init(argc, argv); /* initialize command line stuff */

    /* ****************** read in a single segment ****************** */

    /* data we are going to use: one segment within one channel */
    char testFile[] = "test/TestData/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_";

    /* set up the IP_rectangle-s to be used, these are used to define what portion of the image to take  */
    IP_rectangle sourceRectangle;
    IP_rectangle destinationRectangle;
    INIT_IP_RECTANGLE(sourceRectangle);
    INIT_IP_RECTANGLE(destinationRectangle);
    sourceRectangle.ul.x=1000; sourceRectangle.ul.y=3712-400; sourceRectangle.lr.x=2000; sourceRectangle.lr.y=3712-100;
    destinationRectangle.ul.x=0; destinationRectangle.ul.y=0; destinationRectangle.lr.x=1000; destinationRectangle.lr.y=300;

    /* Initialize and allocate the destionation image */
    IP_image *pDestinationImage = NULL;
    pDestinationImage = IP_ImageAlloc(INT2, 1000, 300); JOE; /* JOE jumps to the error handler in case of error in the previous */

    /* Use the function to read the HRIT image and fill the appropriate portion from the segment */
    HritFillImageFromSegment(testFile,&sourceRectangle,pDestinationImage,&destinationRectangle);

    #ifdef HAVE_RSLIB
    /* Write the image to disk */
    IP_ImageWrite((char *)"example/Output/oneSegment.rof", pDestinationImage); JOE; 
    #endif

    /* Reclaim the memory, this is for safety repeated after the error handling  */
    IP_ImageAdrFree(&pDestinationImage); /* use this if you want to be worry free (no dangling pointers), it sets the pointer to zero */


    /* ******************  read in multiple segments ******************  */

    /* set up the IP_rectangle-s to be used, these are used to define what portion of the image to take  */
    sourceRectangle.ul.x=1000; sourceRectangle.ul.y=3712-3100; sourceRectangle.lr.x=2000; sourceRectangle.lr.y=3712-100;
    destinationRectangle.ul.x=0; destinationRectangle.ul.y=0; destinationRectangle.lr.x=1000; destinationRectangle.lr.y=3000;

    /* Allocate the destionation image. */
    pDestinationImage = IP_ImageAlloc(INT2, 1000, 3000); JOE;

    /* Initialize and allocate the SM_strings structure containing the data files */
    SM_strings *pTestFilesSM = NULL;
    pTestFilesSM = SM_StringsAlloc(0);

    /* Add the files to be read to the SM_Strings structure */
    SM_StringsAddToStrings(pTestFilesSM,
    "test/TestData/H-000-MSG2__-MSG2________-WV_062___-000006___-201004191200-C_",
    "test/TestData/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_",
    "test/TestData/H-000-MSG2__-MSG2________-WV_062___-000002___-201004191200-C_",
    "test/TestData/H-000-MSG2__-MSG2________-WV_062___-000003___-201004191200-C_",
    "test/TestData/H-000-MSG2__-MSG2________-WV_062___-000004___-201004191200-C_",
    "test/TestData/H-000-MSG2__-MSG2________-WV_062___-000005___-201004191200-C_",
    "test/TestData/H-000-MSG2__-MSG2________-WV_062___-000007___-201004191200-C_",
    "test/TestData/H-000-MSG2__-MSG2________-WV_062___-000008___-201004191200-C_",
    /* if you uncomment the next line you can see the error handler for HritFillImageFromSegments in action 
       as this segemnt is not in the same channel as the pervious ones */
    /*"test/TestData/H-000-MSG2__-MSG2________-VIS008___-000004___-201004191200-C_",*/
    NULL);

    /* show what files we are going to process */
    SM_StringsDisplay(pTestFilesSM);

    /* Use the function to read the HRIT image and fill the appropriate portion from the segments */
    HritFillImageFromSegments(pTestFilesSM,&sourceRectangle,pDestinationImage,&destinationRectangle); JOE;

    /* Get a value as an integer */
    printf("We have as value at (x=100,y=200) : %i\n",IP_ImageGetIntValue(pDestinationImage, 100, 200));

    #ifdef HAVE_RSLIB
    /* Write the image to disk */
    IP_ImageWrite((char *)"example/Output/multipleSegment.rof", pDestinationImage); JOE; 
    #endif

    /* Reclaim the memory */
    IP_ImageAdrFree(&pDestinationImage);

    ERROR_HANDLER_BEGIN("driver");
    returnValue += 1;
    ERROR_HANDLER_END;
    IP_ImageAdrFree(&pDestinationImage);     /* All reclaiming of memory, should for safety be repeated after the error handling, 
                                                otherwise possible leaks might occur */
    ERROR_END;

    return(returnValue);
};
