/**
 * \file
 * \brief Header file containing the function prototypes and data structures used for processing of xrit files. 
 *
 * If used in C programs it provides the interface to the function and data structures that can be used in these.
 *
 * For C++ it also provides additional functionality.
 */

/*
 * Copyright 2010, 2011 Stijn Nevens (RMIB)
 *
 * This file is part of libxrit
 *
 * libxrit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libxrit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libxrit.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef HRITPROCESSING_H
#define HRITPROCESSING_H

/** \brief Number of line segments in an HRIT image segment */
#define NUM_LINE_SEGMENT 464 
/** \brief Length of the segment header in an HRIT image segment */
#define HRIT_SEGMENT_HEADER_LENGTH (163+3+NUM_LINE_SEGMENT*13)

/*
 * Header files used by both C and C++. 
 */

#ifdef __cplusplus
extern "C" {
#endif

/* bats library headers */

#ifdef XRIT_USE_SMALL_LIB

#include "rmibsmalllib.h"

#else                 /* For use when the full RMIB library is present. The header files here
                         actually tell the extract script what to take from the lib. Also, this allows
                         testing whether a problem is due to the small library or a real problem. */

#include "err_mng.h"  /* Used for error handling by both C and C++ */
#include "image.h"    /* For IP_Image and more */
#include "const.h"    /* int1, int2,...  */
#include "strlist.h"  /* SM_strings */

#endif

#ifdef __cplusplus
}
#endif

/*
 * Structures used by both C and C++. 
 */

#ifdef __cplusplus
extern "C" {
#endif

/** \brief A structure to take the CCSDS Day Segmented time in its SHORT version
 *
 * Short CDS time
 * CCSDS Day Segmented time in its SHORT (6 bytes, day since epoch + number of milliseconds in day) format. 
 * For the CDS time, the epoch is 1958 January 1st starting with 0. 
 * The CDS time is UTC-based and takes into account leap second corrections.
 * Day - UNSIGNED SHORT - 2 bytes 
 * Milliseconds of day - UNSIGNED - 4 bytes
 * 
 * See "MSG Level 1.5 Image Data Format Description" for further information.
 * See "Design Specification Standard Volume F: Data Types and Encoding Rules." for further information.
 */
typedef struct {
    char pField;         /**< \brief 1 byte should be 01000000 */
    u_int2 days;         /**< \brief UNSIGNED SHORT - 2 bytes */
    u_int4 milliSeconds; /**< \brief UNSIGNED - 4 bytes */
} MsgCdsTimeShort;


/** \brief A structure to take the CCSDS Day Segmented time in its EXPANDED version
 *
 * CDS time 
 * CCSDS Day Segmented time in its EXTENDED (10 bytes, day since epoch + see below) format. 
 * For the CDS time, the epoch is 1958 January 1st starting with 0. 
 * The CDS time is UTC-based and takes into account leap second corrections.
 * Day - UNSIGNED SHORT - 2 bytes 
 * Milliseconds of day - UNSIGNED - 4 bytes
 * MicroSeconds of Milliseconds - UNSIGNED SHORT - 2 bytes 
 * NanoSeconds of MicroSeconds - UNSIGNED SHORT - 2 bytes 
 * 
 * See "Design Specification Standard Volume F: Data Types and Encoding Rules." for further information.
 */
typedef struct {
    u_int2 days;         /**< \brief UNSIGNED SHORT - 2 bytes */
    u_int4 milliSeconds; /**< \brief UNSIGNED - 4 bytes */
    u_int2 microSeconds; /**< \brief UNSIGNED SHORT - 2 bytes */
    u_int2 nanoSeconds;  /**< \brief UNSIGNED SHORT - 2 bytes */
} MsgCdsTimeExpanded;


/** \brief A structure to take the CCSDS Day Segmented time in its CUC version
 *
 * CDS time 
 * CCSDS Day Segmented time in its CUC (4 + 3 bytes, coarse Time (seconds since epoch) + fine time) format. 
 * The CUC time in seconds since the epoch is given by :
 * C1*2563 + C2*2562 + C3*256 + C4 + F1*256-1 + F2*256-2 + F3*256-3.
 * For the CUC time, the epoch is application dependent. 
 * The CUC time is not UTC-based and leap second corrections do not apply.
 * 
 * See "Design Specification Standard Volume F: Data Types and Encoding Rules." for further information.
 */
typedef struct {
    u_int1 C1; /**< \brief  UNSIGNED BYTE - 1 byte */
    u_int1 C2; /**< \brief  UNSIGNED BYTE - 1 byte */
    u_int1 C3; /**< \brief  UNSIGNED BYTE - 1 byte */
    u_int1 C4; /**< \brief  UNSIGNED BYTE - 1 byte */
    u_int1 F1; /**< \brief  UNSIGNED BYTE - 1 byte */
    u_int1 F2; /**< \brief  UNSIGNED BYTE - 1 byte */
    u_int1 F3; /**< \brief  UNSIGNED BYTE - 1 byte */
} MsgCdsTimeUnsegmented;


/** \brief A structure to take the Generalized Time in its standard version
 *
 * The (standard) Generalized Time and the Expanded Generalized Time are both UTC-based and must be
 * followed by the character "Z". The Expanded Generalized Time must contain the character "." between
 * the coarse time and the fine time.
 * The HH, MM, SS characters stand for the hours in the day, minutes in the hour and seconds in the
 * minute, and vary in the range 00..23, 00..59 and 00.59 respectively.
 * 
 * See "Design Specification Standard Volume F: Data Types and Encoding Rules." for further information.
 */

typedef struct {
    char coarseTime; /**< \brief  CHARACTER STRING - 15 bytes - YYYYMMDDHHMMSS */
} MsgGeneralizedTimeStandard;


/** \brief A structure to take the Generalized Time in its standard version
 *
 * The (standard) Generalized Time and the Expanded Generalized Time are both UTC-based and must be
 * followed by the character "Z". The Expanded Generalized Time must contain the character "." between
 * the coarse time and the fine time.
 * The HH, MM, SS characters stand for the hours in the day, minutes in the hour and seconds in the
 * minute, and vary in the range 00..23, 00..59 and 00.59 respectively.
 * 
 * See "Design Specification Standard Volume F: Data Types and Encoding Rules." for further information.
 */

typedef struct {
    char coarseTime; /**< \brief  CHARACTER STRING - 15 bytes - YYYYMMDDHHMMSS */
    char fineTime;   /**< \brief  CHARACTER STRING - 10 bytes - mmmµµµnnn */
} MsgGeneralizedTimeExpanded;


/**
 * \brief Structure containing the primary header record
 * 
 * See "Coordination Group for Meteorological Satellites LRIT/HRIT Global Specification" for further information.
 */

typedef struct {
    u_int1 headerType;         /**< \brief header type, should be 0 */ 
    u_int2 headerRecordLength; /**< \brief header record length, should be 16 */
    u_int1 fillTypeCode;       /**< \brief fill type code */
    u_int4 totalHeaderLength;  /**< \brief total header length of all headers in octets */
    u_int_8 dataFieldLength;   /**< \brief data field length, ??? */
} HritPrimaryHeaderRecord;


/**
 * \brief Structure containing the image structure record
 * 
 * See "Coordination Group for Meteorological Satellites LRIT/HRIT Global Specification" for further information.
 */

typedef struct {
    u_int1 headerType;         /**< \brief header type, should be 1 */ 
    u_int2 headerRecordLength; /**< \brief header record length, should be set to 9 */
    u_int1 numberBitsPerPixel; /**< \brief the number of bits per pixel, should be 10 */
    u_int2 numberColumns;      /**< \brief the number of columns, should be 3712 or 5568 */
    u_int2 numberLines;        /**< \brief the number of lines, should be 464 */
    u_int1 compressionFlag;    /**< \brief compression flag: 0 = no compression, 1 = lossless compression, 2 = lossy compression */
} HritImageStructureRecord;


/**
 * \brief Structure containing the Image Navigation Record
 * 
 * See "Coordination Group for Meteorological Satellites LRIT/HRIT Global Specification" for further information.
 */

typedef struct {
    u_int1 headerType;           /**< \brief header type, should be 2 */ 
    u_int2 headerRecordLength;   /**< \brief header record length, should be set to 51 */
    char projectionName[33];     /**< \brief projection name, one extra byte for the end of string char */
    int4  columnScalingFactor;   /**< \brief column scaling factor */
    int4  lineScalingFactor;     /**< \brief line scaling factor */
    int4  columnOffset;          /**< \brief column offset */
    int4  lineOffset;            /**< \brief line offset*/
} HritImageNavigationRecord;


/**
 * \brief Structure containing the Image Data Function Record
 * 
 * See "Coordination Group for Meteorological Satellites LRIT/HRIT Global Specification" for further information.
 */

typedef struct {
    u_int1 headerType;                 /**< \brief header type, should be 3 */ 
    u_int2 headerRecordLength;         /**< \brief header record length, should be set to anything between 3 and 67 (for our use 64) */
    char dataDefinitionBlock[65532+1]; /**< \brief data definition block */
} HritImageDataFunctionRecord;

/**
 * \brief Structure containing the Annotation Record 
 * 
 * See "Coordination Group for Meteorological Satellites LRIT/HRIT Global Specification" for further information.
 */

typedef struct {
    u_int1 headerType;         /**< \brief header type, should be 4 */ 
    u_int2 headerRecordLength; /**< \brief header record length, should be set to ??? (unused) */
    char annotationText[64+1]; /**< \brief annotation text */
} HritAnnotationRecord;

/**
 * \brief Structure containing the Time Stamp Record
 *
 * See "Coordination Group for Meteorological Satellites LRIT/HRIT Global Specification" for further information.
 */

typedef struct {
    u_int1 headerType;            /**< \brief header type, should be 5 */ 
    u_int2 headerRecordLength;    /**< \brief header record length, should be set to anything between 3 and 67 (for our use 64) */
    MsgCdsTimeShort timeStamp;    /**< \brief time stamp record of the CCSDS time type */
} HritTimeStampRecord;


/**
 * \brief Structure containing the Segment Identification Record
 *
 * This file type provides additional information about the
 * satellite/ image processing status known at the start of a
 * SEVIRI level 1.5 data / FSD repeat cycle or processing
 * information of meteorological products.
 *
 * See "MSG Ground Segment LRIT/HRIT Mission Specific Implementation" for further information.
 */

typedef struct {
    u_int1 headerType;                /**< \brief header type, should be 128 */ 
    u_int2 headerRecordLength;        /**< \brief header record length, should be set to 13 */
    u_int2 gpScId;                    /**< \brief ??? */
    u_int1 spectralChannelID;         /**< \brief Spectral channel ID */
    u_int2 segmSeqNumber;             /**< \brief the segment sequence number, one for the first, 2 for the second, and so on... */
    u_int2 plannedStartSegmSeqNumber; /**< \brief planned start segment sequence number */
    u_int2 plannedEndSegmSeqNumber;   /**< \brief planned end segment sequence number */
    u_int1 dataFieldRepresentation;   /**< \brief defines the representation of the LRIT/HRIT data filed
                                       * no specific formatting (0), JPEG interchange format (1), 
                                       * T.4 coded file format (2), Wavelet coded file format (3)}
                                       */
} HritSegmentIdentificationRecord;


/**
 * \brief Structure containing the Line Quality Entry for a single line
 *
 * See "MSG Ground Segment LRIT/HRIT Mission Specific Implementation" for further information.
 */

typedef struct {
    int4 lineNumberInGrid;            /**< \brief line number in grid */ 
    MsgCdsTimeShort lineMeanAcquisition; /**< \brief line mean acquisition time of the CCSDS time type */ 
    u_int1 lineValidity;              /**< \brief line validity information 
                                       * Not derived (0), Nominal (1), Based on missing data (2),
                                       * Based on corrupted data (3), Based on replaced or interpolated (4)
                                       */
    u_int1 lineRadiometricQuality;    /**< \brief line validity information 
                                       * Not derived (0), Nominal (1), Usable (2),
                                       * Suspect (3), Do not use (4)
                                       */
    u_int1 lineGeometricQuality;      /**< \brief line validity information 
                                       * Not derived (0), Nominal (1), Usable (2),
                                       * Suspect (3), Do not use (4)
                                       */
} HritLineQualityEntry;


/**
 * \brief Structure containing the Segment Line Quality Record
 *
 * This file type contains status information known at the end 
 * of a repeat cycle of SEVIRI level 1.5 data.
 *
 * See "MSG Ground Segment LRIT/HRIT Mission Specific Implementation" for further information.
 */

typedef struct {
    u_int1 headerType;                                           /**< \brief header type, should be 129 */ 
    u_int2 headerRecordLength;                                   /**< \brief header record length, should be set to */
    HritLineQualityEntry hritLineQualityEntry[NUM_LINE_SEGMENT]; /**< \brief containing the Line Quality Entry for each single line */
} HritSegmentLineQualityRecord;


/**
 * \brief A structure containing the HRIT image segment header.
 *
 * See "MSG Level 1.5 Image Data Format Description" for further information.
 *
 * See "Coordination Group for Meteorological Satellites LRIT/HRIT Global Specification" for further information.
 *
 * See "MSG Ground Segment LRIT/HRIT Mission Specific Implementation" for further information.
 */

typedef struct {
    HritPrimaryHeaderRecord primaryHeaderRecord;
    HritImageStructureRecord imageStructureRecord;
    HritImageNavigationRecord imageNavigationRecord;
    HritAnnotationRecord annotationRecord;
    HritTimeStampRecord timeStampRecord;
    HritSegmentIdentificationRecord segmentIdentificationRecord;
    HritSegmentLineQualityRecord segmentLineQualityRecord;
} HritSegmentHeader;


/**
 * \brief A structure containing the HRIT prologue header.

 * - primaryHeaderRecord         -     16 bytes
 * - annotationRecord;           -     64 bytes
 * - timeStampRecord             -     10 bytes
 * 
 * - total  HritPrologueHeader   -     90 bytes
 *
 * See "MSG Level 1.5 Image Data Format Description" for further information.
 *
 * See "Coordination Group for Meteorological Satellites LRIT/HRIT Global Specification" for further information.
 *
 * See "MSG Ground Segment LRIT/HRIT Mission Specific Implementation" for further information.
 * 
 */

typedef struct {
    HritPrimaryHeaderRecord primaryHeaderRecord;
    HritAnnotationRecord annotationRecord;
    HritTimeStampRecord timeStampRecord;
} HritPrologueHeader;


/**
 * \brief A structure containing the SatelliteStatus record.
 * 
 * - satelliteDefinition         -      7 bytes
 * - satelliteOperations         -     28 bytes
 * - orbit                       -  39612 bytes
 * - attitude                    -  20420 bytes
 * - spinRateatRCStar            -      8 bytes
 * - utcCorrelation              -     59 bytes
 * 
 * - total satelliteStatus       -  60134 bytes
 * 
 * Offset including the 90 bytes of the header = 90
 *
 * See "MSG Level 1.5 Image Data Format Description" for further information.
 */

typedef struct {

    struct {
	u_int2 satelliteID;		/**< \brief the satellite‟s identification - 2 bytes
					 * It is the ID received by IMPF in the GS packet sub-header of the
					 * 1.0 data used to produce the 1.5 data.
					 */
        float nominalLongitude;		/**< \brief the satellite nominal longitude in degrees  - 4 bytes
					 * The origin is the Greenwich Meridian and a positive value corresponds 
					 * to a position East of it, a negative value to the West.
                                         * \note that this is the nominal (the zero of the frame) position from which we
                                         * can calculate the real position by calculation the real position using the 
                                         * orbitPolynomial member of HritSatelliteStatus::orbit
                                         */
        u_int1 hritSatelliteStatus;	/**< \brief the status of the satellite  - 1 byte */
    } satelliteDefinition; 		/**< \brief 7 bytes - offset = 90 + 0 = 90 
					 * (as we include the 0, the offset needs no +1 to be the right place in the stream 
					 */

    struct {
        u_int1 lastManoeuvreFlag;
        MsgCdsTimeShort lastManoeuvreStartTime;
        MsgCdsTimeShort lastManoeuvreEndTime;
        u_int1 lastManoeuvreType;
    
        u_int1 nextManoeuvreFlag;
        MsgCdsTimeShort nextManoeuvreStartTime;
        MsgCdsTimeShort nextManoeuvreEndTime;
        u_int1 nextManoeuvreType;
    } satelliteOperations; /**< \brief 28 bytes */

    struct  {
        MsgCdsTimeShort periodStartTime;
        MsgCdsTimeShort periodEndTime;
        struct {
            MsgCdsTimeShort startTime;
            MsgCdsTimeShort endTime;
            double x[8];
            double y[8];
            double z[8];
            double vx[8];
            double vy[8];
            double vz[8];
        } orbitPolynomial[100];
    } orbit;  /**< \brief 39612 bytes */

    struct {
        MsgCdsTimeShort periodStartTime;
        MsgCdsTimeShort periodEndTime;
        double principleAxisOffsetAngle;
        struct {
            MsgCdsTimeShort startTime;
            MsgCdsTimeShort endTime;
            double XofSpinAxis[8];
            double YofSpinAxis[8];
            double ZofSpinAxis[8];
        } attitudeCoef[100];
    } attitude; /**< \brief 20420 bytes */

    double spinRateatRCStart; /**< \brief 8 bytes */

    struct {
        MsgCdsTimeShort periodStartTime;
        MsgCdsTimeShort periodEndTime;
        MsgCdsTimeUnsegmented onBoardTimeStart;
        double varOnBoardTimeStart;
        double a1;
        double varA1;
        double a2;
        double varA2;
    } UtcCorrelation; /**< \brief 59 bytes */

} HritSatelliteStatus; 


/**
 * \brief A structure containing the ImageAcquisition record.
 *
 * - plannedAcquisitionTime      -        30 bytes
 * - radiometerStatus            -        54 bytes    
 * - radiometerSettings          -       586 bytes    
 * - radiometerOperations        -        30 bytes    
 * -
 * - total imageAcquisition      -       700 bytes    
 *
 * See "MSG Level 1.5 Image Data Format Description" for further information.
 */

typedef struct {

    struct {
        MsgCdsTimeExpanded trueRepeatCycleStart;
        MsgCdsTimeExpanded plannedForwardScanEnd;
        MsgCdsTimeExpanded plannedRepeatCycleEnd;
    } plannedAcquisitionTime; /**< \brief 30 bytes */

    struct  {
        u_int1 channelStatus[12];
        u_int1 detectorStatus[42];
    } radiometerStatus; /**< \brief 54 bytes */
    
    struct {
        u_int2 MduSamplingDelays[42];
    
        struct {
            u_int2 MduNomHRVDelay1;
            u_int2 MduNomHRVDelay2;
            struct { unsigned val : (16); } spare; /* make a struct so that struct.val is an unsigned of 16 bits */
            u_int2 MduNomHRVBreakline;
        } HRVFrameOffsets;
        
        u_int1 DHSSSynchSelection;
        u_int2 MduOutGain[42];
        u_int1 MduCoarseGain[42];
        u_int2 MduFineGain[42];
        u_int2 MduNumericalOffset[42];
        u_int2 PUGain[42];
        u_int2 PUOffset[27];
        u_int2 PUBias[15];
        
        struct {
            u_int2 L0_LineCounter;
            u_int2 K1_RetraceLines;
            u_int2 K2_PauseDeciseconds;
         /* u_int2 K3_RetraceLines;*/
            u_int2 K4_PauseDeciseconds;
            u_int2 K5_RetraceLines;
            u_int1 X_DeepSpaceWindowPosition;
        } operationParameters;
    
        u_int2 refocusingLines;
        u_int1 refocusingDirection;
        u_int2 refocusingPosition;
        u_int1 scanRefPosFlag;
        u_int2 scanRefPosNumber;
        float scanRefPotVal;
        u_int2 scanFirstLine;
        u_int2 scanLastLine;
        u_int2 retraceStartLine;
    } radiometerSettings; /**< \brief 586 bytes */

    struct {
        u_int1 lastGainChangeFlag;
        MsgCdsTimeShort lastGainChangTime;
    
        struct {
            u_int1 decontaminationNow;
            MsgCdsTimeShort decontaminationStart;
            MsgCdsTimeShort decontaminationEnd;
        } decontamination;
        
        u_int1 BBCalScheduled;
        u_int1 BBCalibrationType;
        
        u_int2 BBFirstLine;
        u_int2 BBLastLine;
        u_int2 coldFocalPlaneOpTemp;
        u_int2 warmFocalPlaneOpTemp;
    } radiometerOperations; /**< \brief 30 bytes */

} HritImageAcquisition;


/**
 * \brief A structure containing the EarthMoonSunCoef record.
 *
 * See "MSG Level 1.5 Image Data Format Description" for further information.
 */

typedef struct {
    MsgCdsTimeShort startTime;
    MsgCdsTimeShort endTime;
    double alphaCoef[8];
    double betaCoef[8];
} HritEarthMoonSunCoef;


/**
 * \brief A structure containing the CelestialEvents record.
 * 
 * - celestialBodiesPosition     -    326042 bytes
 * - relationToImage             -        16 bytes
 * -
 * - total celestialEvents       -    326058 bytes
 *
 * See "MSG Level 1.5 Image Data Format Description" for further information.
 */

typedef struct {

    struct {
        MsgCdsTimeShort periodStartTime;
        MsgCdsTimeShort periodEndTime;
        MsgGeneralizedTimeStandard relatedOrbitFileTime;
        MsgGeneralizedTimeStandard relatedAttitudeFileTime;
        HritEarthMoonSunCoef earthEphemeris[100];
        HritEarthMoonSunCoef moonEphemeris[100];
        HritEarthMoonSunCoef sunEphemeris[100];
        struct {
            struct {
                u_int2 starId;
                MsgCdsTimeShort startTime;
                MsgCdsTimeShort endTime;
                double alphaCoef[8];
                double betaCoef[8];
            } element[20];
        } starEphemeris[100];
    } celestialBodiesPosition; /**< \brief 326042 bytes */

    struct {
        u_int1 typeofEclipse;              /**< \brief 1 bytes */
        MsgCdsTimeShort eclipseStartTime;  /**< \brief 6 bytes */
        MsgCdsTimeShort eclipseEndTime;    /**< \brief 6 bytes */
        u_int1 visibleBodiesInImage;       /**< \brief 1 bytes */
        u_int1 bodiesClosetoFOV;           /**< \brief 1 bytes */
        u_int1 impactOnImageQuality;       /**< \brief 1 bytes */
    } relationToImage; /**< \brief 16 bytes */

} HritCelestialEvents;


/**
 * \brief A structure containing the ImageDescription record.
 *
 * - projectionDescription       -         5 bytes
 * - referenceGridVIS_IR         -        17 bytes
 * - referenceGridHRV            -        17 bytes
 * - plannedCoverageVIS_IR       -        16 byteS
 * - plannedCoverageHRV          -        32 bytes
 * - level1_5ImageProduction     -        14 bytes
 * 
 * - total imageDescription      -       101 bytes
 *
 * Offset including the 90 bytes of the header = 90+60134+700+326058+101=386982
 * 
 * See "MSG Level 1.5 Image Data Format Description" for further information.
 */

typedef struct {
    struct {
        u_int1 typeOfProjection;  /**< \brief 1 bytes */
        float longitudeOfSSP;     /**< \brief 4 bytes */
    } projectionDescription; /**< \brief 5 bytes */

    struct {
        int4 numberOfLines;      /**< \brief 4 bytes */
        int4 numberOfColumns;    /**< \brief 4 bytes */
        float lineDirGridStep;   /**< \brief 4 bytes */ 
        float columnDirGridStep; /**< \brief 4 bytes */
        u_int1 gridOrigin;       /**< \brief 4  bytes */
    } referenceGridVIS_IR; /**< \brief 17 bytes */

    struct {
        int4 numberOfLines;      /**< \brief 4 bytes */
        int4 numberOfColumns;    /**< \brief 4 bytes */
        float lineDirGridStep;   /**< \brief 4 bytes */ 
        float columnDirGridStep; /**< \brief 4 bytes */
        u_int1 gridOrigin;       /**< \brief 4  bytes */
    } referenceGridHRV; /**< \brief 17 bytes */

    struct {
        int4 southernLinePlanned;
        int4 northernLinePlanned;
        int4 easternColumnPlanned;
        int4 westernColumnPlanned;
    } plannedCoverageVIS_IR; /**< \brief 16 bytes */

    struct {
        int4 lowerSouthLinePlanned;
        int4 lowerNorthLinePlanned;
        int4 lowerEastColumnPlanned;
        int4 lowerWestColumnPlanned;
        int4 upperSouthLinePlanned;
        int4 upperNorthLinePlanned;
        int4 upperEastColumnPlanned;
        int4 upperWestColumnPlanned;
    } plannedCoverageHRV; /**< \brief 32 bytes - offset = 386982+5+17+17+16 = 387037 
                           * (as we include the 0, the offset needs no +1 to be the right place in the stream */

    struct {
        u_int1 imageProcDirection;
        u_int1 pixelGenDirection;
        u_int1 plannedChanProcessing[12]; /**< \brief 12 bytes - offset = 387037 + 32 + 2 = 387071 */
    } level1_5ImageProduction; /**< \brief 14 bytes */

} HritImageDescription;


/**
 * \brief A structure containing the HritRadiometricProcessing record.
 *
 * - RPSummary                   -        72 bytes
 * - level1_5ImageCalibration    -        16 bytes
 * - blackBodyDataUsed           -       967 bytes
 * - MPEFCalFeedBack             -       384 bytes
 * - radTransform[42][64];       -     10752 bytes
 * - radProcMTFAdaptation        -      5376 bytes
 * - straylightCorrection        -      3072 bytes
 *
 * - total radiometricProcessing -     20815 bytes
 *
 * Offset including the 90 bytes of the header = 90+60134+700+326058+101=387083
 *
 * See "MSG Level 1.5 Image Data Format Description" for further information.
 */

typedef struct {

    struct {
        u_int1 radianceLinearization[12]; /* 1 BYTE */
        u_int1 detectorEqualization[12];
        u_int1 onboardCalibrationResult[12];
        u_int1 MPEFCalFeedback[12];
        u_int1 MTFAdaptation[12];
        u_int1 straylightCorrectionFlag[12];
    } RPSummary; /**< \brief 72 bytes - offset = 387083 */

    struct {
        double cal_Slope;  /**< \brief 8 bytes - offset = 387155 */
        double cal_Offset; /**< \brief 8 bytes - offset = 387163 */
    } level1_5ImageCalibration[12]; /**< \brief 16 bytes - offset = 387155 */
    
    struct {
        MsgCdsTimeExpanded BBObservationUTC;
        
        struct {
            MsgCdsTimeUnsegmented onBoardBBTime;
            u_int2 MduOutGain[42];
            u_int1 MduCoarseGain[42];
            u_int2 MduFineGain[42];
            u_int2 MduNumericalOffset[42];
            u_int2 PUGain[42];
            u_int2 PUOffset[27];
            u_int2 PUBias[15];
            struct {  unsigned val : (12); } DCRValues[42]; /* make a struct so that struct.val is an unsigned of 12 bits */
            u_int1 X_DeepSpaceWindowPosition;

            struct {
                u_int2 FCUNominalColdFocalPlaneTemp;
                u_int2 FCURedundantColdFocalPlaneTemp;
            } coldFPTemperature;

            struct {
                u_int2 FCUNominalWarmFocalPlaneVHROTemp;
                u_int2 FCURedundantWarmFocalPlaneVHROTemp;
            } warmFPTemperature;

            struct {
                u_int2 FCUNominalScanMirrorSensor1Temp;
                u_int2 FCURedundantScanMirrorSensor1Temp;
                u_int2 FCUNominalScanMirrorSensor2Temp;
                u_int2 FCURedundantScanMirrorSensor2Temp;
            } scanMirrorTemperature;

            struct {
                u_int2 FCUNominalM1MirrorSensor1Temp;
                u_int2 FCURedundantM1MirrorSensor1Temp;
                u_int2 FCUNominalM1MirrorSensor2Temp;
                u_int2 FCURedundantM1MirrorSensor2Temp;
                u_int1 FCUNominalM23AssemblySensor1Temp;
                u_int1 FCURedundantM23AssemblySensor1Temp;
                u_int1 FCUNominalM23AssemblySensor2Temp;
                u_int1 FCURedundantM23AssemblySensor2Temp;
            } M1M2M3Temperature;

            struct {
                u_int2 FCUNominalM1BaffleTemp;
                u_int2 FCURedundantM1BaffleTemp;
            } baffleTemperature;

            struct {
                u_int2 FCUNominalBlackBodySensorTemp;
                u_int2 FCURedundantBlackBodySensorTemp;
            } blackBodyTemperature;

            struct {
                struct { unsigned val : (16); } FCUNominalSMMStatus; /* make a struct so that struct.val is an unsigned of 16 bits */
                struct { unsigned val : (16); } FCURedundantSMMStatus; /* make a struct so that struct.val is an unsigned of 16 bits */
            } FCUMode;

            struct {
                u_int4 numberOfPixelsUsed;
                float meanCount;
                float RMS;
                u_int2 maxCount;
                u_int2 minCount;
                double BB_Processing_Slope;
                double BB_Processing_Offset;
            } extractedBBData[12];

        } BBRelatedData;
    } blackBodyDataUsed; /**< \brief 967 bytes */


    struct {
        struct {
            u_int1 imageQualityFlag;  /**< \brief 1 byte */
            u_int1 referenceDataFlag; /**< \brief 1 byte */
            u_int1 absCalMethod;      /**< \brief 1 byte */
            char Pad1;                /**< \brief 1 byte */
            float absCalWeightVic;    /**< \brief 4 bytes */
            float absCalWeightXsat;   /**< \brief 4 bytes */
            float absCalCoeff;        /**< \brief 4 bytes */
            float absCalError;        /**< \brief 4 bytes */
            float calMonBias;         /**< \brief 4 bytes */
            float calMonRMS;          /**< \brief 4 bytes */
            float offsetCount;        /**< \brief 4 bytes */
        } element[12];
    } MPEFCalFeedback; /**< \brief 384 bytes */

    float radTransform[42][64]; /**< \brief 42*64*4 = 10752 bytes */

    struct {
        float VIS_IRMTFCorrectionE_W[33][16];
        float VIS_IRMTFCorrectionN_S[33][16];
        float HRVMTFCorrectionE_W[42-34+1][16];
        float HRVMTFCorrectionN_S[42-34+1][16];
    } radProcMTFAdaptation; /**< \brief 9*16*2*4+33*16*2*4 = 5376 bytes*/

    float straylightCorrection[12][8][8]; /**< \brief 12*8*8*4 = 3072 bytes */

} HritRadiometricProcessing;


/**
 * \brief A structure containing the HritGeometricProcessing record.

 * - optAxisDistances            -       336 bytes    
 * - earthModel                  -        25 bytes
 * - atmosphericModel            -     17280 bytes
 * - resamplingFunctions         -        12 bytes
 *
 * - Total geometricProcessing   -     17653 bytes    
 *
 * See "MSG Level 1.5 Image Data Format Description" for further information.
 */

typedef struct {

    struct /* OptAxisDistances */ {
        float E_WFocalPlane[42];
        float N_SFocalPlane[42];
    } optAxisDistances; /**< \brief 336 bytes */

    struct /* EarthModel */ {
        u_int1 typeOfEarthModel;
        double equatorialRadius;
        double northPolarRadius;
        double southPolarRadius;
    } earthModel; /**< \brief 25 bytes */

    float atmosphericModel[12][360]; /**< \brief 12*360*4 = 17280 bytes */
    u_int1 resamplingFunctions[12];  /**< \brief 12 bytes */
} HritGeometricProcessing; 


/**
 * \brief A structure containing the HritImpfConfiguration record. Unused for now.
 *
 * - overallConfiguration        -         4 bytes 
 * - suDetails [50]              -      2550 bytes
 * - warmStartParms              -     17328 bytes
 *
 * - Total impfConfiguration     -     19882 bytes 
 *
 * See "MSG Level 1.5 Image Data Format Description" for further information.
 */

typedef struct {

    struct /* type = GP_CONFIG_ITEM_VERSION */ {
        u_int2 issue;    /**< \brief 2 bytes - UNSIGNED SHORT - issue number */
        u_int2 revision; /**< \brief 2 bytes - UNSIGNED SHORT - revision number */
    } overallConfiguration; /**< \brief  4 bytes */

    struct {
        u_int4 siId;          /**< \brief 4 bytes - GP_SU_ID - guess is that it is an UNSIGNED which is 4 bytes long */
        u_int1 suIdInstance;  /**< \brief 1 bytes - BYTE - no idea on the type, guess is u_int1 */
        u_int1 suMode;        /**< \brief 1 bytes */
        u_int1 suState;       /**< \brief 1 bytes */

        struct {

            struct /* GP_CONFIG_ITEM_VERSION */ {
                u_int2 issue;    /**< \brief 2 bytes - UNSIGNED SHORT - issue number */
                u_int2 revision; /**< \brief 2 bytes - UNSIGNED SHORT - revision number */
            } swVersion; /**< \brief  4 bytes */

            struct /* GP_CONFIG_ITEM_VERSION */ {
                u_int2 issue;    /**< \brief 2 bytes - UNSIGNED SHORT - issue number */
                u_int2 revision; /**< \brief 2 bytes - UNSIGNED SHORT - revision number */
            } infoBaseVersions[10]; /**< \brief  4 bytes */

        } suConfiguration;
    } suDetails [50]; /**< \brief  51*50 = 2550 bytes  */

    struct /* blackBodyDataForWarmStart */ {
        double scanningLaw[1527];       /**< \brief 8*1527 = 12216 bytes */
        double radFramesAlignment [3];  /**< \brief 8*3 = 24 bytes */
        float scanningLawVariation [2]; /**< \brief 4*2 = 8 bytes */

        struct {
            float constCoef;     /**< \brief 4 bytes */
            float LinearCoef;    /**< \brief 4 bytes */
            float QuadraticCoef; /**< \brief 4 bytes */
        } equalisationParms[42]; /**< \brief  12*42 = 504 bytes */

        struct {
            double gTotalForMethod1[12];      /**< \brief 8*2 = 96 bytes */ 
            double gTotalForMethod2[12];      /**< \brief 8*2 = 96 bytes */
            double gTotalForMethod3[12];      /**< \brief 8*2 = 96 bytes */
            double BackForMethod1[12];        /**< \brief 8*2 = 96 bytes */
            double backForMethod2[12];        /**< \brief 8*2 = 96 bytes */
            double backForMethod3[12];        /**< \brief 8*2 = 96 bytes */
            double ratioGTotalToGBack[12];    /**< \brief 8*2 = 96 bytes */
            double gainInFrontOpticsCont[12]; /**< \brief 8*2 = 96 bytes */
            float calibrationConstants[12];   /**< \brief 4*2 = 48 bytes */
            double maxIncidentRadiance[12];   /**< \brief 8*2 = 96 bytes */
            double timeOfColdObsSeconds;      /**< \brief 8 bytes */
            double timeOfColdObsNanoSecs;     /**< \brief 8 bytes */
            double incidenceRadiance[12];     /**< \brief 8*2 = 96 bytes */
            double tempCal;                   /**< \brief 8 bytes */
            double tempM1;                    /**< \brief 8 bytes */
            double tempScan;                  /**< \brief 8 bytes */
            double tempM1Baf;                 /**< \brief 8 bytes */
            double tempCalSurround;           /**< \brief 8 bytes */
        } blackBodyDataForWarmStart; /**< \brief  1064 bytes */

        struct {
            double maxFeedbackVoltage; /**< \brief 8 bytes */
            double minFeedbackVoltage; /**< \brief 8 bytes */
            double mirrorSlipEstimate; /**< \brief 8 bytes */
        } mirrorParameters; /**< \brief  24 bytes */

        double lastSpinPeriod; /**< \brief 8 bytes */

        struct {
            MsgCdsTimeShort timeS0Packet; /**< \brief 6 bytes */
            MsgCdsTimeShort timeS1Packet; /**< \brief 6 bytes */
            MsgCdsTimeShort timeS2Packet; /**< \brief 6 bytes */
            MsgCdsTimeShort timeS3Packet; /**< \brief 6 bytes */
            MsgCdsTimeShort timeS4Packet; /**< \brief 6 bytes */
            MsgCdsTimeShort timeS5Packet; /**< \brief 6 bytes */
            MsgCdsTimeShort timeS6Packet; /**< \brief 6 bytes */
            MsgCdsTimeShort timeS7Packet; /**< \brief 6 bytes */
            MsgCdsTimeShort timeS8Packet; /**< \brief 6 bytes */
            MsgCdsTimeShort timeS9Packet; /**< \brief 6 bytes */
            MsgCdsTimeShort timeSYPacket; /**< \brief 6 bytes */
            MsgCdsTimeShort timePSPacket; /**< \brief 6 bytes */
        } hktmParameters; /**< \brief  12*6=72 bytes */

        char wspReserved[3408];/**< \brief 3408 bytes - OCTETSTRING SIZE (3408)*/

    }  warmStartParms; /**< \brief  17328 bytes */

} HritImpfConfiguration;


/**
 * \brief A structure containing the HRIT image prologue.
 *
 * 90+60134+700+326058+101+20815+17653 = 425551 bytes 
 * this is consistent with the files having 425551 bytes
 *
 * See "MSG Level 1.5 Image Data Format Description" for further information.
 */

typedef struct {
    HritPrologueHeader header;                       /*     90 bytes */ 
    HritSatelliteStatus satelliteStatus;             /*  60134 bytes */ 
    HritImageAcquisition imageAcquisition;           /*    700 bytes */
    HritCelestialEvents celestialEvents;             /* 326058 bytes */
    HritImageDescription imageDescription;           /*    101 bytes */
    HritRadiometricProcessing radiometricProcessing; /*  20815 bytes */
    HritGeometricProcessing geometricProcessing;     /*  17653 bytes */
    /* impfConfiguration is not in the prologue yet 
    HritImpfConfiguration impfConfiguration;             19786 bytes */
} HritPrologue;


/**
 * \brief A structure containing the HRIT image epilogue.
 */

typedef struct {
} HritEpilogue;


#ifdef __cplusplus
}
#endif

/*
 * Exclusive C++ functionality and the headers to be used for this.
 */

#ifdef __cplusplus

#include <sstream> /* used for xRITFileDecompress */

void NavigateTroughHritHeader(std::iostream &sourceStream);

void HritFillPrologueStructureFromPrologue(std::iostream &sourceStream, HritPrologue &prologue);

void HritFillPrologueStructureFromBaseDirSatelliteSlotAndChannel(char *baseDirectory,char *satellite, char *slot, 
                                                                 HritPrologue &prologue);

void xRITFileDecompress(char *filename, std::stringstream& strstream);

void HritFillSegmentHeaderFromSegment(std::stringstream &sourceStream, HritSegmentHeader &destiantionHeaderData);

int HritSegmentHeadersAreConsistent(HritSegmentHeader &headerData1,HritSegmentHeader &headerData2);

void HritFillImageFromSegmentAndHeader(std::stringstream &sourceStream, IP_rectangle &sourceRectangle, 
                                       IP_image &destinationImage, IP_rectangle &destinationRectangle,
                                       const HritSegmentHeader &segmentHeaderData );

void HritFillImageAndHeaderFromSegment(char *filename, IP_rectangle &sourceRectangle, 
                                       IP_image &destinationImage, IP_rectangle &destinationRectangle,
                                       HritSegmentHeader &segmentHeaderData);

void HritFillImageFromSegment(char *filename, IP_rectangle &sourceRectangle, 
                              IP_image &destinationImage, IP_rectangle &destinationRectangle);

void HritFillImageAndHeaderFromSegments(char **filenames, int numberOfFilenames, IP_rectangle &sourceRectangle, 
                                        IP_image &destinationImage, IP_rectangle &destinationRectangle,
                                        HritSegmentHeader &segmentHeaderData);

void HritFillImageAndHeaderFromSegments(char **filenames, int numberOfFilenames, IP_rectangle *pSourceRectangle, 
                                        IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle,
                                        HritSegmentHeader *pSegmentHeaderData);

void HritFillImageFromSegments(char **filenames, int numberOfFilenames, IP_rectangle &sourceRectangle, 
                               IP_image &destinationImage, IP_rectangle &destinationRectangle);

void HritFillImageFromSegments(char **filenames, int numberOfFilenames, IP_rectangle *pSourceRectangle, 
                               IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle);

void HritFillImageAndHeaderFromChannel(char *directory, char *channel, IP_rectangle &sourceRectangle, 
                                       IP_image &destinationImage, IP_rectangle &destinationRectangle,
                                       HritSegmentHeader &segmentHeaderData, char *product_id1 = NULL);

void HritFillImageFromChannel(char *directory, char *channel, IP_rectangle &sourceRectangle, 
                              IP_image &destinationImage, IP_rectangle &destinationRectangle);

#endif

/*
 * C and C++ functionality 
 */

#ifdef __cplusplus
extern "C" {
#endif

int HritSegmentHeadersAreConsistent(HritSegmentHeader* pHeaderData1,HritSegmentHeader* pHeaderData2);

void Unpack4PixelsFrom40Bits(unsigned char *pSourceData, u_int2 *pDestinationData);

char *HritConvertSeviriChannelNumberToChannel(int4 channelNumber);

char * HritGenerateSlotDirectoryName(char *baseDirectory,char *satellite, char *slot);

char * HritGeneratePrologueLocationWithProductID1(char *baseDirectory,char *satellite, char *slot, char *product_id1);

char * HritGeneratePrologueLocation(char *baseDirectory,char *satellite, char *slot); 

void HritFillPrologueStructureFromPrologue(char *filename, HritPrologue *pPrologue);

void HritFillPrologueStructureFromBaseDirSatelliteSlotAndChannel(char *baseDirectory,char *satellite, char *slot, 
                                                                 HritPrologue *pPrologue);

void HritFillImageAndHeaderFromSegment(char *filename, IP_rectangle *pSourceRectangle,
                                       IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle,
                                       HritSegmentHeader *pSegmentHeaderData);

void HritFillImageFromSegment(char *filename, IP_rectangle *pSourceRectangle, 
                              IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle);

void HritFillImageAndHeaderFromSegments(SM_strings *pFilenames, IP_rectangle *pSourceRectangle, 
                                        IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle,
                                        HritSegmentHeader *pSegmentHeaderData);

void HritFillImageFromSegments(SM_strings *pFilenames, IP_rectangle *pSourceRectangle, 
                               IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle);

void HritFillImageAndHeaderFromChannel(char *directory, char *channel, IP_rectangle *pSourceRectangle, 
                                       IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle,
                                       HritSegmentHeader *pSegmentHeaderData);

void HritFillImageFromChannel(char *directory, char *channel, IP_rectangle *pSourceRectangle, 
                              IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle);

void HritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannelWithProductID1(char *baseDirectory,char *satellite, char *product_id1,
                                                                            char *slot, int4 channelNumber, IP_rectangle *pSourceRectangle,
                                                                            IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle,
                                                                            HritSegmentHeader *pSegmentHeaderData);

void HritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel(char *baseDirectory,char *satellite, char *slot, 
                                                              int4 channelNumber, IP_rectangle *pSourceRectangle, 
                                                              IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle,
                                                              HritSegmentHeader *pSegmentHeaderData);

int TestHritProcessing();

#ifdef __cplusplus
}
#endif

#endif /* HRITPROCESSING_H */
