/**
 * \file
 * \brief C++ file containing the function implementations used for processing of xrit files.
 */

/*
 * Copyright 2010, 2011, 2012, 2013, 2014 Stijn Nevens (RMIB)
 * Copyright 2014, 2016 Alessandro Ipe (RMIB)
 * Copyright 2014, 2019, 2020 Edward Baudrez (RMIB)
 *
 * This file is part of libxrit
 *
 * libxrit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libxrit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libxrit.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Project headers */

#include <config.h>
#include "xritlib.h"

/* Standard library headers */

#include <stdlib.h> /* needed for now in xRITFileDecompress */

/* bats library headers */

#ifdef HAVE_RSLIB  /* For use when the full RMIB library is present. The header files here
                      actually tell the extract script what to take from the lib. Also, this allows
                      testing whether a problem is due to the small library or a real problem. */

#include "allocate.h"     /* allocation and deallocation */
#include "byte_swap.h"    /* byte handling */
#include "verbose.h"      /* verbose mode */
#include "comline.h"      /* command line input */
#include "file_mng.h"     /* for FM_FileRemove */
#include "data_preproc.h" /* for LU_DataPreProcess */ 
#include "regex_filter.h" /* for LU_StringExtractRegex */
#include "util.h"         /* for StringCopy */
#include "rect.h"         /* for IP_rectangle*/
#include "unix_file.h"    /* for FileExistence*/
#include "stringut.h"     /* for strings*/
#include "help.h"         /* for basename extraction*/

#else               /* For use when the small RMIB library is to be used */

#include "rmibsmalllib.h"

#endif

/* External headers (customized and debugged third party code) from the DISE library */

#include "CxRITFile.h"             /* From the "DISE" lib */
#include "CxRITFileDecompressed.h" /* From the "DISE" lib */

/* Extra headers used for debugging and testing (only used if feature is turned on) */

#ifdef DEBUG2
#include <iostream> 
#ifdef HAVE_RSLIB
#include "unix_time.h" /* for LU_DisplayTime */
#else   /* For use when the small RMIB library is to be used */
#include "rmibsmalllib.h"
#endif
#endif

#ifdef DEBUG
#include <iostream> 
#ifdef HAVE_RSLIB
#include "unix_time.h" /* for LU_DisplayTime */
#else /* For use when the small RMIB library is to be used */
#include "rmibsmalllib.h"
#endif
#endif

#ifdef DEBUG3
#include "xpifio.h"
#endif

/********************************************* Note from the editor ********************************************* 

Current development has ceased, one might like to implement the following functionality (and the version without Header):

void HritFillImageAndHeaderFromChannelNumber(char *directory, int4 channel,...){}; -> use a switch statement to go to HritFillImageFromChannel

 *********************************************** That's all folks ************************************************/

enum  bands { HRV, VIS006, VIS008, IR_016, IR_039, WV_062, WV_073, IR_087, IR_097, IR_108, IR_120, IR_134, bands_num };
char *bands_str[bands_num] = { "HRV___", "VIS006", "VIS008", "IR_016", "IR_039", "WV_062", "WV_073", "IR_087", "IR_097", "IR_108", "IR_120", "IR_134" };
int4  bands_seg[bands_num] = { 24, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8 };

/**
 * \brief Pad a string at the trailing end, to length \a max, using character \a c
 * \param input Input string as a character array
 * \param max Minimum string length
 * \param c Character to pad with
 * \return Right-padded string, of type \a std::string
 * \author Edward Baudrez
 * \date 11 February 2020
 * If the initial string length is \e larger than \a max, then the initial
 * length is used, in order to avoid truncating the result.
 */
std::string pad_right(char *input, size_t max, char c)
{
    std::string s(input);
    size_t n = s.length();
    if (n < max) s.append(max - n, c);
    return s;
}

void HritFillImageAndHeaderFromChannelWithSegmentSubset(char *directory, char *channel, IP_rectangle &sourceRectangle, 
                                                        IP_image &destinationImage, IP_rectangle &destinationRectangle,
                                                        HritSegmentHeader &segmentHeaderData){
/* use a new to be implemented version of HritFillImageAndHeaderFromChannel with a complain 
   switch about limited subset or provide a C interface to this function 
   with the complainAboutLimitedSubset switch turned on  */ 
}


/**
 * \brief Fill an HritPrologue from a directory containing slots in a tree ala RMIB. C++ only.
 * \param baseDirectory an array of characters containing the name of the directory where the slots are.
 * \param satellite an array of characters containing the satelite: MSG1 or MSG2.
 * \param slot an array of characters containing the slot: YYYYMMDDhhmm
 * \param prologue the HritPrologue that we will fill.
 * \return void
 *
 * \sa HritFillPrologueStructureFromPrologue(std::iostream&, HritPrologue&) for the real implementation
 * 
 * \note There is no C interface function provided yet in 
 *       HritFillPrologueStructureFromBaseDirSatelliteSlotAndChannel(char*,char*,char*,HritPrologue*).
 *
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void HritFillPrologueStructureFromBaseDirSatelliteSlotAndChannel(char *baseDirectory,char *satellite, char *slot, 
                                                                 HritPrologue &prologue){
    #ifdef DEBUG
    std::cout << "***************************************************************\n";
    std::cout << "Entering HritFillPrologueStructureFromBaseDirSatelliteSlotAndChannel\n";
    #endif
    
    /* setting up the internal used datastructures */
    char *prologueLocation = NULL;

    /* fetch the location of the prologue */
    prologueLocation = HritGeneratePrologueLocation(baseDirectory, satellite, slot);JOE;

    /* go to the workhorse and fill the prologue */                  
    HritFillPrologueStructureFromPrologue(prologueLocation,&prologue);JOE;

    ERROR_HANDLER_BEGIN("HritFillPrologueStructureFromBaseDirSatelliteSlotAndChannel");
    ERROR_HANDLER_END;
	deallocate(prologueLocation); prologueLocation=NULL;
    ERROR_END;
}



/**
 * \brief Return the full location of the prologue, with Product ID1 equal to the value supplied
 * \param baseDirectory an array of characters containing the name of the directory where the slots are.
 * \param satellite an array of characters containing the satelite: MSG1 or MSG2.
 * \param slot an array of characters containing the slot: YYYYMMDDhhmm
 * \param product_id1 spacecraft name, optionally augmented with an
 * undetermined qualifier (e.g., 'IODC'). This will be right-padded with
 * underscores.
 * \return a pointer to a string that contains the full location of the prologue file if success
 *         a NULL pointer otherwise. The location is given by 
 *         baseDirectory/satellite_YYYYMMDD/HRIT_YYYYMMDDhhmm/H-000-satellite__-product_id1-_________-PRO______-YYYYMMDDhhmm-__
 *
 * \author Stijn Nevens (RMIB), adapted by Edward Baudrez
 * \date Winter 2011, Winter 2020
 */
char * HritGeneratePrologueLocationWithProductID1(char *baseDirectory, char *satellite, char *slot, char *product_id1)
{
    #ifdef DEBUG
    std::cout << "***************************************************************\n";
    std::cout << "Entering " << __func__ << "\n";
    #endif

    char *pPrologueLocation = NULL;
    char *fullSlotDirectory = NULL;
    char file[100] = ""; // 100 should be sufficient
    std::string actual_product_id1 = pad_right(product_id1, 12, '_');
    char *small_slot = NULL;
    int4 ret;

    /* Get the directory*/
    fullSlotDirectory = HritGenerateSlotDirectoryName(baseDirectory, satellite, slot); JOE;

    /* build the filename (compressed) */
    small_slot = SM_StringExtract(slot, 0, 12); JOE;
    snprintf(file, 100, "H-000-%s__-%s-_________-PRO______-%s-__.bz2", satellite, actual_product_id1.c_str(), small_slot);

    /* get the prologue */
    pPrologueLocation = StringConcat(fullSlotDirectory, file); JOE;
    ret = FM_FileExistence(pPrologueLocation); JOE;
    if (ret != 1)
    {
        snprintf(file, 100, "H-000-%s__-%s-_________-PRO______-%s-__", satellite, actual_product_id1.c_str(), small_slot);
        
        /* get the prologue */
        pPrologueLocation = StringConcat(fullSlotDirectory, file); JOE;
        ret = FM_FileExistence(pPrologueLocation); JOE;
        ERROR_IF_M1(ret!=1,"No prologue file found in: %s", fullSlotDirectory);
    }
    
    ERROR_HANDLER_BEGIN(__func__);
        ADRFREE(pPrologueLocation);
    ERROR_HANDLER_END;
        deallocate(fullSlotDirectory); fullSlotDirectory=NULL;
        ADRFREE(small_slot);
    RETURN(pPrologueLocation);
}

/**
 * \brief Return the full location of the prologue
 * \param baseDirectory an array of characters containing the name of the directory where the slots are.
 * \param satellite an array of characters containing the satelite: MSG1 or MSG2.
 * \param slot an array of characters containing the slot: YYYYMMDDhhmm
 * \return a pointer to a string that contains the full location of the prologue file if success
 *         a NULL pointer otherwise. The location is given by
 *         baseDirectory/satellite_YYYYMMDD/HRIT_YYYYMMDDhhmm/H-000-satellite__-satellite________-_________-PRO______-YYYYMMDDhhmm-__
 *
 * \author Stijn Nevens (RMIB)
 * \date Winter 2011
 */

char * HritGeneratePrologueLocation(char *baseDirectory,char *satellite, char *slot){
    #ifdef DEBUG
    std::cout << "***************************************************************\n";
    std::cout << "Entering HritGeneratePrologueLocation\n";
    #endif

    char *pPrologueLocation = NULL;
    pPrologueLocation = HritGeneratePrologueLocationWithProductID1(baseDirectory, satellite, slot, satellite); JOE;

    ERROR_HANDLER_BEGIN("HritGeneratePrologueLocation");
        ADRFREE(pPrologueLocation);
    ERROR_HANDLER_END;
    RETURN(pPrologueLocation);
}


/**
 * \brief Fill an IP_Image from a directory containing slots in a tree ala RMIB. The input and output rectangle can be specified.
 * \param baseDirectory an array of characters containing the name of the directory where the slots are.
 * \param satellite an array of characters containing the satelite: MSG1 or MSG2.
 * \param product_id1 spacecraft name, optionally augmented with an
 * undetermined qualifier (e.g., 'IODC'). This will be right-padded with
 * underscores.
 * \param slot an array of characters containing the slot: YYYYMMDDhhmm
 * \param channelNumber the channel number
 * \param pSourceRectangle a pointer to an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param pDestinationImage a pointer to an IP_image where we will write the image.
 * \param pDestinationRectangle a pointer to and IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 *
 * The RMIB tree looks like this: baseDirectory/satelite_YYYYMMDD/HRIT_YYYYMMDDhhmm/
 * 
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 * \sa HritConvertSeviriChannelNumberToChannel(int4) for channelnumbers.
 *
 * \author Stijn Nevens (RMIB), adapted by Edward Baudrez (RMIB)
 * \date Winter 2011 (albeit a very sunny day), Winter 2020
 */

void HritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannelWithProductID1(char *baseDirectory,char *satellite, char *product_id1,
                                                                            char *slot, int4 channelNumber, IP_rectangle *pSourceRectangle, 
                                                                            IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle,
                                                                            HritSegmentHeader *pSegmentHeaderData){
    #ifdef DEBUG
    std::cout << "***************************************************************\n";
    std::cout << "Entering " << __func__ << "\n";
    #endif

    char *fullSlotDirectory = NULL;
    char *seviriChannelName = NULL;

    fullSlotDirectory = HritGenerateSlotDirectoryName(baseDirectory,satellite, slot);JOE;
    seviriChannelName = HritConvertSeviriChannelNumberToChannel(channelNumber); JOE;

    /* Go to the workhorse */
    HritFillImageAndHeaderFromChannel(fullSlotDirectory, seviriChannelName, *pSourceRectangle, 
                                      *pDestinationImage, *pDestinationRectangle,*pSegmentHeaderData,
                                      product_id1); JOE;

    ERROR_HANDLER_BEGIN(__func__);
    ERROR_HANDLER_END;
        deallocate(fullSlotDirectory); fullSlotDirectory=NULL;
        deallocate(seviriChannelName); seviriChannelName=NULL;
    ERROR_END;
}

/**
 * \brief Fill an IP_Image from a directory containing slots in a tree ala RMIB. The input and output rectangle can be specified.
 * \param baseDirectory an array of characters containing the name of the directory where the slots are.
 * \param satellite an array of characters containing the satelite: MSG1 or MSG2.
 * \param slot an array of characters containing the slot: YYYYMMDDhhmm
 * \param channelNumber the channel number
 * \param pSourceRectangle a pointer to an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param pDestinationImage a pointer to an IP_image where we will write the image.
 * \param pDestinationRectangle a pointer to and IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 *
 * The RMIB tree looks like this: baseDirectory/satelite_YYYYMMDD/HRIT_YYYYMMDDhhmm/
 * 
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 * \sa HritConvertSeviriChannelNumberToChannel(int4) for channelnumbers.
 *
 * \author Stijn Nevens (RMIB)
 * \date Winter 2011 (albeit a very sunny day)
 */

void HritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel(char *baseDirectory,char *satellite, char *slot, 
                                                              int4 channelNumber, IP_rectangle *pSourceRectangle, 
                                                              IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle,
                                                              HritSegmentHeader *pSegmentHeaderData){
    #ifdef DEBUG
    std::cout << "***************************************************************\n";
    std::cout << "Entering HritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel\n";
    #endif

    HritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannelWithProductID1(baseDirectory, satellite, satellite, slot, 
                                                                           channelNumber, pSourceRectangle, 
                                                                           pDestinationImage, pDestinationRectangle,
                                                                           pSegmentHeaderData); JOE;

    ERROR_HANDLER_BEGIN("HritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel");
    ERROR_HANDLER_END;
    ERROR_END;
}

/**
 * \brief Generate the directory containing the segments from a base directory, a slot and a satelite
 * \param baseDirectory an array of characters containing the base directory containing the HRIT images 
 * \param satellite an array of characters containing the satelite: MSG1 or MSG2.
 * \param slot an array of characters containing the slot: YYYYMMDDhhmm
 * \return a pointer to a string that contains the directory name if succes
 *         a NULL pointer otherwise. The directory name is baseDirectory/satelite_YYYYMMDD/HRIT_YYYYMMDDhhmm/
 * 
 * Example: 
 * - baseDirectory = /myBase
 * - satelite= MSG1
 * - slot = 200406220000
 * - return = /myBase/MSG1_20040622/HRIT_200406220000/
 * 
 * \author Stijn Nevens (RMIB)
 * \date Winter 2011
 */

char * HritGenerateSlotDirectoryName(char *baseDirectory,char *satellite, char *slot){
    std::string fullSlotDirectory;
    char *fullSlotDirectoryAsChar = NULL;

    /* Input verification */
    ERROR_IF_V(baseDirectory==NULL);
    ERROR_IF_V(satellite==NULL);
    ERROR_IF_V(slot==NULL);

    /* Generate the string */
    fullSlotDirectory.append(baseDirectory);
    if(fullSlotDirectory.at(fullSlotDirectory.length()-1)!='/') fullSlotDirectory.append("/"); /* add a / if needed */

    fullSlotDirectory.append(satellite);
    fullSlotDirectory.append("_");
    fullSlotDirectory.append(slot,8);
    fullSlotDirectory.append("/");

    fullSlotDirectory.append("HRIT_");
    fullSlotDirectory.append(slot,12);
    fullSlotDirectory.append("/");

    #ifdef DEBUG
    std::cout << fullSlotDirectory << "\n";
    #endif

    fullSlotDirectoryAsChar = StringCopy(fullSlotDirectory.c_str()); /* convert to a no const string */

    /* Error handling */
    ERROR_HANDLER_BEGIN("HritGenerateSlotDirectoryName");
        deallocate(fullSlotDirectoryAsChar); fullSlotDirectoryAsChar=NULL;
    ERROR_HANDLER_END;
    RETURN(fullSlotDirectoryAsChar);
}


/**
 * \brief Convert a Seviri channel number to the right channel name.
 * \param channelNumber The channelnumber
 * \return a *char containing the channel if verything went ok. NULL otherwise.
 *
 * - 0  = "HRV" 
 * - 1  = "VIS006"
 * - 2  = "VIS008"
 * - 3  = "IR_016"
 * - 4  = "IR_039" 
 * - 5  = "WV_062"
 * - 6  = "WV_073"
 * - 7  = "IR_087"
 * - 8  = "IR_097"
 * - 9  = "IR_108"
 * - 10 = "IR_120"
 * - 11 = "IR_134"
 *
 * \note EUMETSAT uses the logic for its vectors:
 * - 1  = "VIS006"
 * - 2  = "VIS008"
 * - 3  = "IR_016"
 * - 4  = "IR_039" 
 * - 5  = "WV_062"
 * - 6  = "WV_073"
 * - 7  = "IR_087"
 * - 8  = "IR_097"
 * - 9  = "IR_108"
 * - 10 = "IR_120"
 * - 11 = "IR_134"
 * - 12 = "HRV" 
 * With the array index starting at 1 (altought their code is no doubt C++)
 * Such that whenever you need an array read from their data you need the following numbers:
 * - 0  = "VIS006"
 * - 1  = "VIS008"
 * - 2  = "IR_016"
 * - 3  = "IR_039" 
 * - 4  = "WV_062"
 * - 5  = "WV_073"
 * - 6  = "IR_087"
 * - 7  = "IR_097"
 * - 8  = "IR_108"
 * - 9 = "IR_120"
 * - 10 = "IR_134"
 * - 11 = "HRV" 
 * This is channelNumber - 1 for all but the HRV channel
 * To convert to EUMETSAT logic use the following code:
 *
 * \code 
 * int eumetsatChannelNumber, channelNumber;
 * eumetsatChannelNumber = (channelNumber + 11) % 12;
 * \endcode
 *
 * where 11 = 12 - 1
 *
 * \author Stijn Nevens (RMIB)
 * \date Winter 2011
 */

char *HritConvertSeviriChannelNumberToChannel(int4 channelNumber){
    char *channelName = NULL;

    switch(channelNumber){
        case 0:  channelName = StringCopy("HRV");    break;
        case 1:  channelName = StringCopy("VIS006"); break;
        case 2:  channelName = StringCopy("VIS008"); break;
        case 3:  channelName = StringCopy("IR_016"); break;
        case 4:  channelName = StringCopy("IR_039"); break;
        case 5:  channelName = StringCopy("WV_062"); break;
        case 6:  channelName = StringCopy("WV_073"); break;
        case 7:  channelName = StringCopy("IR_087"); break;
        case 8:  channelName = StringCopy("IR_097"); break;
        case 9:  channelName = StringCopy("IR_108"); break;
        case 10: channelName = StringCopy("IR_120"); break;
        case 11: channelName = StringCopy("IR_134"); break;
        default: ERROR_M1("The channelnumber is out of range (0<= %i <=11)", channelNumber);
    }

    /* Error handling */
    ERROR_HANDLER_BEGIN("HritConvertSeviriChannelNumberToChannel");
        deallocate(channelName); channelName=NULL;
    ERROR_HANDLER_END;
    RETURN(channelName);
}


/**
 * \brief Fill an HritPrologue from a stream containing the content of a decompressed HRIT prologue file. C++ only.
 * \param sourceStream is a iostream passed by reference containing the decompressed content of an HRIT prologue file.
 * \param prologue the HritPrologue that we will fill.
 * \return void
 *
 * Read some fields of a HRIT "image repeat cycle prologue" file. Here 
 * the "PlannedCoverageHRV" record is read from the 
 * "ImageDescription" record and the "Level1_5ImageCalibration" 
 * records are read for the 12 channels in the "RadiometricProcessing" 
 * record. All this information is stored in the HritPrologue structure.
 *
 * ATTENTION: bring your own code, if you need more of the structure to be filled, add the code below,
 * the structure is fully set up. Don't forget to supplement tests.
 * 
 * \note There is a C interface function provided in 
 *       HritFillPrologueStructureFromPrologue(char*,HritPrologue*).
 *
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void HritFillPrologueStructureFromPrologue(std::iostream &sourceStream, HritPrologue &prologue) {
    #ifdef DEBUG
    std::cout << "***************************************************************\n";
    std::cout << "Entering HritFillPrologueStructureFromPrologue\n";
    u_int1 test = 0;
    int debugInt = test;
    #endif

    int4 byteSexFlag;
    int4 k;

    /* set the bytesex flag, the HRIT files have big endian */
    byteSexFlag = 0;
    if(ByteSex()==LU_LITTLE_ENDIAN) byteSexFlag=1;
    
    /* -- satelliteStatus -- */
    
    /* fill prologue.satelliteStatus.satelliteDefinition */    
    sourceStream.seekg(90 , std::ios::beg);
    
    sourceStream.read((char *)(&prologue.satelliteStatus.satelliteDefinition.satelliteID),2);
    if(byteSexFlag) MP_DataByteSwap(&prologue.satelliteStatus.satelliteDefinition.satelliteID, U_INT2);JOE;

    sourceStream.read((char *)(&prologue.satelliteStatus.satelliteDefinition.nominalLongitude),4);
    if(byteSexFlag) MP_DataByteSwap(&prologue.satelliteStatus.satelliteDefinition.satelliteID, FLOAT);JOE;
    
    sourceStream.read((char *)(&prologue.satelliteStatus.satelliteDefinition.hritSatelliteStatus),1); 
    
    #ifdef DEBUG
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Contents of hritPrologue:\n";
    std::cout << "prologue.satelliteStatus.satelliteDefinition.satelliteID = " 
              <<  prologue.satelliteStatus.satelliteDefinition.satelliteID << "\n";
    std::cout << "prologue.satelliteStatus.satelliteDefinition.nominalLongitude = " 
              <<  prologue.satelliteStatus.satelliteDefinition.nominalLongitude << "\n";
    debugInt = prologue.satelliteStatus.satelliteDefinition.hritSatelliteStatus;
    std::cout << "prologue.satelliteStatus.satelliteDefinition.hritSatelliteStatus = " 
	      <<  debugInt << "\n";      
    #endif

    /* -- imageDescription -- */

    /* fill prologue.imageDescription.plannedCoverageHRV */
    sourceStream.seekg(387037 , std::ios::beg);

    /* a smarter person with more time to think might do this with a for loop using pointer arithmetic -> I am not that guy */
    sourceStream.read((char *)(&prologue.imageDescription.plannedCoverageHRV.lowerSouthLinePlanned),4);
    if(byteSexFlag) MP_DataByteSwap(&prologue.imageDescription.plannedCoverageHRV.lowerSouthLinePlanned, U_INT4);JOE;

    sourceStream.read((char *)(&prologue.imageDescription.plannedCoverageHRV.lowerNorthLinePlanned),4);
    if(byteSexFlag) MP_DataByteSwap(&prologue.imageDescription.plannedCoverageHRV.lowerNorthLinePlanned, U_INT4);JOE;

    sourceStream.read((char *)(&prologue.imageDescription.plannedCoverageHRV.lowerEastColumnPlanned),4);
    if(byteSexFlag) MP_DataByteSwap(&prologue.imageDescription.plannedCoverageHRV.lowerEastColumnPlanned, U_INT4);JOE;

    sourceStream.read((char *)(&prologue.imageDescription.plannedCoverageHRV.lowerWestColumnPlanned),4);
    if(byteSexFlag) MP_DataByteSwap(&prologue.imageDescription.plannedCoverageHRV.lowerWestColumnPlanned, U_INT4);JOE;

    sourceStream.read((char *)(&prologue.imageDescription.plannedCoverageHRV.upperSouthLinePlanned),4);
    if(byteSexFlag) MP_DataByteSwap(&prologue.imageDescription.plannedCoverageHRV.upperSouthLinePlanned, U_INT4);JOE;

    sourceStream.read((char *)(&prologue.imageDescription.plannedCoverageHRV.upperNorthLinePlanned),4);
    if(byteSexFlag) MP_DataByteSwap(&prologue.imageDescription.plannedCoverageHRV.upperNorthLinePlanned, U_INT4);JOE;

    sourceStream.read((char *)(&prologue.imageDescription.plannedCoverageHRV.upperEastColumnPlanned),4);
    if(byteSexFlag) MP_DataByteSwap(&prologue.imageDescription.plannedCoverageHRV.upperEastColumnPlanned, U_INT4);JOE;

    sourceStream.read((char *)(&prologue.imageDescription.plannedCoverageHRV.upperWestColumnPlanned),4);
    if(byteSexFlag) MP_DataByteSwap(&prologue.imageDescription.plannedCoverageHRV.upperWestColumnPlanned, U_INT4);JOE;

    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Contents of hritPrologue:\n";
    std::cout << "prologue.imageDescription.plannedCoverageHRV.lowerSouthLinePlanned = " 
              <<  prologue.imageDescription.plannedCoverageHRV.lowerSouthLinePlanned << "\n";
    std::cout << "prologue.imageDescription.plannedCoverageHRV.lowerNorthLinePlanned = " 
              <<  prologue.imageDescription.plannedCoverageHRV.lowerNorthLinePlanned << "\n";
    std::cout << "prologue.imageDescription.plannedCoverageHRV.lowerEastColumnPlanned = " 
              <<  prologue.imageDescription.plannedCoverageHRV.lowerEastColumnPlanned << "\n";
    std::cout << "prologue.imageDescription.plannedCoverageHRV.lowerWestColumnPlanned = " 
              <<  prologue.imageDescription.plannedCoverageHRV.lowerWestColumnPlanned << "\n";
    std::cout << "prologue.imageDescription.plannedCoverageHRV.upperSouthLinePlanned = " 
              <<  prologue.imageDescription.plannedCoverageHRV.upperSouthLinePlanned << "\n";
    std::cout << "prologue.imageDescription.plannedCoverageHRV.upperNorthLinePlanned = " 
              <<  prologue.imageDescription.plannedCoverageHRV.upperNorthLinePlanned << "\n";
    std::cout << "prologue.imageDescription.plannedCoverageHRV.upperEastColumnPlanned = " 
              <<  prologue.imageDescription.plannedCoverageHRV.upperEastColumnPlanned << "\n";
    std::cout << "prologue.imageDescription.plannedCoverageHRV.upperWestColumnPlanned = " 
              <<  prologue.imageDescription.plannedCoverageHRV.upperWestColumnPlanned << "\n";
    #endif

    /* fill prologue.imageDescription.level1_5ImageProduction.plannedChanProcessing[12] 
       possible values are:
       - 0: not processed
       - 1: processed in spectral radiance
       - 2: processed  in effective radiance */
    sourceStream.seekg(387071 , std::ios::beg); 

    for (k=0; k<12; k++)
    {
        sourceStream.read((char *)(&prologue.imageDescription.level1_5ImageProduction.plannedChanProcessing[k]),1);
	#ifdef DEBUG2
	int debugInt2 = prologue.imageDescription.level1_5ImageProduction.plannedChanProcessing[k];
	std::cout << "prologue.imageDescription.level1_5ImageProduction.plannedChanProcessing[" << k << "] = " 
		  <<  debugInt2 << "\n";
	#endif
    }

    /* -- level1_5ImageCalibration -- */

    /* fill prologue.radiometricProcessing.level1_5ImageCalibration */
    sourceStream.seekg(387155 , std::ios::beg); 

    for (k=0; k<12; k++)
    {
        sourceStream.read((char *)(&prologue.radiometricProcessing.level1_5ImageCalibration[k].cal_Slope),8);
        if(byteSexFlag) MP_DataByteSwap(&prologue.radiometricProcessing.level1_5ImageCalibration[k].cal_Slope, DOUBLE);JOE;

        sourceStream.read((char *)(&prologue.radiometricProcessing.level1_5ImageCalibration[k].cal_Offset),8);
        if(byteSexFlag) MP_DataByteSwap(&prologue.radiometricProcessing.level1_5ImageCalibration[k].cal_Offset, DOUBLE);JOE;
        
        #ifdef DEBUG2
        std::cout.precision(500);
	std::cout << "prologue.radiometricProcessing.level1_5ImageCalibration[" << k << "].cal_Slope = " 
		  << prologue.radiometricProcessing.level1_5ImageCalibration[k].cal_Slope  << "\n";
	std::cout << "prologue.radiometricProcessing.level1_5ImageCalibration[" << k << "].cal_Offset = " 
		  << prologue.radiometricProcessing.level1_5ImageCalibration[k].cal_Offset  << "\n";
	#endif
    }
    /* We need to add error handling here */
    
    /* Error handling */
    ERROR_HANDLER_BEGIN("HritFillPrologueStructureFromPrologue");
    ERROR_HANDLER_END;
    ERROR_END;
}


/**
 * \brief Fill an HritPrologue from a stream containing the content of a decompressed HRIT prologue file.
 * \param filename an array of characters containing the filename of an HRIT prologue file.
 * \param pPrologue a pointer to the HritPrologue that we will fill.
 * \return void
 * 
 * An overloaded version of HritFillPrologueStructureFromPrologue(std::iostream&,HritPrologue&)
 *
 * \author Stijn Nevens (RMIB)
 * \date Summer 2010
 */

void HritFillPrologueStructureFromPrologue(char *filename, HritPrologue *pPrologue) {
    std::fstream fileStream;
    char *pPreProcessedFilename = NULL;
    int4 deleteFlagPreProcessedFile = 0;

    pPreProcessedFilename= LU_DataPreProcess(filename, &deleteFlagPreProcessedFile);JOE;

    try
    {
        fileStream.open(pPreProcessedFilename, std::fstream::in | std::fstream::binary);
    }
    catch (...)
    {
        ERROR_M1("Something went wrong opening the filestream: %s",pPreProcessedFilename);
    }

    /* Run the program */
    HritFillPrologueStructureFromPrologue(fileStream,*pPrologue);JOE;

    /* Error handling */
    ERROR_HANDLER_BEGIN("HritFillPrologueStructureFromPrologue");
    ERROR_HANDLER_END;
        try
        {
            fileStream.close();
        }
        catch (...)
        {
           /* In fact this is not needed, nor is the explicit closing of the filestream, we do it anyway              
              ERROR_M1("Something went wrong closing the filestream: %s", pPreProcessedFilename); */
        }
        if(deleteFlagPreProcessedFile==1)
        {
            FM_FileRemove(pPreProcessedFilename);
            deallocate(pPreProcessedFilename); pPreProcessedFilename=NULL;
            deleteFlagPreProcessedFile=0;
        }
    ERROR_END;
}


/**
 * \brief Fill an IP_Image from a channel in a directory. The input and output rectangle can be specified. C++ only.
 * \param directory an array of characters containing the name of the directory where the slot (all the files form a particular time) is.
 * \param channel an array of characters containing the channel, these have to be: 
*        \code HRV IR_016 IR_039 IR_087 IR_097 IR_108 IR_120 IR_134 VIS006 VIS008 WV_062 WV_073 \endcode
 * \param sourceRectangle an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param destinationImage an IP_image where we will write the image.
 * \param destinationRectangle and IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \return void
 *
 * This function is provided for your convenience.
 *
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 *
 * \note There is a C interface function provided in 
 * HritFillImageFromChannel(char*, char*, IP_rectangle*, IP_image, IP_rectangle*)
 *
 * \author Stijn Nevens (RMIB)
 * \date Autumn 2010
 */

void HritFillImageFromChannel(char *directory, char *channel, IP_rectangle &sourceRectangle, 
                              IP_image &destinationImage, IP_rectangle &destinationRectangle){
    HritSegmentHeader segmentHeaderData;
    HritFillImageAndHeaderFromChannel(directory, channel, sourceRectangle, 
                                      destinationImage, destinationRectangle,
                                      segmentHeaderData);
}


/**
 * \brief Fill an IP_Image and segmentHeaderData from a channel in a directory. The input and output rectangle can be specified.
 * \param directory an array of characters containing the name of the directory where the slot (all the files form a particular time) is.
 * \param channel an array of characters containing the channel, these have to be: 
*        \code HRV IR_016 IR_039 IR_087 IR_097 IR_108 IR_120 IR_134 VIS006 VIS008 WV_062 WV_073 \endcode
 * \param pSourceRectangle a pointer to an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param pDestinationImage a pointer to an IP_image where we will write the image.
 * \param pDestinationRectangle a pointer to and IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \return void
 *
 * An overloaded version of HritFillImageAndHeaderFromChannel(char*, char*, IP_rectangle&, IP_image&, IP_rectangle&)
 *
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 * 
 * \author Stijn Nevens (RMIB)
 * \date Autumn 2010
 */

void HritFillImageFromChannel(char *directory, char *channel, IP_rectangle *pSourceRectangle, 
                                       IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle){
    HritSegmentHeader segmentHeaderData;
    HritFillImageAndHeaderFromChannel(directory, channel, *pSourceRectangle,
                                      *pDestinationImage, *pDestinationRectangle,
                                      segmentHeaderData);
}


/**
 * \brief Fill an IP_Image and segmentHeaderData from a channel in a directory. The input and output rectangle can be specified. C++ only.
 * \param directory an array of characters containing the name of the directory where the slot (all the files form a particular time) is.
 * \param channel an array of characters containing the channel, these have to be: 
*        \code HRV HRV_ HRV__ IR_016 IR_039 IR_087 IR_097 IR_108 IR_120 IR_134 VIS006 VIS008 WV_062 WV_073 \endcode
 * \param sourceRectangle an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param destinationImage an IP_image where we will write the image.
 * \param destinationRectangle and IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \param segmentHeaderData the HritSegmentHeader where we will write the corresponding to the segment header. 
 * \param product_id1 spacecraft name, optionally augmented with an
 *        undetermined qualifier (e.g., 'IODC'). This will be right-padded with
 *        underscores. If null, the spacecraft name is used.
 * \return void
 *
 * Error handling: The image is filled with all available segements if not all segments are present a warning
 * is issued. If 0 segemts are found or if more than 8 (24 for the HRV channels) segments are found an error is thrown.
 *
 * This function is provided for your convenience.
 *
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 *
 * \note There is a C interface function provided in 
 * HritFillImageAndHeaderFromChannel(char*, char*, IP_rectangle*, IP_image, IP_rectangle*, HritSegmentHeader*)
 * 
 * \author Stijn Nevens (RMIB)
 * \date Summer 2010
 */

void HritFillImageAndHeaderFromChannel(char *directory, char *channel, IP_rectangle &sourceRectangle, 
                                       IP_image &destinationImage, IP_rectangle &destinationRectangle,
                                       HritSegmentHeader &segmentHeaderData, char *product_id1)
{
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering HritFillImageAndHeaderFromChannel\n";
    #endif

    SM_strings *pFilenames = NULL;
    char file[100] = ""; // 100 should be sufficient
    char *dum = NULL;
    std::string actual_product_id1;
    char *satellite = NULL, *slot = NULL, *filename = NULL;
    int4 band, seg, ret;

    /* Input verification */

    ERROR_IF_V(directory==NULL);
    ERROR_IF_V(channel==NULL);
    if (strcmp(channel, "HRV") == 0 ||
        strcmp(channel, "HRV_") == 0 ||
        strcmp(channel, "HRV__") == 0) {
        channel = "HRV___";
    }

    ERROR_IF_M1( /* When one of these equals we have a 0 (false) and hence they are all false */
                 strcmp(channel, "HRV___") &&
                 strcmp(channel, "IR_016") &&
                 strcmp(channel, "IR_039") &&
                 strcmp(channel, "IR_087") &&
                 strcmp(channel, "IR_097") &&
                 strcmp(channel, "IR_108") &&
                 strcmp(channel, "IR_120") &&
                 strcmp(channel, "IR_134") &&
                 strcmp(channel, "VIS006") &&
                 strcmp(channel, "VIS008") &&
                 strcmp(channel, "WV_062") &&
                 strcmp(channel, "WV_073"),
                 "Channel of wrong type: %s", channel);

    dum       = SM_StringExtractBasenameLevel(directory, 2); JOE;
    satellite = SM_StringExtract(dum, 0, 4); JOE;
    ADRFREE(dum);
    actual_product_id1 = pad_right(product_id1 ? product_id1 : satellite, 12, '_');

    dum       = SM_StringExtractBasename(directory); JOE;
    slot      = SM_StringExtract(dum, 5, 17); JOE;
    ADRFREE(dum);
    
    /* get the band index in the enum structure */
    for (band = 0; band < bands_num; band++)
        if (strcmp(channel, bands_str[band]) == 0)
            break;
    ERROR_IF_M2(band >= bands_num, "the band index (%d) is out of range (0..%d)", band, bands_num);
    
    for (seg = 1; seg <= bands_seg[band]; seg++)
    {
        snprintf(file, 100, "H-000-%s__-%s-%s___-0000%02d___-%s-C_", satellite, actual_product_id1.c_str(), channel, seg, slot);

        filename = StringConcat(directory, file); JOE;
        ret = FM_FileExistence(filename); JOE;
        if (ret == 1)
        {
            pFilenames = SM_StringsAddString(pFilenames, filename); JOE;
        }

        ADRFREE(filename);
    }

    if (pFilenames==NULL||pFilenames->num==0)
    {
        for (seg = 1; seg <= bands_seg[band]; seg++)
        {
            snprintf(file, 100, "H-000-%s__-%s-%s___-0000%02d___-%s-__", satellite, actual_product_id1.c_str(), channel, seg, slot);

            filename = StringConcat(directory, file); JOE;
            ret = FM_FileExistence(filename); JOE;
            if (ret == 1)
            {
                pFilenames = SM_StringsAddString(pFilenames, filename); JOE;
            }

            ADRFREE(filename);
        }
    
        ERROR_IF_M2(pFilenames==NULL||pFilenames->num==0,"No files for channel %s found in: %s",channel,directory);
    }

    #ifdef DEBUG2
        printf("pFilenames = ");
        SM_StringsDisplay(pFilenames);
    #endif

    /* check if we have all files */
    if(pFilenames->num < bands_seg[band])
    {
        ErrorPrintf("The channel provided is %s and the number of files (%i) is less than %d\n", 
                    channel,pFilenames->num,bands_seg[band]);
        ErrorPrintf("The segments found are:\n");
        SM_StringsDisplayErr(pFilenames);
        ERROR();
    }
    ERROR_IF_M3(pFilenames->num > bands_seg[band],"The channel provided is %s and the number of files (%i) is more than %d",channel,pFilenames->num,bands_seg[band]);
    #ifdef DEBUG2
    printf("HRV or NULL if channel is not HRV = %s\n",LU_StringExtractRegex(channel, "HRV", 0));
    #endif

    /* fill the image */

    HritFillImageAndHeaderFromSegments(pFilenames->str, pFilenames->num, sourceRectangle, 
                                       destinationImage, destinationRectangle, segmentHeaderData);JOE;

    ERROR_HANDLER_BEGIN("HritFillImageAndHeaderFromChannel");
        ADRFREE(dum);
        ADRFREE(filename);
    ERROR_HANDLER_END;
        ADRFREE(satellite);
        ADRFREE(slot);
        SM_StringsAdrFree(&pFilenames);
    ERROR_END;
}


/**
 * \brief Fill an IP_Image and segmentHeaderData from a channel in a directory. The input and output rectangle can be specified.
 * \param directory an array of characters containing the name of the directory where the slot (all the files form a particular time) is.
 * \param channel an array of characters containing the channel, these have to be: 
*        \code HRV IR_016 IR_039 IR_087 IR_097 IR_108 IR_120 IR_134 VIS006 VIS008 WV_062 WV_073 \endcode
 * \param pSourceRectangle a pointer to an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param pDestinationImage a pointer to an IP_image where we will write the image.
 * \param pDestinationRectangle a pointer to and IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \param pSegmentHeaderData a pointer to the HritSegmentHeader where we will write the corresponding to the segment header. 
 * \return void
 *
 * An overloaded version of HritFillImageAndHeaderFromChannel(char*, char*, IP_rectangle&, IP_image&, IP_rectangle&, HritSegmentHeader&)
 *
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 * 
 * \author Stijn Nevens (RMIB)
 * \date Autumn 2010
 */

void HritFillImageAndHeaderFromChannel(char *directory, char *channel, IP_rectangle *pSourceRectangle, 
                                       IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle,
                                       HritSegmentHeader *pSegmentHeaderData){
    HritFillImageAndHeaderFromChannel(directory, channel, *pSourceRectangle,
                                      *pDestinationImage, *pDestinationRectangle,
                                      *pSegmentHeaderData);
}


/**
 * \brief Fill an IP_Image and segmentHeaderData from a number of compressed or uncompressed HRIT files. The input and output rectangle can be specified. C++ only.
 * \param filenames an array of arrays of characters containing the names of the files to be opened.
 * \param numberOfFilenames the number of filenames in filenames 
 * \param sourceRectangle an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param destinationImage an IP_image where we will write the image.
 * \param destinationRectangle and IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \param segmentHeaderData the HritSegmentHeader where we will write the corresponding to the segment header. 
 * \return void
 *
 * This is basically the same as 
 * HritFillImageFromSegment(char*,IP_rectangle*,IP_image*,IP_rectangle*)
 * but with and bunch of files instead of just one.
 *
 * If the main routine using this function uses the verbose switch processing information is displayed.
 *
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 * 
 * \note There is a C interface function provided in 
 *       HritFillImageAndHeaderFromSegments(SM_strings*,IP_rectangle*,IP_image*,IP_rectangle*,HritSegmentHeader&).
 * 
 * \author Stijn Nevens (RMIB)
 * \date Summer 2010
 */

void HritFillImageAndHeaderFromSegments(char **filenames, int numberOfFilenames, IP_rectangle &sourceRectangle, 
                                        IP_image &destinationImage, IP_rectangle &destinationRectangle,
                                        HritSegmentHeader &segmentHeaderData){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering HritFillImageAndHeaderFromSegments\n";
    #endif

    /** We have no way to check if the array has the stated number of arguments, however:
     * - xRITFileDecompress checks validity of the filenames.
     * - We cross check if multiple elemensts are compatible.
     * This eliminates virtually all possible errors.
     */

    int i;
    int processedSegmentWithoutErrors = false;
    HritSegmentHeader currentSegmentHeaderData;
    std::stringstream sourceStream(std::stringstream::in | std::stringstream::out | std::stringstream::binary);

    for (i=0;i<numberOfFilenames;i++)
    {
        if(CL_InitFlag()){LU_Verbose((char *)"Processing: %s\n",filenames[i]);};

        /* decompress file */
        xRITFileDecompress(filenames[i], sourceStream);
        CCOE_M1("Not processed: %s \n",filenames[i]);CCOE;

        /* Fill the header */
        HritFillSegmentHeaderFromSegment(sourceStream,currentSegmentHeaderData);
        CCOE_M1("Not processed: %s \n",filenames[i]);

        /* Check consistency of headers */
        if(processedSegmentWithoutErrors) /* we have at least one correctly processed segment */
        {
            ERROR_IF_M2(HritSegmentHeadersAreConsistent(currentSegmentHeaderData,segmentHeaderData)!=true,
                        "Segment headers inconsistent for: \n %s \n %s", filenames[i-1], filenames[i]);
        }

        /* Fill the image */
        HritFillImageFromSegmentAndHeader(sourceStream,sourceRectangle,destinationImage,destinationRectangle,currentSegmentHeaderData);
        CCOE_M1("Not processed: %s \n",filenames[i]);

        /* Things to do if we have processed our first segment without errors */
        if(!processedSegmentWithoutErrors)
        {
            /* Set the segmentHeaderData to the current one 
               for output and to compare with the headers form the next segments */
            segmentHeaderData = currentSegmentHeaderData;
    
            /* We have processed the segment correctly */
            processedSegmentWithoutErrors = true;
        }
    };

    ERROR_IF_M(!processedSegmentWithoutErrors,
               "Unable to process at least one segment (see above for details)\n")

    ERROR_HANDLER_BEGIN("HritFillImageAndHeaderFromSegments");
    ERROR_HANDLER_END;
    ERROR_END;
}


/**
 * \brief Fill an IP_Image and segmentHeaderData from a number of compressed or uncompressed HRIT files. The input and output rectangle can be specified.
 * \param filenames an array of arrays of characters containing the names of the files to be opened.
 * \param numberOfFilenames the number of filenames in filenames.
 * \param pSourceRectangle a pointer to an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param pDestinationImage a pointer to an IP_image where we will write the image.
 * \param pDestinationRectangle a pointer to an IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \param pSegmentHeaderData a pointer to the HritSegmentHeader where we will write the corresponding to the segment header.
 * \return void
 *
 * An overloaded version of HritFillImageAndHeaderFromSegments(char**,int,IP_rectangle&,IP_image&, IP_rectangle&,HritSegmentHeader&).
 * 
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 * 
 * \author Stijn Nevens (RMIB)
 * \date Summer 2010
 */

void HritFillImageAndHeaderFromSegments(char **filenames, int numberOfFilenames, IP_rectangle *pSourceRectangle, 
                                        IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle,
                                        HritSegmentHeader *pSegmentHeaderData){
    HritFillImageAndHeaderFromSegments(filenames,numberOfFilenames,*pSourceRectangle,
                                       *pDestinationImage,*pDestinationRectangle,*pSegmentHeaderData);
};


/**
 * \brief Fill an IP_Image and segmentHeaderData from a number of compressed or uncompressed HRIT files. The input and output rectangle can be specified.
 * \param pFilenames a pointer to a SM_strings array of characters containing the name of the file to be opened.
 * \param pSourceRectangle a pointer to an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param pDestinationImage a pointer to an IP_image where we will write the image.
 * \param pDestinationRectangle a pointer to an IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \param pSegmentHeaderData a pointer to the HritSegmentHeader where we will write the corresponding to the segment header.
 * \return void
 *
 * An overloaded version of HritFillImageAndHeaderFromSegments(char**,int,IP_rectangle&,IP_image&, IP_rectangle&,HritSegmentHeader&).
 * 
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 * 
 * \author Stijn Nevens (RMIB)
 * \date Summer 2010
 */

void HritFillImageAndHeaderFromSegments(SM_strings *pFilenames, IP_rectangle *pSourceRectangle, 
                                        IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle,
                                        HritSegmentHeader *pSegmentHeaderData){
    HritFillImageAndHeaderFromSegments(pFilenames->str,pFilenames->num,*pSourceRectangle,
                                       *pDestinationImage,*pDestinationRectangle,*pSegmentHeaderData);
};


/**
 * \brief Fill an IP_Image from a number of compressed or uncompressed HRIT files. The input and output rectangle can be specified.  C++ only.
 * \param filenames an array of arrays of characters containing the names of the files to be opened.
 * \param numberOfFilenames the number of filenames in filenames 
 * \param sourceRectangle an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param destinationImage an IP_image where we will write the image.
 * \param destinationRectangle and IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \return void
 *
 * This is basically the same as 
 * HritFillImageFromSegment(char*,IP_rectangle*,IP_image*,IP_rectangle*)
 * but with and bunch of files instead of just one.
 *
 * If the main routine using this function uses the verbose switch processing information is displayed.
 *
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 * 
 * \note There is a C interface function provided in 
 *       HritFillImageFromSegments(SM_strings*,IP_rectangle*,IP_image*, IP_rectangle*).
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void HritFillImageFromSegments(char **filenames, int numberOfFilenames, IP_rectangle &sourceRectangle, 
                               IP_image &destinationImage, IP_rectangle &destinationRectangle){
    HritSegmentHeader segmentHeaderData;
    HritFillImageAndHeaderFromSegments(filenames, numberOfFilenames, sourceRectangle, 
                                       destinationImage, destinationRectangle,
                                       segmentHeaderData);
}


/**
 * \brief Fill an IP_Image from a number of compressed or uncompressed HRIT files. The input and output rectangle can be specified.
 * \param filenames an array of arrays of characters containing the names of the files to be opened.
 * \param numberOfFilenames the number of filenames in filenames.
 * \param pSourceRectangle a pointer to an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param pDestinationImage a pointer to an IP_image where we will write the image.
 * \param pDestinationRectangle a pointer to an IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \return void
 *
 * An overloaded version of HritFillImageFromSegments(char**,int,IP_rectangle&,IP_image&, IP_rectangle&).
 * 
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void HritFillImageFromSegments(char **filenames, int numberOfFilenames, IP_rectangle *pSourceRectangle, 
                               IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle){
    HritSegmentHeader segmentHeaderData;
    HritFillImageAndHeaderFromSegments(filenames,numberOfFilenames,*pSourceRectangle,
                                       *pDestinationImage,*pDestinationRectangle,
                                       segmentHeaderData);
};


/**
 * \brief Fill an IP_Image from a number of compressed or uncompressed HRIT files. The input and output rectangle can be specified.
 * \param pFilenames a pointer to a SM_strings array of characters containing the name of the file to be opened.
 * \param pSourceRectangle a pointer to an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param pDestinationImage a pointer to an IP_image where we will write the image.
 * \param pDestinationRectangle a pointer to an IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \return void
 *
 * An overloaded version of HritFillImageFromSegments(char**,int,IP_rectangle&,IP_image&, IP_rectangle&).
 * 
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void HritFillImageFromSegments(SM_strings *pFilenames, IP_rectangle *pSourceRectangle, 
                               IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle){
    HritSegmentHeader segmentHeaderData;
    HritFillImageAndHeaderFromSegments(pFilenames->str,pFilenames->num,*pSourceRectangle,
                                       *pDestinationImage,*pDestinationRectangle,
                                       segmentHeaderData);
};


/**
 * \brief Fill an IP_Image from a compressed or uncompressed HRIT file. The input and output rectangle can be specified. C++ only.
 * \param filename an array of characters containing the name of the file to be opened.
 * \param sourceRectangle an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param destinationImage an IP_image where we will write the image.
 * \param destinationRectangle and IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \return void
 *
 * This is basically the same as 
 * HritFillImageAndHeaderFromSegment(char*,IP_rectangle&,IP_image&,IP_rectangle&,HritSegmentHeader&)
 * but without the need to provide an HritSegmentHeader.
 *
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 *
 * \note There is a C interface function provided in 
 *       HritFillImageFromSegment(char*,IP_rectangle*,IP_image*, IP_rectangle*).
 *
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void HritFillImageFromSegment(char *filename, IP_rectangle &sourceRectangle, 
                              IP_image &destinationImage, IP_rectangle &destinationRectangle){
    HritSegmentHeader segmentHeaderData;
    HritFillImageAndHeaderFromSegment(filename,sourceRectangle,destinationImage,destinationRectangle,segmentHeaderData);
};


/**
 * \brief Fill an IP_Image from a compressed or uncompressed HRIT file. The input and output rectangle can be specified.
 * \param filename an array of characters containing the name of the file to be opened.
 * \param pSourceRectangle an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param pDestinationImage an IP_image where we will write the image.
 * \param pDestinationRectangle and IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \return void
 *
 * This is basically the same as 
 * HritFillImageAndHeaderFromSegment(char*,IP_rectangle&,IP_image&,IP_rectangle&,HritSegmentHeader&)
 * but without the need to provide an HritSegmentHeader.
 *
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 *
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void HritFillImageFromSegment(char *filename, IP_rectangle *pSourceRectangle, 
                              IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle){
    HritSegmentHeader segmentHeaderData;
    HritFillImageAndHeaderFromSegment(filename,*pSourceRectangle,*pDestinationImage,*pDestinationRectangle,segmentHeaderData);
};


/**
 * \brief Fill an IP_Image and segmentHeaderData from a file containing the content of a decompressed HRIT file. The input and output rectangle can be specified. C++ only.
 * \param filename an array of characters containing the name of the file to be opened.
 * \param sourceRectangle an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param destinationImage an IP_image where we will write the image.
 * \param destinationRectangle and IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \param segmentHeaderData the HritSegmentHeader where we will write the corresponding to the segment header. 
 * \return void
 *
 * \note There is a C interface function provided in 
 *       HritFillImageAndHeaderFromSegment(char*,IP_rectangle*,IP_image*, IP_rectangle*,HritSegmentHeader*).
 * 
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 *
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void HritFillImageAndHeaderFromSegment(char *filename, IP_rectangle &sourceRectangle, 
                                       IP_image &destinationImage, IP_rectangle &destinationRectangle,
                                       HritSegmentHeader &segmentHeaderData){
    std::stringstream sourceStream(std::stringstream::in | std::stringstream::out | std::stringstream::binary);

    /* decompress file */
    xRITFileDecompress(filename, sourceStream);JOE;

    /* Fill the header */
    HritFillSegmentHeaderFromSegment(sourceStream,segmentHeaderData);JOE;

    /* Fill the image */
    HritFillImageFromSegmentAndHeader(sourceStream,sourceRectangle,destinationImage,destinationRectangle,segmentHeaderData);JOE;

    ERROR_HANDLER_BEGIN("HritFillImageAndHeaderFromSegment");
    ERROR_HANDLER_END;
    ERROR_END;
};


/**
 * \brief Fill an IP_Image and segmentHeaderData from a file containing the content of a decompressed HRIT file. The input and output rectangle can be specified.
 * \param filename an array of characters containing the name of the file to be opened.
 * \param pSourceRectangle pointer to an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param pDestinationImage pointer to an IP_image where we will write the image.
 * \param pDestinationRectangle pointer to an IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \param pSegmentHeaderData  pointer to the HritSegmentHeader where we will write the corresponding to the segment header. 
 * \return void
 *
 * An overloaded version of HritFillImageAndHeaderFromSegment(char*,IP_rectangle&,IP_image&,IP_rectangle&,HritSegmentHeader&).
 * 
 * \sa HritFillImageFromSegmentAndHeader(std::stringstream&,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * for more information on how the rectangle specifications work.
 *
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void HritFillImageAndHeaderFromSegment(char *filename, IP_rectangle *pSourceRectangle,
                                       IP_image *pDestinationImage, IP_rectangle *pDestinationRectangle,
                                       HritSegmentHeader *pSegmentHeaderData){
    HritFillImageAndHeaderFromSegment(filename,*pSourceRectangle,*pDestinationImage,*pDestinationRectangle,*pSegmentHeaderData);
};


/**
 * \brief Fill an IP_Image from a stream containing the content of a decompressed HRIT file and segmentHeaderData. The input and output rectangle can be specified. C++ only.
 * \param sourceStream is a stringstream passed by reference containing the decompressed content of an HRIT file.
 *        Note that the ios_base::openmode should be of type: 
 *        \code (std::stringstream::in | std::stringstream::out | std::stringstream::binary) \endcode
 *        We should test for this but I did not yet find how.
 * \param sourceRectangle an IP_rectangle that say which part of the source image we need to get 
 *        The coordinates are (like all coordiantes) given in a grid adapted to the whole channel.
 * \param destinationImage an IP_image where we will write the image.
 * \param destinationRectangle and IP_rectangle where we should write the in the image file, 
 *        like this we can allocate one image for multiple segments and write to it all relevant segments.
 * \param segmentHeaderData the HritSegmentHeader corresponding to the segment. 
 *        By allowing this here we allow maximal flexibility. HOWEVER: if you use this, it is presumed that they correspond.
 * \return void
 *
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void HritFillImageFromSegmentAndHeader(std::stringstream &sourceStream, IP_rectangle &sourceRectangle, 
                                       IP_image &destinationImage, IP_rectangle &destinationRectangle,
                                       const HritSegmentHeader &segmentHeaderData ){
    int4 k, i, p;
    int4 byteSexFlag;
    int4 lineSize;
    int4 yCoordinateSource, xCoordinateSourceLastOfFour;
    IP_rectangle segmentRectangle; /* the rectangle where this segment is -> in the full rectangle */
    IP_rectangle channelRectangle; /* the full rectange for the channel where this segment is in */
    IP_rectangle destinationImageRectangle; /* the rectangle that fits the size of the destinationImage */
    u_c lineBufferForFiveBytes[5] = {0,0,0,0,0};
    u_int2 fourPixels[4];

    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering HritFillImageFromSegmentAndHeader\n";
    #endif

    /* set the bytesex flag, the HRIT files have big endian */
    byteSexFlag = 0;
    if(ByteSex()==LU_LITTLE_ENDIAN) byteSexFlag=1;

    /* Verify the source stream */    
    ERROR_IF_M(!sourceStream.good(), "The stream containing the content of a decompressed HRIT file is corrupted");

    /* Verify the destination image */
    ERROR_IF_M(&destinationImage==NULL,"Destination image has NULL pointer.");
    ERROR_IF_M(!(destinationImage.type==INT2 || destinationImage.type==U_INT2),
               "Destination image is not a 16 bit integer (int2, int32_t, u_int2 or uint32_t).");

    /* Set size info from the segmentHeaderData */

    if (segmentHeaderData.segmentIdentificationRecord.spectralChannelID == 12)
    {
        segmentRectangle.ul.x=0;
        segmentRectangle.ul.y= (24-segmentHeaderData.segmentIdentificationRecord.segmSeqNumber)*NUM_LINE_SEGMENT;
        segmentRectangle.lr.x=5568;
        segmentRectangle.lr.y= (25-segmentHeaderData.segmentIdentificationRecord.segmSeqNumber)*NUM_LINE_SEGMENT;
    
        channelRectangle.ul.x=0;
        channelRectangle.ul.y=0;
        channelRectangle.lr.x=5568;
        channelRectangle.lr.y=11136;

        lineSize = segmentHeaderData.imageStructureRecord.numberBitsPerPixel*(5568/8); 
        /* number of bits in a line = number of pixels in a line * number of bits per pixel 
         * number of bytes in a line = number of bits in a line / 8
         * lineSize = number of bytes in a line 
         *
         * this mean that for every 5 bytes = 40 bits we have information on 4 pixels
         */
    }
    else
    {
        segmentRectangle.ul.x=0;
        segmentRectangle.ul.y= (8-segmentHeaderData.segmentIdentificationRecord.segmSeqNumber)*NUM_LINE_SEGMENT;
        segmentRectangle.lr.x=3712;
        segmentRectangle.lr.y= (9-segmentHeaderData.segmentIdentificationRecord.segmSeqNumber)*NUM_LINE_SEGMENT;
    
        channelRectangle.ul.x=0;
        channelRectangle.ul.y=0;
        channelRectangle.lr.x=3712;
        channelRectangle.lr.y=3712; 

        lineSize = segmentHeaderData.imageStructureRecord.numberBitsPerPixel*(3712/8); /* see above */
    };

    destinationImageRectangle.ul.x=0;
    destinationImageRectangle.ul.y=0;
    destinationImageRectangle.lr.x=destinationImage.width;
    destinationImageRectangle.lr.y=destinationImage.height;
    
    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Size info for segmentRectangle, channelRectangle and destinationImageRectangle:\n";
    IP_RectangleDisplay(&segmentRectangle);
    IP_RectangleDisplay(&channelRectangle);
    IP_RectangleDisplay(&destinationImageRectangle);
    #endif

    /** 
     * The behaviour derived from the input sourceRectangle and destinationRectangle is as follows:
     * We repeat here that the coordinates are given in a grid adapted to the whole channel.
     * - Both are supposed to be allocated already, 
     *   If any of these has a NULL pointer the function returns without writing anything. 
     */

    ERROR_IF_V(&sourceRectangle==NULL);
    ERROR_IF_V(&destinationRectangle==NULL);

    /** 
     * - Instead of feeding in IP_rectangle with NULL pointer you can feed default rectangles.
     * That is all values are -1. For a rectangle rec this means:
     * \code rec.ul.x=-1; rec.ul.y=-1; rec.lr.x=-1; rec.lr.y=-1; \endcode
     * if(sourceRectangle==default) it will be set to the size of the of the channel that is fed. \n
     * This means that the destinationImage AND (if provided) destinationRectangle needs to bigger 
     * or equal the size of the channelRectangle \n
     * if(destinationRectangle==default) it will be set to be adapted to the size of the destinationImage.
     * This means that the destinationImage needs to bigger or equal the size of the channelRectangle \n
     * All the above means that if you provide default rectangles it is assumed that the upper left corner of the 
     * source is written to the upper left corner of the destination. Furthermore it is assumed that all segments are
     * going to be processed. IF you need to process a few segements: PROVIDE rectangles.
     */

    if (IP_IsDefaultRectangle(sourceRectangle)) sourceRectangle=channelRectangle;
    if (IP_IsDefaultRectangle(destinationRectangle)) destinationRectangle=destinationImageRectangle;

    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Size info for sourceRectangle and destinationRectangle:\n";
    IP_RectangleDisplay(&sourceRectangle);
    IP_RectangleDisplay(&destinationRectangle);
    #endif

    /** 
     * - We verify the sourceRectangle, it is supposed to be inside the channelRectangle.
     * For the rest you are free and all other situations are handled gracefully. 
     */

    ERROR_IF_M(not(IP_RectangleInclude(sourceRectangle, channelRectangle)), 
               "The provided sourceRectangle is not inside the channelRectangle.");

    #ifdef DEBUG2
    if(not(IP_RectangleOverlap(segmentRectangle,sourceRectangle))) 
    {
        std::cout << "---------------------------------------------------------------\n";
        std::cout <<"There is no overlap between sourceRectangle and segmentRectangle for segment "
                  << segmentHeaderData.segmentIdentificationRecord.segmSeqNumber << " so we end here.\n";
        IP_RectangleDisplay(&sourceRectangle);
        IP_RectangleDisplay(&segmentRectangle);
        std::cout << "***************************************************************\n";
    }
    #endif

    TERMINATE_IF(not(IP_RectangleOverlap(sourceRectangle,segmentRectangle))); /* If there is no overlap we need not write anything */

    #ifdef DEBUG2
        std::cout << "---------------------------------------------------------------\n";
        std::cout <<"There is an overlap between sourceRectangle and segmentRectangle for segment "
                  << segmentHeaderData.segmentIdentificationRecord.segmSeqNumber << " so we proceed.\n";
        IP_RectangleDisplay(&sourceRectangle);
        IP_RectangleDisplay(&segmentRectangle);
    #endif

    /**
     * - We verify the destinationRectangle, it is supposed to be inside the destinationImage.
     */

    ERROR_IF_M(not(IP_RectangleInclude(destinationRectangle, destinationImageRectangle)), 
               "The provided destinationImageRectangle is not inside the destinationImage.");

    /**
     * - We verify if the destinationRectangle is big enough to hold the sourceRectangle. 
     * If this is not the case we de nothing and print error because at that point there 
     * is no way to know what the user would want in this case
     * (well there might be cases where we would but I feel no need to implement these).
     */

    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Size info for sourceRectangle and destinationRectangle:\n";
    IP_RectangleDisplay(&sourceRectangle);
    IP_RectangleDisplay(&destinationRectangle);
    std::cout << "***************************************************************\n";
    #endif

    ERROR_IF_M(IP_RectangleHasBiggerSide(sourceRectangle, destinationRectangle), 
               "The provided destinationRectangle is not big enough to hold the sourceRectangle.");

    /* start reading and writing of image */
    sourceStream.seekg(HRIT_SEGMENT_HEADER_LENGTH, std::ios::beg); /* move pointer to beginning of image data */

    if (destinationImage.type==INT2) /* As we allow the destination image to be 2 types: 
                                      * 16 bit signed and unsigned integer we need to cast differently
                                      * We do it outside the loop to avoid too mucht testing inside the loop
                                      * the only difference between the consequent and the alternative 
                                      * is the casting of the data type: (int2 **) versus (u_int2 **) 
                                      */
    {
        for(k=0;k<NUM_LINE_SEGMENT;k++) /* loop over all lines */
        {
            /* The segments are upside down so the first pixel is the one at the 
            * lower-right (lr) corner. Hence the minus sign 
            * lineNumberInGrid starts at 1 hence no need for a extra minus sign. */
            yCoordinateSource = channelRectangle.lr.y - segmentHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineNumberInGrid; /* pixel's y coordinate */
    
            for(i=0;i<channelRectangle.lr.x;i=i+4) /* loop over all pixels */
            {
                ERROR_IF_M(sourceStream.readsome((char*)lineBufferForFiveBytes, 5)!=5,"datastream read error");
    
                /* The segments are upside down so the first pixel is the one at the 
                * lower-right (lr) corner. The colum  starts at 0 */
                xCoordinateSourceLastOfFour = channelRectangle.lr.x-1-i; /* last pixel's x coordinate*/

                Unpack4PixelsFrom40Bits(lineBufferForFiveBytes, fourPixels); 

                /* as we already verified the validity of the rectangles we can check whether a pixel is in the 
                * source rectangle and if it is we write it to the right offset in the destination 
                * the pixels are stored like this p[3]p[2]p[1]p[0] -> not correct yet */
                for(p=0;p<4;p++)
                {
                    if(IP_RectangleSurroundCoordinate(sourceRectangle,xCoordinateSourceLastOfFour-p,yCoordinateSource))
                    {
                        /* in this case we cast to an int2 data pointer */
                        ((int2 **)destinationImage.data)
                        [destinationRectangle.ul.y-sourceRectangle.ul.y+yCoordinateSource]
                        [destinationRectangle.ul.x-sourceRectangle.ul.x+xCoordinateSourceLastOfFour-p] 
                        = fourPixels[p];
                    };
                };
            };
        };
    }
    else if (destinationImage.type==U_INT2)
    {
        for(k=0;k<NUM_LINE_SEGMENT;k++)
        {
            /* The segments are upside down so the first pixel is the one at the 
            * lower-right (lr) corner. Hence the minus sign 
            * lineNumberInGrid starts at 1 hence no need for a extra minus sign. */
            yCoordinateSource = channelRectangle.lr.y - segmentHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineNumberInGrid; /* pixel's y coordinate */
    
            for(i=0;i<channelRectangle.lr.x;i=i+4) /* loop over all pixels */
            {
                ERROR_IF_M(sourceStream.readsome((char*)lineBufferForFiveBytes, 5)!=5,"datastream read error");
    
                /* The segments are upside down so the first pixel is the one at the 
                * lower-right (lr) corner. The colum  starts at 0 */
                xCoordinateSourceLastOfFour = channelRectangle.lr.x-1-i; /* last pixel's x coordinate*/

                Unpack4PixelsFrom40Bits(lineBufferForFiveBytes, fourPixels); 

                /* as we already verified the validity of the rectangles we can check whether a pixel is in the 
                * source rectangle and if it is we write it to the right offset in the destination 
                * the pixels are stored like this p[3]p[2]p[1]p[0] -> not correct yet */
                for(p=0;p<4;p++)
                {
                    if(IP_RectangleSurroundCoordinate(sourceRectangle,xCoordinateSourceLastOfFour-p,yCoordinateSource))
                    {
                        /* in this case we cast to an u_int2 data pointer */
                        ((u_int2 **)destinationImage.data)
                        [destinationRectangle.ul.y-sourceRectangle.ul.y+yCoordinateSource]
                        [destinationRectangle.ul.x-sourceRectangle.ul.x+xCoordinateSourceLastOfFour-p] 
                        = fourPixels[p];
                    };
                };
            };
        };
    };

    ERROR_HANDLER_BEGIN("HritFillImageFromSegmentAndHeader");
    ERROR_HANDLER_END;
    ERROR_END;
};


/**
 * \brief Fill an HritSegmentHeader from a stream containing the content of a decompressed HRIT file. C++ only.
 * \param sourceStream is a stringstream passed by reference containing the decompressed content of an HRIT segment file.
 *        Note that the ios_base::openmode should be of type: 
 *        \code (std::stringstream::in | std::stringstream::out | std::stringstream::binary) \endcode
 *        We should test for this but I did not yet find how.
 * \param destinationHeaderData the HritSegmentHeader that we will fill.
 * \return void
 *
 * This function also performs a bunch of sanity checks on the image segement being read.
 *
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void HritFillSegmentHeaderFromSegment(std::stringstream &sourceStream, HritSegmentHeader &destinationHeaderData){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering HritFillSegmentHeaderFromSegment\n";
    u_int1 test = 0;
    int debugInt = test;
    bool debugBool = 0;
    #endif

    /* set the bytesex flag, the HRIT files have big endian */
    int4 byteSexFlag = 0;
    if(ByteSex()==LU_LITTLE_ENDIAN) byteSexFlag=1;

    /* Verify the HritSegmentHeader */
   ERROR_IF_V(&destinationHeaderData==NULL);

    /* Verify the source stream */
    ERROR_IF_M(!sourceStream.good(), "The stream containing the content of a decompressed HRIT file is corrupted");

    /* set the position of the get pointer to the beginning of the stream */
    sourceStream.seekg(0, std::ios::beg); 

    /* fill primaryHeaderRecord */
    sourceStream.read((char *)(&destinationHeaderData.primaryHeaderRecord.headerType), 1);  
    ERROR_IF_V(destinationHeaderData.primaryHeaderRecord.headerType!=0);

    sourceStream.read((char *)(&destinationHeaderData.primaryHeaderRecord.headerRecordLength),2);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.primaryHeaderRecord.headerRecordLength, U_INT2); JOE;
    ERROR_IF_V(destinationHeaderData.primaryHeaderRecord.headerRecordLength!=16);

    sourceStream.read((char *)(&destinationHeaderData.primaryHeaderRecord.fillTypeCode), 1); 

    sourceStream.read((char *)(&destinationHeaderData.primaryHeaderRecord.totalHeaderLength), 4);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.primaryHeaderRecord.totalHeaderLength, U_INT4); JOE;
    ERROR_IF_V(destinationHeaderData.primaryHeaderRecord.totalHeaderLength!=HRIT_SEGMENT_HEADER_LENGTH );

    sourceStream.read((char *)(&destinationHeaderData.primaryHeaderRecord.headerRecordLength), 8);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.primaryHeaderRecord.headerRecordLength, U_INT_8); JOE;

    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Contents of primaryHeaderRecord:\n";
    debugInt = destinationHeaderData.primaryHeaderRecord.headerType;
    std::cout << "header type (0)                     = " << debugInt << "\n";
    std::cout << "header record length (16)           = " << destinationHeaderData.primaryHeaderRecord.headerRecordLength << "\n";
    debugInt = destinationHeaderData. primaryHeaderRecord.fillTypeCode;
    std::cout << "fill type code                      = " << debugInt << "\n";
    std::cout << "total header length                 = " << destinationHeaderData.primaryHeaderRecord.totalHeaderLength << "\n";
    std::cout << "header record lenght                = " << destinationHeaderData.primaryHeaderRecord.headerRecordLength << "\n";
    std::cout << "end position                        = " << sourceStream.tellg() << "\n";
    #endif

    /* fill imageStructureRecord */
    sourceStream.read((char *)(&destinationHeaderData.imageStructureRecord.headerType), 1);  
    ERROR_IF_V(destinationHeaderData.imageStructureRecord.headerType!=1);

    sourceStream.read((char *)(&destinationHeaderData.imageStructureRecord.headerRecordLength),2);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.imageStructureRecord.headerRecordLength, U_INT2); JOE;
    ERROR_IF_V(destinationHeaderData.imageStructureRecord.headerRecordLength!=9);

    sourceStream.read((char *)(&destinationHeaderData.imageStructureRecord.numberBitsPerPixel), 1); 
    ERROR_IF_V(destinationHeaderData.imageStructureRecord.numberBitsPerPixel!=10);

    sourceStream.read((char *)(&destinationHeaderData.imageStructureRecord.numberColumns), 2);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.imageStructureRecord.numberColumns, U_INT2); JOE;
    ERROR_IF_V(destinationHeaderData.imageStructureRecord.numberColumns!=3712 &&
               destinationHeaderData.imageStructureRecord.numberColumns!=5568);

    sourceStream.read((char *)(&destinationHeaderData.imageStructureRecord.numberLines), 2);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.imageStructureRecord.numberLines, U_INT2); JOE;
    ERROR_IF_V(destinationHeaderData.imageStructureRecord.numberLines!=464);

    sourceStream.read((char *)(&destinationHeaderData.imageStructureRecord.compressionFlag), 1);  
    ERROR_IF_M(destinationHeaderData.imageStructureRecord.compressionFlag==2,"Segment has lossy compression.");

    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Contents of imageStructureRecord:\n";
    debugInt = destinationHeaderData.imageStructureRecord.headerType;
    std::cout << "header type (1)                     = " << debugInt << "\n";
    std::cout << "header record length (9)            = " << destinationHeaderData.imageStructureRecord.headerRecordLength << "\n";
    debugInt = destinationHeaderData. imageStructureRecord.numberBitsPerPixel;
    std::cout << "number bits/pixel (10)              = " << debugInt << "\n";
    std::cout << "numberColumns(3712||5568)           = " << destinationHeaderData.imageStructureRecord.numberColumns << "\n";
    std::cout << "numberLines (464)                   = " << destinationHeaderData.imageStructureRecord.numberLines << "\n";
    debugInt = destinationHeaderData.imageStructureRecord.compressionFlag;
    std::cout << "compressionFlag                     = " << debugInt << "\n";
    std::cout << "end position                        = " << sourceStream.tellg() << "\n";
    #endif

    /* fill imageNavigationRecord */

    sourceStream.read((char *)(&destinationHeaderData.imageNavigationRecord.headerType), 1);  
    ERROR_IF_V(destinationHeaderData.imageNavigationRecord.headerType!=2);

    sourceStream.read((char *)(&destinationHeaderData.imageNavigationRecord.headerRecordLength),2);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.imageNavigationRecord.headerRecordLength, U_INT2); JOE;
    ERROR_IF_V(destinationHeaderData.imageNavigationRecord.headerRecordLength!=51);

    sourceStream.read(destinationHeaderData.imageNavigationRecord.projectionName, 32);
    destinationHeaderData.imageNavigationRecord.projectionName[32] = '\0'; /* end of string */

    sourceStream.read((char *)(&destinationHeaderData.imageNavigationRecord.columnScalingFactor), 4);
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.imageNavigationRecord.columnScalingFactor, INT4); JOE;

    sourceStream.read((char *)(&destinationHeaderData.imageNavigationRecord.lineScalingFactor), 4);
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.imageNavigationRecord.lineScalingFactor, INT4); JOE;

    sourceStream.read((char *)(&destinationHeaderData.imageNavigationRecord.columnOffset), 4);
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.imageNavigationRecord.columnOffset, INT4); JOE;

    sourceStream.read((char *)(&destinationHeaderData.imageNavigationRecord.lineOffset), 4);
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.imageNavigationRecord.lineOffset, INT4); JOE;

    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Contents of imageNavigationRecord:\n";
    debugInt = destinationHeaderData.imageNavigationRecord.headerType;
    std::cout << "header type (2)                     = " << debugInt << "\n";
    std::cout << "header record length (51)           = " << destinationHeaderData.imageNavigationRecord.headerRecordLength << "\n";
    std::cout << "projectionName                      = " << destinationHeaderData.imageNavigationRecord.projectionName << "\n";
    std::cout << "columnScalingFactor                 = " << destinationHeaderData.imageNavigationRecord.columnScalingFactor << "\n";
    std::cout << "lineScalingFactor                   = " << destinationHeaderData.imageNavigationRecord.lineScalingFactor << "\n";
    std::cout << "columnOffset                        = " << destinationHeaderData.imageNavigationRecord.columnOffset << "\n";
    std::cout << "lineOffset                          = " << destinationHeaderData.imageNavigationRecord.lineOffset << "\n";
    std::cout << "end position                        = " << sourceStream.tellg() << "\n";
    #endif

    /* fill annotationRecord */

    sourceStream.read((char *)(&destinationHeaderData.annotationRecord.headerType), 1);  
    ERROR_IF_V(destinationHeaderData.annotationRecord.headerType!=4);

    sourceStream.read((char *)(&destinationHeaderData.annotationRecord.headerRecordLength),2);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.annotationRecord.headerRecordLength, U_INT2); JOE;
    ERROR_IF_V( destinationHeaderData.annotationRecord.headerRecordLength != 64);

    sourceStream.read(destinationHeaderData.annotationRecord.annotationText,    
                      destinationHeaderData.annotationRecord.headerRecordLength-3);
    destinationHeaderData.annotationRecord.annotationText[destinationHeaderData.annotationRecord.headerRecordLength-3] = '\0';

    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Contents of annotationRecord:\n";
    debugInt = destinationHeaderData.annotationRecord.headerType;
    std::cout << "header type (4)                     = " << debugInt << "\n";
    std::cout << "header record length (64)           = " << destinationHeaderData.annotationRecord.headerRecordLength << "\n";
    std::cout << "annotationText                      = " << destinationHeaderData.annotationRecord.annotationText << "\n";
    std::cout << "end position (140)                  = " << sourceStream.tellg() << "\n";
    #endif

    /* fill timeStampRecord */

    sourceStream.read((char *)(&destinationHeaderData.timeStampRecord.headerType), 1);  
    ERROR_IF_V(destinationHeaderData.timeStampRecord.headerType!=5);

    sourceStream.read((char *)(&destinationHeaderData.timeStampRecord.headerRecordLength),2);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.timeStampRecord.headerRecordLength, U_INT2); JOE;
    ERROR_IF_V( destinationHeaderData.timeStampRecord.headerRecordLength != 10);

    sourceStream.read((char *)(&destinationHeaderData.timeStampRecord.timeStamp.pField), 1);

    sourceStream.read((char *)(&destinationHeaderData.timeStampRecord.timeStamp.days),2);
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.timeStampRecord.timeStamp.days, U_INT2); JOE;
    sourceStream.read((char *)(&destinationHeaderData.timeStampRecord.timeStamp.milliSeconds),4);
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.timeStampRecord.timeStamp.milliSeconds, U_INT4); JOE;
    /** 
     * There is something weird going on here with the "destinationHeaderData.timeStampRecord.timeStamp"
     *, the days record is always zero and the milliseconds is jiberish, DO NOT USE THEM 
     */

    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Contents of timeStampRecord:\n";
    debugInt = destinationHeaderData.timeStampRecord.headerType;
    std::cout << "header type (5)                     = " << debugInt << "\n";
    std::cout << "header record length (10)           = " << destinationHeaderData.timeStampRecord.headerRecordLength << "\n"; 
    std::cout << "P-field (01000000)                  = ";
    for(int ll=7;ll>=0;ll--) 
    {
        debugBool = (destinationHeaderData.timeStampRecord.timeStamp.pField>>ll)&1;    // stores nth bit of x in y.  y becomes 0 or 1.
        std::cout << debugBool ;
    };
    std::cout << "\n";
    std::cout << "days                                = " << destinationHeaderData.timeStampRecord.timeStamp.days << "\n";
    std::cout << "milliseconds                        = " << destinationHeaderData.timeStampRecord.timeStamp.milliSeconds << "\n";
    std::cout << "end position                        = " << sourceStream.tellg() << "\n";
    #endif

    /* fill segmentIdentificationRecord */

    sourceStream.read((char *)(&destinationHeaderData.segmentIdentificationRecord.headerType), 1);  
    ERROR_IF_V(destinationHeaderData.segmentIdentificationRecord.headerType!=128);

    sourceStream.read((char *)(&destinationHeaderData.segmentIdentificationRecord.headerRecordLength),2);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.segmentIdentificationRecord.headerRecordLength, U_INT2); JOE;
    ERROR_IF_V( destinationHeaderData.segmentIdentificationRecord.headerRecordLength != 13);

    sourceStream.read((char *)(&destinationHeaderData.segmentIdentificationRecord.gpScId),2);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.segmentIdentificationRecord.gpScId, U_INT2); JOE;

    sourceStream.read((char *)(&destinationHeaderData.segmentIdentificationRecord.spectralChannelID), 1); 
 
    sourceStream.read((char *)(&destinationHeaderData.segmentIdentificationRecord.segmSeqNumber),2);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.segmentIdentificationRecord.segmSeqNumber, U_INT2); JOE;

    sourceStream.read((char *)(&destinationHeaderData.segmentIdentificationRecord.plannedStartSegmSeqNumber),2);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.segmentIdentificationRecord.plannedStartSegmSeqNumber, U_INT2); JOE;

    sourceStream.read((char *)(&destinationHeaderData.segmentIdentificationRecord.plannedEndSegmSeqNumber),2);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.segmentIdentificationRecord.plannedEndSegmSeqNumber, U_INT2); JOE;

    sourceStream.read((char *)(&destinationHeaderData.segmentIdentificationRecord.dataFieldRepresentation),1);  
    
    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Contents of segmentIdentificationRecord:\n";
    debugInt = destinationHeaderData.segmentIdentificationRecord.headerType;
    std::cout << "header type (128)                   = " << debugInt << "\n";
    std::cout << "header record length (13)           = " << destinationHeaderData.segmentIdentificationRecord.headerRecordLength << "\n";
    debugInt = destinationHeaderData.segmentIdentificationRecord.spectralChannelID;
    std::cout << "spectral channel                    = " << debugInt << "\n";
    std::cout << "segmSeqNumber                       = " << destinationHeaderData.segmentIdentificationRecord.segmSeqNumber << "\n";
    std::cout << "plannedStartSegmSeqNumber           = " << destinationHeaderData.segmentIdentificationRecord.plannedStartSegmSeqNumber << "\n";
    std::cout << "plannedEndSegmSeqNumber             = "<< destinationHeaderData.segmentIdentificationRecord.plannedEndSegmSeqNumber << "\n";
    debugInt = destinationHeaderData.segmentIdentificationRecord.dataFieldRepresentation;
    std::cout << "dataFieldRepresentation             = " << debugInt << "\n";
    std::cout << "end position                        = " << sourceStream.tellg() << "\n";
    #endif

    /* fill segmentLineQualityRecord */

    sourceStream.read((char *)(&destinationHeaderData.segmentLineQualityRecord.headerType), 1);  
    ERROR_IF_V(destinationHeaderData.segmentLineQualityRecord.headerType!=129);

    sourceStream.read((char *)(&destinationHeaderData.segmentLineQualityRecord.headerRecordLength),2);  
    if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.segmentLineQualityRecord.headerRecordLength, U_INT2); JOE;
    ERROR_IF_V( destinationHeaderData.segmentLineQualityRecord.headerRecordLength != 3 + 13*NUM_LINE_SEGMENT); 
    /* 3 bytes for the header, 13 for each line segment */

    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Contents of segmentLineQualityRecord:\n";
    debugInt = destinationHeaderData.segmentLineQualityRecord.headerType;
    std::cout << "header type                         = " << debugInt << "\n";
    std::cout << "header record length                = " << destinationHeaderData.segmentLineQualityRecord.headerRecordLength << "\n";
    std::cout << "---------------------------------------------------------------\n";
    #endif

    for(int k=0;k<NUM_LINE_SEGMENT;k++) 
    {

        sourceStream.read((char *)(&destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineNumberInGrid),4);
        if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineNumberInGrid, INT4); JOE;

        sourceStream.read((char *)(&destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineMeanAcquisition.days),2);
        if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineMeanAcquisition.days, U_INT2); JOE;
        sourceStream.read((char *)(&destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineMeanAcquisition.milliSeconds),4);
        if(byteSexFlag) MP_DataByteSwap(&destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineMeanAcquisition.milliSeconds, U_INT4); JOE;

        sourceStream.read((char *)(&destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineValidity),1);

        sourceStream.read((char *)(&destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineRadiometricQuality),1);

        sourceStream.read((char *)(&destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineGeometricQuality),1);

        #ifdef DEBUG3
        std::cout << "Contents of hritLineQualityEntry[k] for line number k = " << k << "\n";
        std::cout << "lineNumberInGrid                    = "
                  << destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineNumberInGrid << "\n";
        std::cout << "lineMeanAcquisition.days            = " 
                  << destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineMeanAcquisition.days << "\n";
        std::cout << "lineMeanAcquisition.msec            = " 
                  << destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineMeanAcquisition.milliSeconds << "\n";
        std::cout << "MeanAcquisition Unix time as double = " 
                  << LU_CdsTimeToUnix(destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineMeanAcquisition.days,
                                      destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineMeanAcquisition.milliSeconds)
                  << "\n";
        std::cout << "MeanAcquisition human readable time = ";
        LU_DisplayTime(LU_CdsTimeToUnix(destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineMeanAcquisition.days,
                                        destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineMeanAcquisition.milliSeconds));
        debugInt = destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineValidity;
        std::cout << "line validity                       = " << debugInt << "\n";
        debugInt = destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineRadiometricQuality;
        std::cout << "lineRadiometricQuality              = " << debugInt << "\n";
        debugInt = destinationHeaderData.segmentLineQualityRecord.hritLineQualityEntry[k].lineGeometricQuality;
        std::cout << "lineGeometricQuality                = " << debugInt << "\n";
        std::cout << "end position                        = " << sourceStream.tellg() << "\n"; 
        std::cout << "- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -\n";
        #endif
    };

    #ifdef DEBUG2
    std::cout << "end position                        = " << sourceStream.tellg() << "\n";
    std::cout << "********************************************************************\n";
    #endif

    ERROR_HANDLER_BEGIN("HritFillSegmentHeaderFromSegment");
    ERROR_HANDLER_END;
    ERROR_END;
};


/**
 * \fn HritSegmentHeadersAreConsistent(HritSegmentHeader&,HritSegmentHeader&)
 * \brief Verify if two HritSegmentHeader-s are constistent.
 * \returns true if the HritSegmentHeaders are consistent, false if not -> int(true) = 1 and int(false) = 0.
 * \param headerData1 a HritSegmentHeader that we compare to the second one.
 * \param headerData2 the HritSegmentHeader that we compare to the first one.
 *
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

int HritSegmentHeadersAreConsistent(HritSegmentHeader &headerData1,HritSegmentHeader &headerData2){
    int returnValue = true;

    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering HritSegmentHeadersAreConsistent\n";
    u_int1 test = 0;
    int debugInt = test;
    #endif

    /* check relevant parts of imageStructureRecord */

    if(headerData1.segmentIdentificationRecord.spectralChannelID != 
       headerData2.segmentIdentificationRecord.spectralChannelID) returnValue = false;


    if(headerData1.segmentIdentificationRecord.plannedStartSegmSeqNumber != 
       headerData2.segmentIdentificationRecord.plannedStartSegmSeqNumber) returnValue = false;

    if(headerData1.segmentIdentificationRecord.plannedEndSegmSeqNumber != 
       headerData2.segmentIdentificationRecord.plannedEndSegmSeqNumber) returnValue = false;

    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Contents of segmentIdentificationRecords:\n";
    debugInt = headerData1.segmentIdentificationRecord.spectralChannelID;
    std::cout << "spectral channel 1                  = " << debugInt << "\n";
    debugInt = headerData2.segmentIdentificationRecord.spectralChannelID;
    std::cout << "spectral channel 2                  = " << debugInt << "\n";
    std::cout << "plannedStartSegmSeqNumber 1         = " << headerData1.segmentIdentificationRecord.plannedStartSegmSeqNumber << "\n";
    std::cout << "plannedStartSegmSeqNumber 2         = " << headerData2.segmentIdentificationRecord.plannedStartSegmSeqNumber << "\n";
    std::cout << "plannedEndSegmSeqNumber 1           = " << headerData1.segmentIdentificationRecord.plannedEndSegmSeqNumber << "\n";
    std::cout << "plannedEndSegmSeqNumber 2           = " << headerData2.segmentIdentificationRecord.plannedEndSegmSeqNumber << "\n";
    #endif

    if(headerData1.imageNavigationRecord.columnScalingFactor != 
       headerData2.imageNavigationRecord.columnScalingFactor) returnValue = false;

    if(headerData1.imageNavigationRecord.lineScalingFactor != 
       headerData2.imageNavigationRecord.lineScalingFactor) returnValue = false;

    if(headerData1.segmentIdentificationRecord.spectralChannelID!=12) /* <- check for all bands except the high resolution visible band, HRV, 12 */
    {
        if(headerData1.imageNavigationRecord.columnOffset != 
           headerData2.imageNavigationRecord.columnOffset) returnValue = false; 
    }

    if(headerData1.imageNavigationRecord.lineOffset != headerData2.imageNavigationRecord.lineOffset 
       + NUM_LINE_SEGMENT
       *(headerData2.segmentIdentificationRecord.segmSeqNumber-headerData1.segmentIdentificationRecord.segmSeqNumber)) 
       returnValue = false;

    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Contents of imageNavigationRecords:\n";
    std::cout << "columnScalingFactor 1               = " << headerData1.imageNavigationRecord.columnScalingFactor << "\n";
    std::cout << "columnScalingFactor 2               = " << headerData2.imageNavigationRecord.columnScalingFactor << "\n";
    std::cout << "lineScalingFactor 1                 = " << headerData1.imageNavigationRecord.lineScalingFactor << "\n";
    std::cout << "lineScalingFactor 2                 = " << headerData2.imageNavigationRecord.lineScalingFactor << "\n";
    std::cout << "columnOffset 1                      = " << headerData1.imageNavigationRecord.columnOffset << "\n";
    std::cout << "columnOffset 2                      = " << headerData2.imageNavigationRecord.columnOffset << "\n";
    std::cout << "lineOffset 1                        = " << headerData1.imageNavigationRecord.lineOffset << "\n";
    std::cout << "lineOffset 2                        = " << headerData2.imageNavigationRecord.lineOffset << "\n";
    #endif

    /* If the documentation would be right we would do the following test but as mentioned before: FISHY
    if(headerData1.timeStampRecord.timeStamp.days != 
       headerData2.timeStampRecord.timeStamp.days) return false;

    if(headerData1.timeStampRecord.timeStamp.milliSeconds != 
       headerData2.timeStampRecord.timeStamp.milliSeconds) return false;*/

    /* we now test whether the day and date of the slot for the two files match. 
     * As mentioned above we cannot use the timeStampRecord instead we use the strings in the annotationtext.
     * from a typical H-000-MSG2__-MSG2________-WV_062___-000007___-201004201200-__
     * we check "201004201200".
     */

    if(strncmp(&headerData1.annotationRecord.annotationText[46],
               &headerData2.annotationRecord.annotationText[46],12)!=0) returnValue = false;

    // by doing it like this we point to the right pointer
    // char test = "test" 
    // test -> is pointer to first position
    // &test -> is pointer to (the pointer to the first position)
    // char[n] -> is content of n-th position,
    // &char[n] -> is pointer to n-th position

    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Contents of annotationRecords:\n";
    std::cout << "annotationText 1                    = " << headerData1.annotationRecord.annotationText << "\n";
    std::cout << "annotationText 2                    = " << headerData2.annotationRecord.annotationText << "\n";
    std::cout << "annotationText 1, date time         = " << &headerData1.annotationRecord.annotationText[46] << "\n";
    std::cout << "annotationText 2, date time         = " << &headerData2.annotationRecord.annotationText[46] << "\n";
    std::cout << "strncmp (0 if equal)                = " << strncmp (&headerData1.annotationRecord.annotationText[46],
                                                                      &headerData2.annotationRecord.annotationText[46],12) << "\n"; 
    std::cout << "***************************************************************\n";
    #endif

    return returnValue;
};


/**
 * \brief Verify if two HritSegmentHeader-s are constistent.
 * \returns true if the HritSegmentHeaders are consistent, false if not -> int(true) = 1 and int(false) = 0.
 * \param pHeaderData1 a pointer to HritSegmentHeader that we compare to the second one.
 * \param pHeaderData2 the pointer to HritSegmentHeader that we compare to the first one.
 * 
 * An overloaded version of HritSegmentHeadersAreConsistent(HritSegmentHeader&,HritSegmentHeader&).
 *
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

int HritSegmentHeadersAreConsistent(HritSegmentHeader* pHeaderData1,HritSegmentHeader* pHeaderData2){
    return HritSegmentHeadersAreConsistent(*pHeaderData1,*pHeaderData2);
};


/**
 * \brief Take a compressed or uncompressed xRIT file. Return the decompressed content in a stringstream. C++ only.
 * \param filename an array of characters containing the name of the file to be opened.
 * \param strstream is a stringstream passed by reference where we will write the decompressed content.
 *        Note that the ios_base::openmode should be of type: 
 *        \code (std::stringstream::in | std::stringstream::out | std::stringstream::binary) \endcode
 *        We should test for this but I did not yet find how.
 * \return void
 *
 * \author Stijn Nevens (RMIB), Alessandro Ipe (RMIB)
 * \date Spring 2010
 *
 *
 * This function takes an compressed or uncompressed xRIT file. In the compressed case the file is decompressed. 
 * Subsequently the decompressed content is written to a stringstream. The return value indicates succes or fail.
 */

void xRITFileDecompress(char *filename, std::stringstream& strstream) {
    DISE::CxRITFile compressedFile; /** First we create a CxRITFile object. */

    try
    {
        strstream.seekp(0, std::ios::beg);// reset the position of the put pointer
        strstream.str(""); // empty content of string
    }
    catch (...)
    {
        ERROR_M1("Something went wrong resetting or emptying the stringstream: %s", filename);
    }

    try
    {
        compressedFile = DISE::CxRITFile(filename); /** Then we construct a CxRITFile object from the file 
                                                     *  using the constructor that takes a filename as input. 
                                                     */
    }
    catch (...)
    {
        ERROR_M1("Something went wrong reading input file: %s", filename);
    }

    try
    {
        /** Decompress input file into an object of type "CxRITFileDecompressed". 
         * This is done by the constructor method of this class. 
         * It constructs a decompressed version of the input file. If the file was already compressed it keeps it like it is. 
         */
        DISE::CxRITFileDecompressed decompressedFile(compressedFile);
    
        decompressedFile.Write(strstream); /** Finally we write the decompressed file stream to strstream */
    }
    catch (...)
    {
        ERROR_M1("Something went wrong decompressing input file: %s", filename);
    }   

    ERROR_HANDLER_BEGIN("xRITFileDecompress");
    ERROR_HANDLER_END;
    ERROR_END;
};


/**
 * \brief Navigate trought the header a stream containing the content of a decompressed HRIT file. C++ only.
 * \param sourceStream is a iostream passed by reference containing the decompressed content of an HRIT file.
 * \return void
 *
 * Output some relevant fields to the standard output.
 * This was a development function but I keep it here for diagnostic reasons.
 *
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void NavigateTroughHritHeader(std::iostream &sourceStream){
    std::cout << "***************************************************************\n";
    std::cout << "NavigateTroughHritHeader\n";
    u_int1 test = 0;
    int tmpInt = test;

    u_int1 headerType, fillTypeCode;
    u_int2 headerRecordLength;
    u_int4 totalHeaderLength;

    /* set the bytesex flag, the HRIT files have big endian */
    int4 byteSexFlag = 0;
    if(ByteSex()==LU_LITTLE_ENDIAN) byteSexFlag=1;

    /* set position */

    int position = 0;

    /* Verify the source stream */    
    //ERROR_IF_M(!sourceStream.good(), "The stream containing the content of a decompressed HRIT file is corrupted");

    /* set the position of the get pointer to the beginning of the stream */
    sourceStream.seekg(position, std::ios::beg); 

    /* get info from the  primaryHeaderRecord */
    sourceStream.read((char *)(&headerType), 1);

    sourceStream.read((char *)(&headerRecordLength),2);  
    if(byteSexFlag) MP_DataByteSwap(&headerRecordLength, U_INT2);

    sourceStream.read((char *)(&fillTypeCode), 1); 

    sourceStream.read((char *)(&totalHeaderLength), 4);  
    if(byteSexFlag) MP_DataByteSwap(&totalHeaderLength, U_INT4);

    position += headerRecordLength;

    #ifdef DEBUG2
    std::cout << "---------------------------------------------------------------\n";
    std::cout << "Contents of record:\n";
    tmpInt = headerType;
    std::cout << "header type                         = " << tmpInt << "\n";
    std::cout << "header record length                = " << headerRecordLength << "\n";
    tmpInt = fillTypeCode;
    std::cout << "fill type code                      = " << tmpInt << "\n";
    std::cout << "total header length                 = " << totalHeaderLength << "\n";
    std::cout << "header record lenght                = " << headerRecordLength << "\n";
    std::cout << "next start position                 = " << position << "\n";
    #endif

    while (position < totalHeaderLength)
    {
        sourceStream.seekg(position, std::ios::beg); 

        /* get info from the record */
        sourceStream.read((char *)(&headerType), 1);

        sourceStream.read((char *)(&headerRecordLength),2);  
        if(byteSexFlag) MP_DataByteSwap(&headerRecordLength, U_INT2);

        position += headerRecordLength;

        std::cout << "---------------------------------------------------------------\n";
        std::cout << "Contents of record:\n";
        tmpInt = headerType;
        std::cout << "header type                         = " << tmpInt << "\n";
        std::cout << "header record length                = " << headerRecordLength << "\n";
        std::cout << "next start position                 = " << position << "\n";
    }

    std::cout << "***************************************************************\n";
}


/** \brief Unpack 4 SEVIRI pixels from 40 bits stored as 5 times an 8 bit unsigned char
 * \param pSourceData 40 bits stored as an array with five elements corresponding to 5 times an 8 bit unsigned char
 * \param pDestinationData an array of 4 SEVIRI pixels of a 16 bit unsigned integer type
 * 
 * The SEVIRI Level 1.5 pixel binary representation is 10-bit corresponding to 
 * linearised and equalised image information 
 * (correcting for differences in response between the contributing detectors).
 * The information stored is a digital count between 0 and 1024 (2^10).
 * This information is stored in 5 times an 8 bit unsigned char that together compose 
 * 4 SEVIRI pixels.
 * 
 * This function extracts the values for 4 SEVIRI pixels (in an array) of a 16 bit unsigned integer type
 * from an array with five elements corresponding to 5 times an 8 bit unsigned char.
 *
 * The endianness of the machine running the code is of no importance.
 *
 * No input verification is done (for speed reasons), this is your problem!!!
 *
 * The information is stored as follows:
 * \code Source      = 00000000 11111111 22222222 33333333 44444444 55555555 \endcode
 * \code Destination = 00000000 00111111 11222222 22223333 33333344 44444444  \endcode
 * Where the number indicates to which byte (in source or destination) the bits belong 
 * (the bits are shown most significant bit left).
 * The byte with the lowest memory adress (left) is always the most significant (MSG is Big Endian).
 * 
 * Functionality adopted from the SAF NWC library.
 * Reimplemented for ease of understanding and forced the pDestinationData to be a 2 byte unsigned integer.
 * 
 * I will explain in a most-significant bit/byte left language how the conversion works.
 * Remember: pDestinationData[i] is 16 bits and pSourceData[i] is 8 bits.
 * 
 * - pSourceData[i]*256 : shift all the 8 bits in the i-th byte 8 bits to the left (2^8=256).
 *   the left 8 bits are now filled.
 * - + pSourceData[i+1] : ocupy the right 8 bits with the 8 bits from the i+1-th byte.
 *   the right 8 bits are now filled.
 *
 * The rest we do case by case: 
 *
 * \author Stijn Nevens (RMIB)
 * \date Autumn 2010
 */

void Unpack4PixelsFrom40Bits(unsigned char *pSourceData, u_int2 *pDestinationData){
    pDestinationData[0]=(pSourceData[0]*256+pSourceData[1])/64;
    /** - Case 1
        - /64 : shift all bits 6 bits to the right (2^6=64). */
    pDestinationData[1]=((pSourceData[1]*256+pSourceData[2])&16368)/16;
    /** - Case 2 
        - &16368 : keep bits 5 till 14 (right to left) 
          by bitwise and (&) with 16368 (binary representation = 00111111 11110000).
        - /16 : shift all bits 4 bits to the right (2^4=16) 
      */
    pDestinationData[2]=((pSourceData[2]*256+pSourceData[3])&4092)/4;
    /** - Case 3 
        - &4092 : keep bits 3 till 12 (right to left) 
          by bitwise and (&) with 4092 (binary representation = 00001111 11111100).
        - /4 : shift all bits 2 bits to the right (2^2=16) 
      */
    pDestinationData[3]=((pSourceData[3]*256+pSourceData[4])&1023);
    /** - Case 4 
        - &1023 : keep bits 1 till 10 (right to left) 
          by bitwise and (&) with 1023 (binary representation = 00000011 11111111).
      */
};


/**
 * \brief Function used for testing purposes: returns 0 if everything is ok.
 * \return 0
 *
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 *
 * Used to check if the library compiles well and if the C to C++ link works.
 */

int TestHritProcessing() {
    return 0;
};
