/**
 * \file
 * \brief The implemetation for the tests to check the processing of xrit files. 
 */

/*
 * Copyright 2010, 2011, 2012 Stijn Nevens (RMIB)
 * Copyright 2014, 2019, 2020 Edward Baudrez (RMIB)
 *
 * This file is part of libxrit
 *
 * libxrit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libxrit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libxrit.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Project headers */

#include <config.h>
#include "xritlib.h" /* the thing we are testing here */
#include "testBattery.h"

/* Standard library headers */

#include <iostream>
#include <fstream>
#include <string.h>

/* bats library headers */

#ifdef HAVE_RSLIB  /* For use when the full RMIB library is present. The header files here
                      actually tell the extract script what to take from the lib. Also, this allows
                      testing whether a problem is due to the small library or a real problem. */

#include "image_io.h"     /* for IP_ImageWrite */
#include "data_preproc.h" /* for LU_DataPreProcess */ 
#include "file_mng.h"     /* for FM_FileRemove */
#include "allocate.h"     /* for allocation and deallocation */
#include "rect.h"         /* for IP_rectangle*/

#else              /* For use when the small RMIB library is to be used */

#include "rmibsmalllib.h"

#endif

/**
 * \brief Run all tests.
 */

void TestBattery(int* pReturnValue){
    std::cout << "**********************************************************\n";
    std::cout << "Starting tests.\n";

/* */

    TestHritProcessingObjectLinking(pReturnValue);

    TestIP_RectangleHasBiggerSide(pReturnValue);

    TestIP_IsDefaultRectangle(pReturnValue);

    TestIP_RectangleOverlap(pReturnValue);

/* */

    TestxRITFileDecompress(pReturnValue);JOE;

    TestHritFillSegmentHeaderFromSegment(pReturnValue);JOE;

    TestHritSegmentHeadersAreConsistent(pReturnValue);JOE;

    TestHritSegmentHeadersAreConsistentPointers(pReturnValue);JOE;

    TestHritFillImageFromSegmentAndHeader(pReturnValue);JOE;

    TestHritFillImageAndHeaderFromSegment(pReturnValue);JOE;

    TestHritFillImageAndHeaderFromSegmentPointers(pReturnValue);JOE;

    TestHritFillImageFromSegment(pReturnValue);JOE;

    TestHritFillImageFromSegmentPointers(pReturnValue);JOE;

/* */

    TestHritFillImageAndHeaderFromSegments(pReturnValue);JOE;

    TestHritFillImageAndHeaderFromSegmentsPointers(pReturnValue);JOE;

    TestHritFillImageAndHeaderFromSegmentsSM_strings(pReturnValue);JOE;

/* */

    TestHritFillImageFromSegments(pReturnValue);JOE;

    TestHritFillImageFromSegmentsPointers(pReturnValue);JOE;

    TestHritFillImageFromSegmentsSM_strings(pReturnValue);JOE;

/* */

    TestHritFillPrologueStructureFromPrologue(pReturnValue);JOE;

    TestHritFillPrologueStructureFromPrologueFile(pReturnValue);JOE;

/* */

    TestHritFillImageAndHeaderFromChannel(pReturnValue);JOE;

    TestHritFillImageAndHeaderFromChannelPointers(pReturnValue);JOE;

/* */

    TestHritFillImageFromChannel(pReturnValue);JOE;

    TestHritFillImageFromChannelPointers(pReturnValue);JOE;

/* */

    TestHritConvertSeviriChannelNumberToChannel(pReturnValue);JOE;

    TestHritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel(pReturnValue);JOE;

/* */

    TestHritGenerateSlotDirectoryName(pReturnValue);JOE;

    TestHritGeneratePrologueLocation(pReturnValue);JOE;
    
/* */
    
    TestHritFillPrologueStructureFromBaseDirSatelliteSlotAndChannel(pReturnValue);JOE;

/* */

    TestIODCHritGeneratePrologueLocation(pReturnValue); JOE;

    TestIODCHritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel(pReturnValue); JOE;
    
    /* Error handling */
    ERROR_HANDLER_BEGIN("testBattery");
    *pReturnValue += 1;
    std::cout << "FAILED: error in unknown function in Test \n";
    ERROR_HANDLER_END;
    ERROR_END;

    if (*pReturnValue != 0)
    {
        std::cout << "FAILED: " << *pReturnValue << " tests\n";
    }
    else
    {
        std::cout << "PASSED: All tests\n";
    } 

    std::cout << "**********************************************************\n";
}


/** \brief Test if we correctly linked to the hritProcessing.a
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritProcessingObjectLinking(int *pReturnValue){
    if (TestHritProcessing() != 0) 
    {
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: Compilation of or linking to hritProcessing.o\n";
    }
    else
    {
        std::cout << "PASSED: Compilation of or linking to hritProcessing.o\n";
    }
}


/** \brief Test IP_RectangleHasBiggerSide(IP_rectangle, IP_rectangle)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestIP_RectangleHasBiggerSide(int *pReturnValue){
   IP_rectangle rectangleBig1, rectangleBig2, rectangleSmall;

    rectangleBig1.ul.x=1; rectangleBig1.ul.y=1; rectangleBig1.lr.x=10; rectangleBig1.lr.y=2;
    rectangleBig2.ul.x=1; rectangleBig2.ul.y=1; rectangleBig2.lr.x=2; rectangleBig2.lr.y=10;
    rectangleSmall.ul.x=1; rectangleSmall.ul.y=1; rectangleSmall.lr.x=2; rectangleSmall.lr.y=2;

    if (IP_RectangleHasBiggerSide(rectangleBig1,rectangleSmall) &&
        IP_RectangleHasBiggerSide(rectangleBig2,rectangleSmall) &&
        not(IP_RectangleHasBiggerSide(rectangleSmall,rectangleBig1)) &&
        not(IP_RectangleHasBiggerSide(rectangleSmall,rectangleBig2)) &&
        not(IP_RectangleHasBiggerSide(rectangleSmall,rectangleSmall))
       )
    {
        std::cout << "PASSED: IP_RectangleHasBiggerSide\n";
    }
    else
    {
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: IP_RectangleHasBiggerSide\n";
    } 
}


/** \brief Test IP_IsDefaultRectangle(IP_rectangle)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestIP_IsDefaultRectangle(int *pReturnValue){
    IP_rectangle rectangle1, rectangle2;

    rectangle1.ul.x=-1; rectangle1.ul.y=-1; rectangle1.lr.x=-1; rectangle1.lr.y=-1;
    rectangle2.ul.x=1; rectangle2.ul.y=-1; rectangle2.lr.x=-1; rectangle2.lr.y=-1;

    if (IP_IsDefaultRectangle(rectangle1) &&
        not(IP_IsDefaultRectangle(rectangle2))
      )
    {
        std::cout << "PASSED: IP_IsDefaultRectangle\n";
    }
    else
    {
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: IP_IsDefaultRectangle\n";
    } 
}


/** \brief Test IP_RectangleOverlap(IP_rectangle,IP_rectangle)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestIP_RectangleOverlap(int *pReturnValue){
    IP_rectangle rectangle1, rectangle2, rectangle3;

    rectangle1.ul.x=5; rectangle1.ul.y=5; rectangle1.lr.x=6; rectangle1.lr.y=6;
    rectangle2.ul.x=5; rectangle2.ul.y=5; rectangle2.lr.x=7; rectangle2.lr.y=7;
    rectangle3.ul.x=6; rectangle3.ul.y=6; rectangle3.lr.x=7; rectangle3.lr.y=7;

    if (IP_RectangleOverlap(rectangle1,rectangle1) &&
        IP_RectangleOverlap(rectangle1,rectangle2) &&
        IP_RectangleOverlap(rectangle2,rectangle1) &&
        not(IP_RectangleOverlap(rectangle1,rectangle3)) &&
        not(IP_RectangleOverlap(rectangle3,rectangle1)) &&
        IP_RectangleOverlap(rectangle2,rectangle3) &&
        IP_RectangleOverlap(rectangle3,rectangle2)
      )
    {
        std::cout << "PASSED: IP_RectangleOverlap\n";
    }
    else
    {
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: IP_RectangleOverlap\n";
    } 
}


/** \brief Test xRITFileDecompress(char*,std::stringstream&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestxRITFileDecompress(int *pReturnValue){
    std::stringstream testStream(std::stringstream::in | std::stringstream::out | std::stringstream::binary);
    char testFile[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_";

    xRITFileDecompress(testFile, testStream);
    if (UNLIKELY(Err.ErrStat))
    {
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: xRITFileDecompress\n";
        ERROR_CLEAR;
    }
    else
    {
        std::cout << "PASSED: xRITFileDecompress\n";
    } 
}

/** \brief Test HritFillSegmentHeaderFromSegment(std::stringstream&, HritSegmentHeader&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritFillSegmentHeaderFromSegment(int *pReturnValue){
    /* seting up the tests */
    std::stringstream testStream(std::stringstream::in | std::stringstream::out | std::stringstream::binary);
    char testFile[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_";
    HritSegmentHeader destinationHeaderData;

    xRITFileDecompress(testFile, testStream);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillSegmentHeaderFromSegment: xRITFileDecompress\n";
        JOE;
    }

    HritFillSegmentHeaderFromSegment(testStream, destinationHeaderData); JOE;

    if (false /* functional tests */)
    {
        ERROR();
        JOE; /* if you add more tests we need to raise an error before the JOE */
    }
    else
    {
        std::cout << "PASSED: HritFillHeaderFromSegment\n";
    } 

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillSegmentHeaderFromSegment");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillHeaderFromSegment\n";
    ERROR_HANDLER_END;
    ERROR_END;
    ERROR_CLEAR;
}


/** \brief Test HritSegmentHeadersAreConsistent(HritSegmentHeader&,HritSegmentHeader&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritSegmentHeadersAreConsistent(int *pReturnValue){
    /* seting up the tests */
    std::stringstream testStream(std::stringstream::in | std::stringstream::out | std::stringstream::binary);
    char testFile1[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000007___-201004191200-C_";
    char testFile2[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000002___-201004191200-C_";
    HritSegmentHeader destinationHeaderData1;
    HritSegmentHeader destinationHeaderData2;
    HritSegmentHeader destinationHeaderData3;
    HritSegmentHeader destinationHeaderData4;

    xRITFileDecompress(testFile1, testStream);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritSegmentHeadersAreConsistent: xRITFileDecompress\n";
        JOE;
    }

    HritFillSegmentHeaderFromSegment(testStream, destinationHeaderData1);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritSegmentHeadersAreConsistent: HritFillSegmentHeaderFromSegment\n";
        JOE;
    }

    xRITFileDecompress(testFile2, testStream);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritSegmentHeadersAreConsistent: xRITFileDecompress\n";
        JOE;
    }

    HritFillSegmentHeaderFromSegment(testStream, destinationHeaderData2);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritSegmentHeadersAreConsistent: HritFillSegmentHeaderFromSegment\n";
        JOE;
    }

    destinationHeaderData3 = destinationHeaderData2;
    destinationHeaderData3.annotationRecord.annotationText[46] = 'Z';
    destinationHeaderData4 = destinationHeaderData2;
    destinationHeaderData4.annotationRecord.annotationText[57] = 'Z';

    /* the tests */
    if (HritSegmentHeadersAreConsistent(destinationHeaderData1,destinationHeaderData2) == false || 
        HritSegmentHeadersAreConsistent(destinationHeaderData1,destinationHeaderData3) == true  || 
        HritSegmentHeadersAreConsistent(destinationHeaderData1,destinationHeaderData4) == true 
       )
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritSegmentHeadersAreConsistent\n";
    }

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritSegmentHeadersAreConsistent");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritSegmentHeadersAreConsistent\n";
    ERROR_HANDLER_END;
    ERROR_END;
    ERROR_CLEAR;
}


/** \brief Test HritSegmentHeadersAreConsistent(HritSegmentHeader*,HritSegmentHeader*)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritSegmentHeadersAreConsistentPointers(int *pReturnValue){
    /* seting up the tests */
    std::stringstream testStream(std::stringstream::in | std::stringstream::out | std::stringstream::binary);
    char testFile1[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000007___-201004191200-C_";
    char testFile2[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000002___-201004191200-C_";
    HritSegmentHeader destinationHeaderData1;
    HritSegmentHeader destinationHeaderData2;
    HritSegmentHeader destinationHeaderData3;
    HritSegmentHeader destinationHeaderData4;

    xRITFileDecompress(testFile1, testStream);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritSegmentHeadersAreConsistent with Pointers: xRITFileDecompress\n";
        JOE;
    }

    HritFillSegmentHeaderFromSegment(testStream, destinationHeaderData1);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritSegmentHeadersAreConsistent with Pointers: HritFillSegmentHeaderFromSegment\n";
        JOE;
    }

    xRITFileDecompress(testFile2, testStream);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritSegmentHeadersAreConsistent with Pointers: xRITFileDecompress\n";
        JOE;
    }

    HritFillSegmentHeaderFromSegment(testStream, destinationHeaderData2);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritSegmentHeadersAreConsistent with Pointers: HritFillSegmentHeaderFromSegment\n";
        JOE;
    }

    destinationHeaderData3 = destinationHeaderData2;
    destinationHeaderData3.annotationRecord.annotationText[46] = 'Z';
    destinationHeaderData4 = destinationHeaderData2;
    destinationHeaderData4.annotationRecord.annotationText[57] = 'Z';

    /* the tests */
    if (HritSegmentHeadersAreConsistent(&destinationHeaderData1,&destinationHeaderData2) == false || 
        HritSegmentHeadersAreConsistent(&destinationHeaderData1,&destinationHeaderData3) == true  || 
        HritSegmentHeadersAreConsistent(&destinationHeaderData1,&destinationHeaderData4) == true 
       )
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritSegmentHeadersAreConsistent with Pointers\n";
    }

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritSegmentHeadersAreConsistentPointers");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritSegmentHeadersAreConsistent with Pointers\n";
    ERROR_HANDLER_END;
    ERROR_END;
    ERROR_CLEAR;
}


/** \brief Test HritFillImageFromSegmentAndHeader(std::stringstream&sourceStream,IP_rectangle&,IP_image&,IP_rectangle&,const HritSegmentHeader&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritFillImageFromSegmentAndHeader(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageFromSegmentAndHeader\n";
    #endif

    /* seting up the test data */
    std::stringstream testStream(std::stringstream::in | std::stringstream::out | std::stringstream::binary);
    char testFile[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_";
    HritSegmentHeader headerData;

    IP_rectangle sourceRectangle;
    IP_rectangle destinationRectangle;
    INIT_IP_RECTANGLE(sourceRectangle);
    INIT_IP_RECTANGLE(destinationRectangle);
    sourceRectangle.ul.x=1000; sourceRectangle.ul.y=3712-400; sourceRectangle.lr.x=2000; sourceRectangle.lr.y=3712-100;
    destinationRectangle.ul.x=0; destinationRectangle.ul.y=0; destinationRectangle.lr.x=1000; destinationRectangle.lr.y=300;

    IP_image *pDestinationImage = NULL;
    pDestinationImage = IP_ImageAlloc(INT2, 1000, 300); JOE;
    //pDestinationImage = IP_ImageAlloc(INT2, 3712, 3712); JOE;
    //pDestinationImage = IP_ImageAlloc(INT2, 5568, 11136); JOE;

    #ifdef DEBUG2
    IP_RectangleDisplay(&sourceRectangle);
    IP_RectangleDisplay(&destinationRectangle);
    #endif


    xRITFileDecompress(testFile, testStream);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageFromSegmentAndHeader: xRITFileDecompress\n";
        JOE;
    }

    HritFillSegmentHeaderFromSegment(testStream, headerData);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageFromSegmentAndHeader: HritFillSegmentHeaderFromSegment\n";
        JOE;
    }

    HritFillImageFromSegmentAndHeader(testStream,sourceRectangle,*pDestinationImage,destinationRectangle,headerData); JOE;

    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageFromSegmentAndHeader\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputHritFillImageFromSegmentAndHeader.rof", pDestinationImage); /* Err.ErrStat = -1; */
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST HritFillImageFromSegmentAndHeader: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageFromSegmentAndHeader");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageFromSegmentAndHeader\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImage);
    ERROR_CLEAR;
}


/** \brief Test HritFillImageAndHeaderFromSegment(char*,IP_rectangle&,IP_image&,IP_rectangle&,HritSegmentHeader&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritFillImageAndHeaderFromSegment(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageAndHeaderFromSegment\n";
    #endif

    /* seting up the test data */
    char testFile[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_";
    HritSegmentHeader headerData;

    IP_rectangle sourceRectangle;
    IP_rectangle destinationRectangle;
    INIT_IP_RECTANGLE(sourceRectangle);
    INIT_IP_RECTANGLE(destinationRectangle);
    sourceRectangle.ul.x=1000; sourceRectangle.ul.y=3712-400; sourceRectangle.lr.x=2000; sourceRectangle.lr.y=3712-100;
    destinationRectangle.ul.x=0; destinationRectangle.ul.y=0; destinationRectangle.lr.x=1000; destinationRectangle.lr.y=300;

    IP_image *pDestinationImage = NULL;
    pDestinationImage = IP_ImageAlloc(INT2, 1000, 300); JOE;
    //pDestinationImage = IP_ImageAlloc(INT2, 3712, 3712); JOE;
    //pDestinationImage = IP_ImageAlloc(INT2, 5568, 11136); JOE;

    #ifdef DEBUG2
    IP_RectangleDisplay(&sourceRectangle);
    IP_RectangleDisplay(&destinationRectangle);
    #endif

    HritFillImageAndHeaderFromSegment(testFile,sourceRectangle,*pDestinationImage,destinationRectangle,headerData); JOE;

    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageAndHeaderFromSegment\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputHritFillImageAndHeaderFromSegment.rof", pDestinationImage); JOE; 
    /* std::cout << "FAILED WITHIN TEST TestHritFillImageAndHeaderFromSegment: IP_ImageWrite\n"; */
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageAndHeaderFromSegment");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageAndHeaderFromSegment\n";
    ERROR_HANDLER_END;
        IP_ImageAdrFree(&pDestinationImage);
    ERROR_END;
    ERROR_CLEAR;
}


/** \brief Test HritFillImageAndHeaderFromSegment(char*,IP_rectangle*,IP_image*,IP_rectangle*,HritSegmentHeader*)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritFillImageAndHeaderFromSegmentPointers(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageAndHeaderFromSegmentPointers\n";
    #endif

    /* seting up the test data */
    char testFile[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_";
    HritSegmentHeader headerData;

    IP_rectangle sourceRectangle;
    IP_rectangle destinationRectangle;
    INIT_IP_RECTANGLE(sourceRectangle);
    INIT_IP_RECTANGLE(destinationRectangle);
    sourceRectangle.ul.x=1000; sourceRectangle.ul.y=3712-400; sourceRectangle.lr.x=2000; sourceRectangle.lr.y=3712-100;
    destinationRectangle.ul.x=0; destinationRectangle.ul.y=0; destinationRectangle.lr.x=1000; destinationRectangle.lr.y=300;

    IP_image *pDestinationImage = NULL;
    pDestinationImage = IP_ImageAlloc(INT2, 1000, 300); JOE;
    //pDestinationImage = IP_ImageAlloc(INT2, 3712, 3712); JOE;
    //pDestinationImage = IP_ImageAlloc(INT2, 5568, 11136); JOE;

    #ifdef DEBUG2
    IP_RectangleDisplay(&sourceRectangle);
    IP_RectangleDisplay(&destinationRectangle);
    #endif

    HritFillImageAndHeaderFromSegment(testFile,&sourceRectangle,pDestinationImage,&destinationRectangle,&headerData); JOE;

    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageAndHeaderFromSegment with Pointers\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputHritFillImageAndHeaderFromSegmentWithPointers.rof", pDestinationImage);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageAndHeaderFromSegmentPointers: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageAndHeaderFromSegmentPointers");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageAndHeaderFromSegment with Pointers\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImage);
    ERROR_CLEAR;
}


/** \brief Test HritFillImageFromSegment(char*,IP_rectangle&,IP_image&,IP_rectangle&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritFillImageFromSegment(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageFromSegment\n";
    #endif

    /* seting up the test data */
    char testFileGenericChannel[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000003___-201004191200-C_";

    IP_rectangle sourceRectangleGenericChannelFull;
    IP_rectangle destinationRectangleGenericChannelFull;
    IP_image *pDestinationImageGenericChannelFull = NULL;

    IP_rectangle sourceRectangleGenericChannelPart;
    IP_rectangle destinationRectangleGenericChannelPart;
    IP_image *pDestinationImageGenericChannelPart = NULL;

    char testFileVisibleChannel[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000008___-201004191200-C_";

    IP_rectangle sourceRectangleVisibleChannelFull;
    IP_rectangle destinationRectangleVisibleChannelFull;
    IP_image *pDestinationImageVisibleChannelFull = NULL;

    IP_rectangle sourceRectangleVisibleChannelPart;
    IP_rectangle destinationRectangleVisibleChannelPart;
    IP_image *pDestinationImageVisibleChannelPart = NULL;

    /* init */

    INIT_IP_RECTANGLE(sourceRectangleGenericChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleGenericChannelFull);
    pDestinationImageGenericChannelFull = IP_ImageAlloc(U_INT2, 3712, 3712); JOE;

    pDestinationImageGenericChannelPart = IP_ImageAlloc(U_INT2, 500, 1000); JOE;
    sourceRectangleGenericChannelPart.ul.x=1000; sourceRectangleGenericChannelPart.ul.y=3712-2200; 
    sourceRectangleGenericChannelPart.lr.x=1500; sourceRectangleGenericChannelPart.lr.y=3712-1200;
    destinationRectangleGenericChannelPart.ul.x=0; destinationRectangleGenericChannelPart.ul.y=0; 
    destinationRectangleGenericChannelPart.lr.x=500; destinationRectangleGenericChannelPart.lr.y=1000;
 // -> we should test this too -> INIT_IP_RECTANGLE(destinationRectangleGenericChannelPart);

    INIT_IP_RECTANGLE(sourceRectangleVisibleChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleVisibleChannelFull);
    pDestinationImageVisibleChannelFull = IP_ImageAlloc(U_INT2, 5568, 11136); JOE;

    pDestinationImageVisibleChannelPart = IP_ImageAlloc(U_INT2, 2000, 8000); JOE;
    sourceRectangleVisibleChannelPart.ul.x=1000; sourceRectangleVisibleChannelPart.ul.y=11136-10100; 
    sourceRectangleVisibleChannelPart.lr.x=3000; sourceRectangleVisibleChannelPart.lr.y=11136-2100;
    destinationRectangleVisibleChannelPart.ul.x=0; destinationRectangleVisibleChannelPart.ul.y=0;
    destinationRectangleVisibleChannelPart.lr.x=2000; destinationRectangleVisibleChannelPart.lr.y=8000;

    #ifdef DEBUG2
    //IP_RectangleDisplay(&sourceRectangle);
    //IP_RectangleDisplay(&destinationRectangle);
    #endif

    HritFillImageFromSegment(testFileGenericChannel,sourceRectangleGenericChannelFull,
                             *pDestinationImageGenericChannelFull,destinationRectangleGenericChannelFull); JOE;

    HritFillImageFromSegment(testFileGenericChannel,sourceRectangleGenericChannelPart,
                             *pDestinationImageGenericChannelPart,destinationRectangleGenericChannelPart); JOE;

    HritFillImageFromSegment(testFileVisibleChannel,sourceRectangleVisibleChannelFull,
                             *pDestinationImageVisibleChannelFull,destinationRectangleVisibleChannelFull); JOE;

    HritFillImageFromSegment(testFileVisibleChannel,sourceRectangleVisibleChannelPart,
                             *pDestinationImageVisibleChannelPart,destinationRectangleVisibleChannelPart); JOE;

    /* functional tests */

    ERROR_IF_V(IP_ImageGetIntValue(pDestinationImageGenericChannelFull, 2000, 2500) != 735);
    ERROR_IF_V(IP_ImageGetIntValue(pDestinationImageGenericChannelPart, 250, 900) != 373);
    ERROR_IF_V(IP_ImageGetIntValue(pDestinationImageVisibleChannelFull, 2500, 7500) != 589);
    ERROR_IF_V(IP_ImageGetIntValue(pDestinationImageVisibleChannelPart, 1000, 6500) != 392);

    std::cout << "PASSED: HritFillImageFromSegment\n"; 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputHritFillImageFromSegmentGenericChannelFull.rof", pDestinationImageGenericChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputHritFillImageFromSegmentGenericChannelPart.rof", pDestinationImageGenericChannelPart);
    IP_ImageWrite((char *)SRCDIR "/Output/outputHritFillImageFromSegmentVisibleChannelFull.rof", pDestinationImageVisibleChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputHritFillImageFromSegmentVisibleChannelPart.rof", pDestinationImageVisibleChannelPart);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageFromSegment: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageFromSegment");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageFromSegment\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImageVisibleChannelPart);
        IP_ImageAdrFree(&pDestinationImageVisibleChannelFull);
        IP_ImageAdrFree(&pDestinationImageGenericChannelPart);
        IP_ImageAdrFree(&pDestinationImageGenericChannelFull);
    ERROR_CLEAR;
}


/** \brief Test HritFillImageFromSegment(char*,IP_rectangle*,IP_image*,IP_rectangle*)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritFillImageFromSegmentPointers(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageFromSegmentPointers\n";
    #endif

    /* seting up the test data */
    char testFile[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_";

    IP_rectangle sourceRectangle;
    IP_rectangle destinationRectangle;
    INIT_IP_RECTANGLE(sourceRectangle);
    INIT_IP_RECTANGLE(destinationRectangle);
    sourceRectangle.ul.x=1000; sourceRectangle.ul.y=3712-400; sourceRectangle.lr.x=2000; sourceRectangle.lr.y=3712-100;
    destinationRectangle.ul.x=0; destinationRectangle.ul.y=0; destinationRectangle.lr.x=1000; destinationRectangle.lr.y=300;

    IP_image *pDestinationImage = NULL;
    pDestinationImage = IP_ImageAlloc(INT2, 1000, 300); JOE;
    //pDestinationImage = IP_ImageAlloc(INT2, 3712, 3712); JOE;
    //pDestinationImage = IP_ImageAlloc(INT2, 5568, 11136); JOE;

    #ifdef DEBUG2
    IP_RectangleDisplay(&sourceRectangle);
    IP_RectangleDisplay(&destinationRectangle);
    #endif

    HritFillImageFromSegment(testFile,&sourceRectangle,pDestinationImage,&destinationRectangle); JOE;

    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageFromSegment with Pointers\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputHritFillImageFromSegmentWithPointers.rof", pDestinationImage); 
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageFromSegmentPointers: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageFromSegmentPointers");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageFromSegment with Pointers\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImage);
    ERROR_CLEAR;
}


/** \brief Test HritFillImageAndHeaderFromSegments(char**,int,IP_rectangle&,IP_image&, IP_rectangle&,HritSegmentHeader&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Summer 2010
 */

void TestHritFillImageAndHeaderFromSegments(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageAndHeaderFromSegments\n";
    #endif

    /* setting up the test data */

    HritSegmentHeader headerData;

    char*testFiles[8] = {(char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000006___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000002___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000003___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000004___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000005___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000007___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000008___-201004191200-C_"};

    IP_image *pDestinationImage = NULL;
    SM_strings *pTestFilesSM = NULL;

    IP_rectangle sourceRectangle;
    IP_rectangle destinationRectangle;
    INIT_IP_RECTANGLE(sourceRectangle);
    INIT_IP_RECTANGLE(destinationRectangle);

    sourceRectangle.ul.x=1000; sourceRectangle.ul.y=3712-3100; sourceRectangle.lr.x=2000; sourceRectangle.lr.y=3712-100;
    destinationRectangle.ul.x=0; destinationRectangle.ul.y=0; destinationRectangle.lr.x=1000; destinationRectangle.lr.y=3000;
    
    /*IP_RectangleDisplay(&sourceRectangle);
    IP_RectangleDisplay(&destinationRectangle);*/

    pDestinationImage = IP_ImageAlloc(INT2, 1000, 3000);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageAndHeaderFromSegments: IP_ImageAlloc\n";
        JOE;
    }

    HritFillImageAndHeaderFromSegments(testFiles,8,sourceRectangle,*pDestinationImage,destinationRectangle,headerData);JOE;

    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageAndHeaderFromSegments\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromSegments.rof", pDestinationImage);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageAndHeaderFromSegments: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageAndHeaderFromSegments");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageAndHeaderFromSegments\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImage);
    ERROR_CLEAR;
}


/** \brief Test HritFillImageAndHeaderFromSegments(char**,int,IP_rectangle&,IP_image&, IP_rectangle&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Summer 2010
 */

void TestHritFillImageAndHeaderFromSegmentsPointers(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImagAndHeadereFromSegmentsPointer\n";
    #endif

    /* setting up the test data */

    HritSegmentHeader headerData;

    char*testFiles[8] = {(char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000006___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000002___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000003___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000004___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000005___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000007___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000008___-201004191200-C_"};

    IP_image *pDestinationImage = NULL;
    SM_strings *pTestFilesSM = NULL;

    IP_rectangle sourceRectangle;
    IP_rectangle destinationRectangle;
    INIT_IP_RECTANGLE(sourceRectangle);
    INIT_IP_RECTANGLE(destinationRectangle);

    sourceRectangle.ul.x=1000; sourceRectangle.ul.y=3712-3100; sourceRectangle.lr.x=2000; sourceRectangle.lr.y=3712-100;
    destinationRectangle.ul.x=0; destinationRectangle.ul.y=0; destinationRectangle.lr.x=1000; destinationRectangle.lr.y=3000;
    
    /*IP_RectangleDisplay(&sourceRectangle);
    IP_RectangleDisplay(&destinationRectangle);*/

    pDestinationImage = IP_ImageAlloc(INT2, 1000, 3000);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageAndHeaderFromSegmentsPointers: IP_ImageAlloc\n";
        JOE;
    }

    HritFillImageAndHeaderFromSegments(testFiles,8,&sourceRectangle,pDestinationImage,&destinationRectangle,&headerData);JOE;

    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageAndHeaderFromSegments with Pointers\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromSegmentsPointers.rof", pDestinationImage);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageAndHeaderFromSegmentsPointers: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageAndHeaderFromSegmentsPointers");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageAndHeaderFromSegments with Pointers\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImage);
    ERROR_CLEAR;
}


/** \brief Test HritFillImageAndHeaderFromSegments(SM_strings*,IP_rectangle&,IP_image&, IP_rectangle&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Summer 2010
 */

void TestHritFillImageAndHeaderFromSegmentsSM_strings(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageAndHeaderFromSegmentsSM_strings\n";
    #endif

    /* setting up the test data */

    /* headers */

    HritSegmentHeader headerDataGenericChannelFull;
    HritSegmentHeader headerDataGenericChannelPart;
    HritSegmentHeader headerDataVisibleChannelFull;
    HritSegmentHeader headerDataVisibleChannelPart;

    /* data for channels with 8 segments */

    SM_strings *pTestFilesSMGenericChannel = NULL;

    IP_rectangle sourceRectangleGenericChannelFull;
    IP_rectangle destinationRectangleGenericChannelFull;
    IP_image *pDestinationImageGenericChannelFull = NULL;

    IP_rectangle sourceRectangleGenericChannelPart;
    IP_rectangle destinationRectangleGenericChannelPart;
    IP_image *pDestinationImageGenericChannelPart = NULL;

    /* data for the visible channel with 20 segments */

    SM_strings *pTestFilesSMVisibleChannel = NULL;

    IP_rectangle sourceRectangleVisibleChannelFull;
    IP_rectangle destinationRectangleVisibleChannelFull;
    IP_image *pDestinationImageVisibleChannelFull = NULL;

    IP_rectangle sourceRectangleVisibleChannelPart;
    IP_rectangle destinationRectangleVisibleChannelPart;
    IP_image *pDestinationImageVisibleChannelPart = NULL;

    /* init */

    pDestinationImageGenericChannelFull = IP_ImageAlloc(INT2, 3712, 3712); JOE;
    INIT_IP_RECTANGLE(sourceRectangleGenericChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleGenericChannelFull);

    pDestinationImageGenericChannelPart = IP_ImageAlloc(INT2, 1000, 3000); JOE;
    sourceRectangleGenericChannelPart.ul.x=1000; sourceRectangleGenericChannelPart.ul.y=3712-3100; 
    sourceRectangleGenericChannelPart.lr.x=2000; sourceRectangleGenericChannelPart.lr.y=3712-100;
    destinationRectangleGenericChannelPart.ul.x=0; destinationRectangleGenericChannelPart.ul.y=0; 
    destinationRectangleGenericChannelPart.lr.x=1000; destinationRectangleGenericChannelPart.lr.y=3000;

    pTestFilesSMGenericChannel = SM_StringsAlloc(0); JOE;
    SM_StringsAddToStrings(pTestFilesSMGenericChannel,
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000006___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000002___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000003___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000004___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000005___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000007___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000008___-201004191200-C_",
    NULL); JOE;

    pDestinationImageVisibleChannelFull = IP_ImageAlloc(INT2, 5568, 11136); JOE;
    INIT_IP_RECTANGLE(sourceRectangleVisibleChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleVisibleChannelFull);

    pDestinationImageVisibleChannelPart = IP_ImageAlloc(INT2, 2000, 8000); JOE;
    sourceRectangleVisibleChannelPart.ul.x=1000; sourceRectangleVisibleChannelPart.ul.y=11136-10100; 
    sourceRectangleVisibleChannelPart.lr.x=3000; sourceRectangleVisibleChannelPart.lr.y=11136-2100;
    destinationRectangleVisibleChannelPart.ul.x=0; destinationRectangleVisibleChannelPart.ul.y=0;
    destinationRectangleVisibleChannelPart.lr.x=2000; destinationRectangleVisibleChannelPart.lr.y=8000;

    pTestFilesSMVisibleChannel = SM_StringsAlloc(0); JOE;
    SM_StringsAddToStrings(pTestFilesSMVisibleChannel,
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000001___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000002___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000003___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000004___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000005___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000006___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000007___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000008___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000009___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000010___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000011___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000012___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000013___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000014___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000015___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000016___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000017___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000018___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000019___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000020___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000021___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000022___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000023___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000024___-201004191200-C_",
    NULL); JOE;

    /*SM_StringsDisplay(pTestFilesSM);*/

    /* do processing */

    HritFillImageAndHeaderFromSegments(pTestFilesSMGenericChannel,&sourceRectangleGenericChannelFull,
                                       pDestinationImageGenericChannelFull,&destinationRectangleGenericChannelFull,
                                       &headerDataGenericChannelFull);JOE;

    HritFillImageAndHeaderFromSegments(pTestFilesSMGenericChannel,&sourceRectangleGenericChannelPart,
                                       pDestinationImageGenericChannelPart,&destinationRectangleGenericChannelPart,
                                       &headerDataGenericChannelPart);JOE;

    HritFillImageAndHeaderFromSegments(pTestFilesSMVisibleChannel,&sourceRectangleVisibleChannelFull,
                                       pDestinationImageVisibleChannelFull,&destinationRectangleVisibleChannelFull,
                                       &headerDataVisibleChannelFull);JOE;

   HritFillImageAndHeaderFromSegments(pTestFilesSMVisibleChannel,&sourceRectangleVisibleChannelPart,
                                      pDestinationImageVisibleChannelPart,&destinationRectangleVisibleChannelPart,
                                      &headerDataVisibleChannelPart);JOE;

    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageAndHeaderFromSegments with SM_strings\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromSegmentsSM_stringsGenericChannelFull.rof", pDestinationImageGenericChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromSegmentsSM_stringsGenericChannelPart.rof", pDestinationImageGenericChannelPart);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromSegmentsSM_stringsVisibleChannelFull.rof", pDestinationImageVisibleChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromSegmentsSM_stringsVisibleChannelPart.rof", pDestinationImageVisibleChannelPart);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageAndHeaderFromSegmentsSM_strings: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageAndHeaderFromSegmentsSM_strings");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageAndHeaderFromSegments with SM_strings\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImageVisibleChannelPart);
        IP_ImageAdrFree(&pDestinationImageVisibleChannelFull);
        IP_ImageAdrFree(&pDestinationImageGenericChannelPart);
        IP_ImageAdrFree(&pDestinationImageGenericChannelFull);
    ERROR_CLEAR;
}


/** \brief Test HritFillImageFromSegments(char**,int,IP_rectangle&,IP_image&, IP_rectangle&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritFillImageFromSegments(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageFromSegments\n";
    #endif

    /* setting up the test data */

    char*testFiles[8] = {(char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000006___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000002___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000003___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000004___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000005___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000007___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000008___-201004191200-C_"};

    IP_image *pDestinationImage = NULL;
    SM_strings *pTestFilesSM = NULL;

    IP_rectangle sourceRectangle;
    IP_rectangle destinationRectangle;
    INIT_IP_RECTANGLE(sourceRectangle);
    INIT_IP_RECTANGLE(destinationRectangle);

    sourceRectangle.ul.x=1000; sourceRectangle.ul.y=3712-3100; sourceRectangle.lr.x=2000; sourceRectangle.lr.y=3712-100;
    destinationRectangle.ul.x=0; destinationRectangle.ul.y=0; destinationRectangle.lr.x=1000; destinationRectangle.lr.y=3000;
    
    /*IP_RectangleDisplay(&sourceRectangle);
    IP_RectangleDisplay(&destinationRectangle);*/

    pDestinationImage = IP_ImageAlloc(INT2, 1000, 3000);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageFromSegments: IP_ImageAlloc\n";
        JOE;
    }

    HritFillImageFromSegments(testFiles,8,sourceRectangle,*pDestinationImage,destinationRectangle);JOE;

    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageFromSegments\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromSegments.rof", pDestinationImage);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageFromSegments: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageFromSegments");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageFromSegments\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImage);
    ERROR_CLEAR;
}


/** \brief Test HritFillImageFromSegments(char**,int,IP_rectangle&,IP_image&, IP_rectangle&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritFillImageFromSegmentsPointers(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageFromSegmentsPointer\n";
    #endif

    /* setting up the test data */

    char*testFiles[8] = {(char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000006___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000002___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000003___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000004___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000005___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000007___-201004191200-C_",
                         (char *)SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000008___-201004191200-C_"};

    IP_image *pDestinationImage = NULL;
    SM_strings *pTestFilesSM = NULL;

    IP_rectangle sourceRectangle;
    IP_rectangle destinationRectangle;
    INIT_IP_RECTANGLE(sourceRectangle);
    INIT_IP_RECTANGLE(destinationRectangle);

    sourceRectangle.ul.x=1000; sourceRectangle.ul.y=3712-3100; sourceRectangle.lr.x=2000; sourceRectangle.lr.y=3712-100;
    destinationRectangle.ul.x=0; destinationRectangle.ul.y=0; destinationRectangle.lr.x=1000; destinationRectangle.lr.y=3000;
    
    /*IP_RectangleDisplay(&sourceRectangle);
    IP_RectangleDisplay(&destinationRectangle);*/

    pDestinationImage = IP_ImageAlloc(INT2, 1000, 3000);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageFromSegmentsPointers: IP_ImageAlloc\n";
        JOE;
    }

    HritFillImageFromSegments(testFiles,8,&sourceRectangle,pDestinationImage,&destinationRectangle);JOE;

    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageFromSegments with Pointers\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromSegmentsPointers.rof", pDestinationImage);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageFromSegmentsPointers: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageFromSegmentsPointers");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageFromSegments with Pointers\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImage);
    ERROR_CLEAR;
}


/** \brief Test HritFillImageFromSegments(SM_strings*,IP_rectangle&,IP_image&, IP_rectangle&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritFillImageFromSegmentsSM_strings(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageFromSegmentsSM_strings\n";
    #endif

    /* setting up the test data */

    /* data for channels with 8 segments */

    SM_strings *pTestFilesSMGenericChannel = NULL;

    IP_rectangle sourceRectangleGenericChannelFull;
    IP_rectangle destinationRectangleGenericChannelFull;
    IP_image *pDestinationImageGenericChannelFull = NULL;

    IP_rectangle sourceRectangleGenericChannelPart;
    IP_rectangle destinationRectangleGenericChannelPart;
    IP_image *pDestinationImageGenericChannelPart = NULL;

    /* data for the visible channel with 20 segments */

    SM_strings *pTestFilesSMVisibleChannel = NULL;

    IP_rectangle sourceRectangleVisibleChannelFull;
    IP_rectangle destinationRectangleVisibleChannelFull;
    IP_image *pDestinationImageVisibleChannelFull = NULL;

    IP_rectangle sourceRectangleVisibleChannelPart;
    IP_rectangle destinationRectangleVisibleChannelPart;
    IP_image *pDestinationImageVisibleChannelPart = NULL;

    /* init */

    pDestinationImageGenericChannelFull = IP_ImageAlloc(INT2, 3712, 3712); JOE;
    INIT_IP_RECTANGLE(sourceRectangleGenericChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleGenericChannelFull);

    pDestinationImageGenericChannelPart = IP_ImageAlloc(INT2, 1000, 3000); JOE;
    sourceRectangleGenericChannelPart.ul.x=1000; sourceRectangleGenericChannelPart.ul.y=3712-3100; 
    sourceRectangleGenericChannelPart.lr.x=2000; sourceRectangleGenericChannelPart.lr.y=3712-100;
    destinationRectangleGenericChannelPart.ul.x=0; destinationRectangleGenericChannelPart.ul.y=0; 
    destinationRectangleGenericChannelPart.lr.x=1000; destinationRectangleGenericChannelPart.lr.y=3000;

    pTestFilesSMGenericChannel = SM_StringsAlloc(0); JOE;
    SM_StringsAddToStrings(pTestFilesSMGenericChannel,
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000006___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000002___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000003___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000004___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000005___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000007___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000008___-201004191200-C_",
    NULL); JOE;

    pDestinationImageVisibleChannelFull = IP_ImageAlloc(INT2, 5568, 11136); JOE;
    INIT_IP_RECTANGLE(sourceRectangleVisibleChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleVisibleChannelFull);

    pDestinationImageVisibleChannelPart = IP_ImageAlloc(INT2, 2000, 8000); JOE;
    sourceRectangleVisibleChannelPart.ul.x=1000; sourceRectangleVisibleChannelPart.ul.y=11136-10100; 
    sourceRectangleVisibleChannelPart.lr.x=3000; sourceRectangleVisibleChannelPart.lr.y=11136-2100;
    destinationRectangleVisibleChannelPart.ul.x=0; destinationRectangleVisibleChannelPart.ul.y=0;
    destinationRectangleVisibleChannelPart.lr.x=2000; destinationRectangleVisibleChannelPart.lr.y=8000;

    pTestFilesSMVisibleChannel = SM_StringsAlloc(0); JOE;
    SM_StringsAddToStrings(pTestFilesSMVisibleChannel,
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000001___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000002___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000003___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000004___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000005___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000006___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000007___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000008___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000009___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000010___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000011___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000012___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000013___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000014___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000015___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000016___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000017___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000018___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000019___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000020___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000021___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000022___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000023___-201004191200-C_",
    SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-HRV______-000024___-201004191200-C_",
    NULL); JOE;

    /*SM_StringsDisplay(pTestFilesSM);*/

    /* do processing */

    HritFillImageFromSegments(pTestFilesSMGenericChannel,&sourceRectangleGenericChannelFull,
                              pDestinationImageGenericChannelFull,&destinationRectangleGenericChannelFull);JOE;

    HritFillImageFromSegments(pTestFilesSMGenericChannel,&sourceRectangleGenericChannelPart,
                              pDestinationImageGenericChannelPart,&destinationRectangleGenericChannelPart);JOE;

    HritFillImageFromSegments(pTestFilesSMVisibleChannel,&sourceRectangleVisibleChannelFull,
                              pDestinationImageVisibleChannelFull,&destinationRectangleVisibleChannelFull);JOE;

   HritFillImageFromSegments(pTestFilesSMVisibleChannel,&sourceRectangleVisibleChannelPart,
                              pDestinationImageVisibleChannelPart,&destinationRectangleVisibleChannelPart);JOE;

    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageFromSegments with SM_strings\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromSegmentsSM_stringsGenericChannelFull.rof", pDestinationImageGenericChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromSegmentsSM_stringsGenericChannelPart.rof", pDestinationImageGenericChannelPart);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromSegmentsSM_stringsVisibleChannelFull.rof", pDestinationImageVisibleChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromSegmentsSM_stringsVisibleChannelPart.rof", pDestinationImageVisibleChannelPart);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageFromSegmentsSM_strings: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageFromSegmentsSM_strings");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageFromSegments with SM_strings\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImageVisibleChannelPart);
        IP_ImageAdrFree(&pDestinationImageVisibleChannelFull);
        IP_ImageAdrFree(&pDestinationImageGenericChannelPart);
        IP_ImageAdrFree(&pDestinationImageGenericChannelFull);
    ERROR_CLEAR;
}


/** \brief Test HritFillPrologueStructureFromPrologue(std::iostream&,HritPrologue&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritFillPrologueStructureFromPrologue(int *pReturnValue){
    int debint = 0;
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillPrologueStructureFromPrologue\n";
    #endif

    /* seting up the tests */

    HritPrologue prologue;

    std::fstream testFileStream;

    char testFilePrologue[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-_________-PRO______-201004191200-__.bz2";

    char *pPreProcessedFilePrologue = NULL;
    int4 del_flag=0;

    pPreProcessedFilePrologue = LU_DataPreProcess(testFilePrologue, &del_flag);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillPrologueStructureFromPrologue: LU_DataPreProcess\n";
        JOE;
    }

    testFileStream.open(pPreProcessedFilePrologue, std::fstream::in | std::fstream::out | std::fstream::binary);

    //std::cout << pPreProcessedFilePrologue << "\n";

    /* run the program */

    HritFillPrologueStructureFromPrologue(testFileStream,prologue);JOE; /* if we have an error here -> we cannot pass the test */

    /* close all that is open and not needed */

    testFileStream.close();
    if(del_flag==1)
    {
        FM_FileRemove(pPreProcessedFilePrologue);
        deallocate(pPreProcessedFilePrologue); pPreProcessedFilePrologue=NULL;
        del_flag=0;
    }

    /* do the functional tests */

    if (prologue.imageDescription.plannedCoverageHRV.lowerSouthLinePlanned == 1 &&
        prologue.imageDescription.level1_5ImageProduction.plannedChanProcessing[2] == 2 &&
        prologue.radiometricProcessing.level1_5ImageCalibration[3].cal_Offset == -0.186592010349636849841914454373181797564029693603515625)
    {
        std::cout << "PASSED: HritFillPrologueStructureFromPrologue\n";
    }
    else
    {
        ERROR();
        JOE;
    }


    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillPrologueStructureFromPrologue");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillPrologueStructureFromPrologue\n";
        testFileStream.close();
        if(del_flag==1)
        {
            FM_FileRemove(pPreProcessedFilePrologue);
            deallocate(pPreProcessedFilePrologue); pPreProcessedFilePrologue=NULL;
            del_flag=0;
        }
    ERROR_HANDLER_END;
    ERROR_END;
    ERROR_CLEAR;
}


/** \brief Test HritFillPrologueStructureFromPrologueFile(char*,HritPrologue*)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestHritFillPrologueStructureFromPrologueFile(int *pReturnValue){
    int debint = 0;
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillPrologueStructureFromPrologueFile\n";
    #endif

    /* seting up the tests */

    HritPrologue prologueBz2Zipped;
    HritPrologue prologueUnzipped;

    char testFilePrologueBz2Zipped[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-_________-PRO______-201004191200-__.bz2";
    char testFilePrologueUnzipped[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-_________-PRO______-201004191200-__";

    
    /* run the program */

    HritFillPrologueStructureFromPrologue(testFilePrologueBz2Zipped,&prologueBz2Zipped);JOE; /* if we have an error here -> we cannot pass the test */
    HritFillPrologueStructureFromPrologue(testFilePrologueUnzipped,&prologueUnzipped);JOE; /* if we have an error here -> we cannot pass the test */

    /* do the functional tests */

    if (prologueBz2Zipped.imageDescription.plannedCoverageHRV.lowerSouthLinePlanned == 1 &&
        prologueBz2Zipped.imageDescription.level1_5ImageProduction.plannedChanProcessing[2] == 2 &&
        prologueBz2Zipped.radiometricProcessing.level1_5ImageCalibration[3].cal_Offset == -0.186592010349636849841914454373181797564029693603515625 &&
        prologueUnzipped.imageDescription.plannedCoverageHRV.lowerSouthLinePlanned == 1 &&
        prologueUnzipped.imageDescription.level1_5ImageProduction.plannedChanProcessing[2] == 2 &&
        prologueUnzipped.radiometricProcessing.level1_5ImageCalibration[3].cal_Offset == -0.186592010349636849841914454373181797564029693603515625)
    {
        std::cout << "PASSED: HritFillPrologueStructureFromPrologue from File\n";
    }
    else
    {
        ERROR();
        JOE;
    }


    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillPrologueStructureFromPrologueFile");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillPrologueStructureFromPrologue from File\n";
    ERROR_HANDLER_END;
    ERROR_END;
    ERROR_CLEAR;
}


/** \brief Test NavigateTroughHritHeader(std::iotream&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Spring 2010
 */

void TestNavigateTroughHritHeader(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestNavigateTroughHritHeader\n";
    #endif

    /* seting up the tests */
    std::stringstream testStringStream(std::stringstream::in | std::stringstream::out | std::stringstream::binary);
    std::fstream testFileStream; //(std::fstream::in | std::fstream::out | std::fstream::binary);

    char testFileSegment[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-WV_062___-000001___-201004191200-C_";

    //char testFilePrologue[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-_________-PRO______-201004191200-__.bz2";
    char testFilePrologue[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-_________-EPI______-201004191200-__.bz2" ;
    char *pPreProcessedFilePrologue = NULL;
    int4 del_flag=0;

//     xRITFileDecompress(testFileSegment, testStringStream);
//     if (UNLIKELY(Err.ErrStat))
//     {
//         std::cout << "FAILED WITHIN TEST TestHritFillSegmentHeaderFromSegment: xRITFileDecompress\n";
//         JOE;
//     }
// 
//     NavigateTroughHritHeader(testStringStream);

    pPreProcessedFilePrologue = LU_DataPreProcess(testFilePrologue, &del_flag); JOE;

    testFileStream.open(pPreProcessedFilePrologue, std::fstream::in | std::fstream::out | std::fstream::binary);

    std::cout << pPreProcessedFilePrologue << "\n";


    NavigateTroughHritHeader(testFileStream);

        /*if(del_flag==1)
            {
            FM_FileRemove(file);
            de
(file); file=NULL;
            del_flag=0;
            }*/

    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: NavigateTroughHritHeader\n";
    } 

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestNavigateTroughHritHeader");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: TestNavigateTroughHritHeader\n";
    ERROR_HANDLER_END;
    ERROR_END;
    ERROR_CLEAR;
}


/** \brief Test HritFillImageAndHeaderFromChannel(char*,char*,IP_rectangle&,IP_image&,IP_rectangle&,HritPrologue&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Summer 2010
 */

void TestHritFillImageAndHeaderFromChannel(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageAndHeaderFromChannel\n";
    #endif

    /* setting up the test data */

    char directory[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/";

    /* headers */

    HritSegmentHeader headerDataGenericChannelFull;
    HritSegmentHeader headerDataGenericChannelPart;
    HritSegmentHeader headerDataVisibleChannelFull;
    HritSegmentHeader headerDataVisibleChannelPart;

    /* data for channels with 8 segments */

    char channelGenericChannel[] = "WV_062";

    IP_rectangle sourceRectangleGenericChannelFull;
    IP_rectangle destinationRectangleGenericChannelFull;
    IP_image *pDestinationImageGenericChannelFull = NULL;

    IP_rectangle sourceRectangleGenericChannelPart;
    IP_rectangle destinationRectangleGenericChannelPart;
    IP_image *pDestinationImageGenericChannelPart = NULL;

    /* data for the visible channel with 24 segments */

    char channelVisibleChannel[] = "HRV_";

    IP_rectangle sourceRectangleVisibleChannelFull;
    IP_rectangle destinationRectangleVisibleChannelFull;
    IP_image *pDestinationImageVisibleChannelFull = NULL;

    IP_rectangle sourceRectangleVisibleChannelPart;
    IP_rectangle destinationRectangleVisibleChannelPart;
    IP_image *pDestinationImageVisibleChannelPart = NULL;

    /* init */

    pDestinationImageGenericChannelFull = IP_ImageAlloc(INT2, 3712, 3712); JOE;
    INIT_IP_RECTANGLE(sourceRectangleGenericChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleGenericChannelFull);

    pDestinationImageGenericChannelPart = IP_ImageAlloc(INT2, 1000, 3000); JOE;
    sourceRectangleGenericChannelPart.ul.x=1000; sourceRectangleGenericChannelPart.ul.y=3712-3100; 
    sourceRectangleGenericChannelPart.lr.x=2000; sourceRectangleGenericChannelPart.lr.y=3712-100;
    destinationRectangleGenericChannelPart.ul.x=0; destinationRectangleGenericChannelPart.ul.y=0; 
    destinationRectangleGenericChannelPart.lr.x=1000; destinationRectangleGenericChannelPart.lr.y=3000;


    pDestinationImageVisibleChannelFull = IP_ImageAlloc(INT2, 5568, 11136); JOE;
    INIT_IP_RECTANGLE(sourceRectangleVisibleChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleVisibleChannelFull);

    pDestinationImageVisibleChannelPart = IP_ImageAlloc(INT2, 2000, 8000); JOE;
    sourceRectangleVisibleChannelPart.ul.x=1000; sourceRectangleVisibleChannelPart.ul.y=11136-10100; 
    sourceRectangleVisibleChannelPart.lr.x=3000; sourceRectangleVisibleChannelPart.lr.y=11136-2100;
    destinationRectangleVisibleChannelPart.ul.x=0; destinationRectangleVisibleChannelPart.ul.y=0;
    destinationRectangleVisibleChannelPart.lr.x=2000; destinationRectangleVisibleChannelPart.lr.y=8000;

    /* do processing */

    HritFillImageAndHeaderFromChannel(directory,channelGenericChannel,sourceRectangleGenericChannelFull,
                                      *pDestinationImageGenericChannelFull,destinationRectangleGenericChannelFull,
                                      headerDataGenericChannelFull);JOE;

    HritFillImageAndHeaderFromChannel(directory,channelGenericChannel,sourceRectangleGenericChannelPart,
                                      *pDestinationImageGenericChannelPart,destinationRectangleGenericChannelPart,
                                      headerDataGenericChannelPart);JOE;

    HritFillImageAndHeaderFromChannel(directory,channelVisibleChannel,sourceRectangleVisibleChannelFull,
                                      *pDestinationImageVisibleChannelFull,destinationRectangleVisibleChannelFull,
                                      headerDataVisibleChannelFull);JOE;

    HritFillImageAndHeaderFromChannel(directory,channelVisibleChannel,sourceRectangleVisibleChannelPart,
                                      *pDestinationImageVisibleChannelPart,destinationRectangleVisibleChannelPart,
                                      headerDataVisibleChannelPart);JOE;


    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageAndHeaderFromChannel\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromChannelGenericChannelFull.rof", pDestinationImageGenericChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromChannelGenericChannelPart.rof", pDestinationImageGenericChannelPart);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromChannelVisibleChannelFull.rof", pDestinationImageVisibleChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromChannelVisibleChannelPart.rof", pDestinationImageVisibleChannelPart);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageAndHeaderFromChannel: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageAndHeaderFromChannel");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageAndHeaderFromChannel\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImageVisibleChannelPart);
        IP_ImageAdrFree(&pDestinationImageVisibleChannelFull);
        IP_ImageAdrFree(&pDestinationImageGenericChannelPart);
        IP_ImageAdrFree(&pDestinationImageGenericChannelFull);
    ERROR_CLEAR;
}


/** \brief Test HritFillImageAndHeaderFromChannel(char*,char*,IP_rectangle*,IP_image*,IP_rectangle*,HritPrologue*)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Autumn 2010
 */

void TestHritFillImageAndHeaderFromChannelPointers(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageAndHeaderFromChannelPointers\n";
    #endif

    /* setting up the test data */

    char directory[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/";

    /* headers */

    HritSegmentHeader headerDataGenericChannelFull;
    HritSegmentHeader headerDataGenericChannelPart;
    HritSegmentHeader headerDataVisibleChannelFull;
    HritSegmentHeader headerDataVisibleChannelPart;

    /* data for channels with 8 segments */

    char channelGenericChannel[] = "WV_062";

    IP_rectangle sourceRectangleGenericChannelFull;
    IP_rectangle destinationRectangleGenericChannelFull;
    IP_image *pDestinationImageGenericChannelFull = NULL;

    IP_rectangle sourceRectangleGenericChannelPart;
    IP_rectangle destinationRectangleGenericChannelPart;
    IP_image *pDestinationImageGenericChannelPart = NULL;

    /* data for the visible channel with 24 segments */

    char channelVisibleChannel[] = "HRV_";

    IP_rectangle sourceRectangleVisibleChannelFull;
    IP_rectangle destinationRectangleVisibleChannelFull;
    IP_image *pDestinationImageVisibleChannelFull = NULL;

    IP_rectangle sourceRectangleVisibleChannelPart;
    IP_rectangle destinationRectangleVisibleChannelPart;
    IP_image *pDestinationImageVisibleChannelPart = NULL;

    /* init */

    pDestinationImageGenericChannelFull = IP_ImageAlloc(INT2, 3712, 3712); JOE;
    INIT_IP_RECTANGLE(sourceRectangleGenericChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleGenericChannelFull);

    pDestinationImageGenericChannelPart = IP_ImageAlloc(INT2, 1000, 3000); JOE;
    sourceRectangleGenericChannelPart.ul.x=1000; sourceRectangleGenericChannelPart.ul.y=3712-3100; 
    sourceRectangleGenericChannelPart.lr.x=2000; sourceRectangleGenericChannelPart.lr.y=3712-100;
    destinationRectangleGenericChannelPart.ul.x=0; destinationRectangleGenericChannelPart.ul.y=0; 
    destinationRectangleGenericChannelPart.lr.x=1000; destinationRectangleGenericChannelPart.lr.y=3000;


    pDestinationImageVisibleChannelFull = IP_ImageAlloc(INT2, 5568, 11136); JOE;
    INIT_IP_RECTANGLE(sourceRectangleVisibleChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleVisibleChannelFull);

    pDestinationImageVisibleChannelPart = IP_ImageAlloc(INT2, 2000, 8000); JOE;
    sourceRectangleVisibleChannelPart.ul.x=1000; sourceRectangleVisibleChannelPart.ul.y=11136-10100; 
    sourceRectangleVisibleChannelPart.lr.x=3000; sourceRectangleVisibleChannelPart.lr.y=11136-2100;
    destinationRectangleVisibleChannelPart.ul.x=0; destinationRectangleVisibleChannelPart.ul.y=0;
    destinationRectangleVisibleChannelPart.lr.x=2000; destinationRectangleVisibleChannelPart.lr.y=8000;

    /* do processing */

    HritFillImageAndHeaderFromChannel(directory,channelGenericChannel,&sourceRectangleGenericChannelFull,
                                      pDestinationImageGenericChannelFull,&destinationRectangleGenericChannelFull,
                                      &headerDataGenericChannelFull);JOE;

    HritFillImageAndHeaderFromChannel(directory,channelGenericChannel,&sourceRectangleGenericChannelPart,
                                      pDestinationImageGenericChannelPart,&destinationRectangleGenericChannelPart,
                                      &headerDataGenericChannelPart);JOE;

    HritFillImageAndHeaderFromChannel(directory,channelVisibleChannel,&sourceRectangleVisibleChannelFull,
                                      pDestinationImageVisibleChannelFull,&destinationRectangleVisibleChannelFull,
                                      &headerDataVisibleChannelFull);JOE;

    HritFillImageAndHeaderFromChannel(directory,channelVisibleChannel,&sourceRectangleVisibleChannelPart,
                                      pDestinationImageVisibleChannelPart,&destinationRectangleVisibleChannelPart,
                                      &headerDataVisibleChannelPart);JOE;


    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageAndHeaderFromChannel with Pointers\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromChannelPointersGenericChannelFull.rof", pDestinationImageGenericChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromChannelPointersGenericChannelPart.rof", pDestinationImageGenericChannelPart);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromChannelPointersVisibleChannelFull.rof", pDestinationImageVisibleChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageAndHeaderFromChannelPointersVisibleChannelPart.rof", pDestinationImageVisibleChannelPart);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageAndHeaderFromChannel with Pointers: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageAndHeaderFromChannelPointers");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageAndHeaderFromChannel with Pointers\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImageVisibleChannelPart);
        IP_ImageAdrFree(&pDestinationImageVisibleChannelFull);
        IP_ImageAdrFree(&pDestinationImageGenericChannelPart);
        IP_ImageAdrFree(&pDestinationImageGenericChannelFull);
    ERROR_CLEAR;
}


/** \brief Test HritFillImageFromChannel(char*,char*,IP_rectangle&,IP_image&,IP_rectangle&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Autumn 2010
 */

void TestHritFillImageFromChannel(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageFromChannel\n";
    #endif

    /* setting up the test data */

    char directory[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/";

    /* data for channels with 8 segments */

    char channelGenericChannel[] = "WV_062";

    IP_rectangle sourceRectangleGenericChannelFull;
    IP_rectangle destinationRectangleGenericChannelFull;
    IP_image *pDestinationImageGenericChannelFull = NULL;

    IP_rectangle sourceRectangleGenericChannelPart;
    IP_rectangle destinationRectangleGenericChannelPart;
    IP_image *pDestinationImageGenericChannelPart = NULL;

    /* data for the visible channel with 24 segments */

    char channelVisibleChannel[] = "HRV_";

    IP_rectangle sourceRectangleVisibleChannelFull;
    IP_rectangle destinationRectangleVisibleChannelFull;
    IP_image *pDestinationImageVisibleChannelFull = NULL;

    IP_rectangle sourceRectangleVisibleChannelPart;
    IP_rectangle destinationRectangleVisibleChannelPart;
    IP_image *pDestinationImageVisibleChannelPart = NULL;

    /* init */

    pDestinationImageGenericChannelFull = IP_ImageAlloc(INT2, 3712, 3712); JOE;
    INIT_IP_RECTANGLE(sourceRectangleGenericChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleGenericChannelFull);

    pDestinationImageGenericChannelPart = IP_ImageAlloc(INT2, 1000, 3000); JOE;
    sourceRectangleGenericChannelPart.ul.x=1000; sourceRectangleGenericChannelPart.ul.y=3712-3100; 
    sourceRectangleGenericChannelPart.lr.x=2000; sourceRectangleGenericChannelPart.lr.y=3712-100;
    destinationRectangleGenericChannelPart.ul.x=0; destinationRectangleGenericChannelPart.ul.y=0; 
    destinationRectangleGenericChannelPart.lr.x=1000; destinationRectangleGenericChannelPart.lr.y=3000;


    pDestinationImageVisibleChannelFull = IP_ImageAlloc(INT2, 5568, 11136); JOE;
    INIT_IP_RECTANGLE(sourceRectangleVisibleChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleVisibleChannelFull);

    pDestinationImageVisibleChannelPart = IP_ImageAlloc(INT2, 2000, 8000); JOE;
    sourceRectangleVisibleChannelPart.ul.x=1000; sourceRectangleVisibleChannelPart.ul.y=11136-10100; 
    sourceRectangleVisibleChannelPart.lr.x=3000; sourceRectangleVisibleChannelPart.lr.y=11136-2100;
    destinationRectangleVisibleChannelPart.ul.x=0; destinationRectangleVisibleChannelPart.ul.y=0;
    destinationRectangleVisibleChannelPart.lr.x=2000; destinationRectangleVisibleChannelPart.lr.y=8000;

    /* do processing */

    HritFillImageFromChannel(directory,channelGenericChannel,sourceRectangleGenericChannelFull,
                             *pDestinationImageGenericChannelFull,destinationRectangleGenericChannelFull);JOE;

    HritFillImageFromChannel(directory,channelGenericChannel,sourceRectangleGenericChannelPart,
                             *pDestinationImageGenericChannelPart,destinationRectangleGenericChannelPart);JOE;

    HritFillImageFromChannel(directory,channelVisibleChannel,sourceRectangleVisibleChannelFull,
                             *pDestinationImageVisibleChannelFull,destinationRectangleVisibleChannelFull);JOE;

    HritFillImageFromChannel(directory,channelVisibleChannel,sourceRectangleVisibleChannelPart,
                             *pDestinationImageVisibleChannelPart,destinationRectangleVisibleChannelPart);JOE;


    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageFromChannel\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromChannelGenericChannelFull.rof", pDestinationImageGenericChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromChannelGenericChannelPart.rof", pDestinationImageGenericChannelPart);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromChannelVisibleChannelFull.rof", pDestinationImageVisibleChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromChannelVisibleChannelPart.rof", pDestinationImageVisibleChannelPart);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageFromChannel: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageFromChannel");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageFromChannel\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImageVisibleChannelPart);
        IP_ImageAdrFree(&pDestinationImageVisibleChannelFull);
        IP_ImageAdrFree(&pDestinationImageGenericChannelPart);
        IP_ImageAdrFree(&pDestinationImageGenericChannelFull);
    ERROR_CLEAR;
}


/** \brief Test HritFillImageFromChannel(char*,char*,IP_rectangle*,IP_image*,IP_rectangle*,HritPrologue*)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Autumn 2010
 */

void TestHritFillImageFromChannelPointers(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageFromChannelPointers\n";
    #endif

    /* setting up the test data */

    char directory[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/";

    /* data for channels with 8 segments */

    char channelGenericChannel[] = "WV_062";

    IP_rectangle sourceRectangleGenericChannelFull;
    IP_rectangle destinationRectangleGenericChannelFull;
    IP_image *pDestinationImageGenericChannelFull = NULL;

    IP_rectangle sourceRectangleGenericChannelPart;
    IP_rectangle destinationRectangleGenericChannelPart;
    IP_image *pDestinationImageGenericChannelPart = NULL;

    /* data for the visible channel with 24 segments */

    char channelVisibleChannel[] = "HRV_";

    IP_rectangle sourceRectangleVisibleChannelFull;
    IP_rectangle destinationRectangleVisibleChannelFull;
    IP_image *pDestinationImageVisibleChannelFull = NULL;

    IP_rectangle sourceRectangleVisibleChannelPart;
    IP_rectangle destinationRectangleVisibleChannelPart;
    IP_image *pDestinationImageVisibleChannelPart = NULL;

    /* init */

    pDestinationImageGenericChannelFull = IP_ImageAlloc(INT2, 3712, 3712); JOE;
    INIT_IP_RECTANGLE(sourceRectangleGenericChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleGenericChannelFull);

    pDestinationImageGenericChannelPart = IP_ImageAlloc(INT2, 1000, 3000); JOE;
    sourceRectangleGenericChannelPart.ul.x=1000; sourceRectangleGenericChannelPart.ul.y=3712-3100; 
    sourceRectangleGenericChannelPart.lr.x=2000; sourceRectangleGenericChannelPart.lr.y=3712-100;
    destinationRectangleGenericChannelPart.ul.x=0; destinationRectangleGenericChannelPart.ul.y=0; 
    destinationRectangleGenericChannelPart.lr.x=1000; destinationRectangleGenericChannelPart.lr.y=3000;


    pDestinationImageVisibleChannelFull = IP_ImageAlloc(INT2, 5568, 11136); JOE;
    INIT_IP_RECTANGLE(sourceRectangleVisibleChannelFull);
    INIT_IP_RECTANGLE(destinationRectangleVisibleChannelFull);

    pDestinationImageVisibleChannelPart = IP_ImageAlloc(INT2, 2000, 8000); JOE;
    sourceRectangleVisibleChannelPart.ul.x=1000; sourceRectangleVisibleChannelPart.ul.y=11136-10100; 
    sourceRectangleVisibleChannelPart.lr.x=3000; sourceRectangleVisibleChannelPart.lr.y=11136-2100;
    destinationRectangleVisibleChannelPart.ul.x=0; destinationRectangleVisibleChannelPart.ul.y=0;
    destinationRectangleVisibleChannelPart.lr.x=2000; destinationRectangleVisibleChannelPart.lr.y=8000;

    /* do processing */

    HritFillImageFromChannel(directory,channelGenericChannel,&sourceRectangleGenericChannelFull,
                             pDestinationImageGenericChannelFull,&destinationRectangleGenericChannelFull);JOE;

    HritFillImageFromChannel(directory,channelGenericChannel,&sourceRectangleGenericChannelPart,
                             pDestinationImageGenericChannelPart,&destinationRectangleGenericChannelPart);JOE;

    HritFillImageFromChannel(directory,channelVisibleChannel,&sourceRectangleVisibleChannelFull,
                             pDestinationImageVisibleChannelFull,&destinationRectangleVisibleChannelFull);JOE;

    HritFillImageFromChannel(directory,channelVisibleChannel,&sourceRectangleVisibleChannelPart,
                             pDestinationImageVisibleChannelPart,&destinationRectangleVisibleChannelPart);JOE;


    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageFromChannel with Pointers\n";
    } 

    #ifdef VISUALTESTS
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromChannelPointersGenericChannelFull.rof", pDestinationImageGenericChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromChannelPointersGenericChannelPart.rof", pDestinationImageGenericChannelPart);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromChannelPointersVisibleChannelFull.rof", pDestinationImageVisibleChannelFull);
    IP_ImageWrite((char *)SRCDIR "/Output/outputTestHritFillImageFromChannelPointersVisibleChannelPart.rof", pDestinationImageVisibleChannelPart);
    if (UNLIKELY(Err.ErrStat))
    {
        std::cout << "FAILED WITHIN TEST TestHritFillImageFromChannel with Pointers: IP_ImageWrite\n";
        JOE;
    }
    #endif

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageFromChannelPointers");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageFromChannel with Pointers\n";
    ERROR_HANDLER_END;
    ERROR_END;
        IP_ImageAdrFree(&pDestinationImageVisibleChannelPart);
        IP_ImageAdrFree(&pDestinationImageVisibleChannelFull);
        IP_ImageAdrFree(&pDestinationImageGenericChannelPart);
        IP_ImageAdrFree(&pDestinationImageGenericChannelFull);
    ERROR_CLEAR;
}


/** \brief Test HritConvertSeviriChannelNumberToChannel(int4)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Autumn 2010
 */

void TestHritConvertSeviriChannelNumberToChannel(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritConvertSeviriChannelNumberToChannel\n";
    #endif

    int4 outOfRangeTestFailed = true;
    int4 functionalTestFailed = true;

    /* Error handler tests */

    if (HritConvertSeviriChannelNumberToChannel(12) != NULL)
    {
        outOfRangeTestFailed =false;
    }
    if (UNLIKELY(Err.ErrStat))
    {
        outOfRangeTestFailed = false;
        ERROR_CLEAR;
    }

    if (HritConvertSeviriChannelNumberToChannel(-1) != NULL)
    {
        outOfRangeTestFailed =false;
    }
    if (UNLIKELY(Err.ErrStat))
    {
        outOfRangeTestFailed = false;
        ERROR_CLEAR;
    }

    /* functional tests MEMORY LEAK HERE, every call here allocates but is never reclaimed*/
    functionalTestFailed =  outOfRangeTestFailed ||
                            strcmp(HritConvertSeviriChannelNumberToChannel(0), "HRV")    || /* When one of these is not equal we have a non 0 (true) 
                                                                                            and hence they are all true */
                            strcmp(HritConvertSeviriChannelNumberToChannel(1), "VIS006") ||
                            strcmp(HritConvertSeviriChannelNumberToChannel(2), "VIS008") ||
                            strcmp(HritConvertSeviriChannelNumberToChannel(3), "IR_016") ||
                            strcmp(HritConvertSeviriChannelNumberToChannel(4), "IR_039") ||
                            strcmp(HritConvertSeviriChannelNumberToChannel(5), "WV_062") ||
                            strcmp(HritConvertSeviriChannelNumberToChannel(6), "WV_073") ||
                            strcmp(HritConvertSeviriChannelNumberToChannel(7), "IR_087") ||
                            strcmp(HritConvertSeviriChannelNumberToChannel(8), "IR_097") ||
                            strcmp(HritConvertSeviriChannelNumberToChannel(9), "IR_108") ||
                            strcmp(HritConvertSeviriChannelNumberToChannel(10), "IR_120")||
                            strcmp(HritConvertSeviriChannelNumberToChannel(11), "IR_134");JOE;

    if (functionalTestFailed)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritConvertSeviriChannelNumberToChannel\n";
    } 

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritConvertSeviriChannelNumberToChannel");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritConvertSeviriChannelNumberToChannel\n";
    ERROR_HANDLER_END;
    ERROR_END;
    ERROR_CLEAR;
}


/** \brief Test HritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel(char*,char*,char*,int4,IP_rectangle*,IP_image*,IP_rectangle*,HritSegmentHeader*)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Autumn 2010
 */

void TestHritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel\n";
    #endif

    /* setting up the test data */

    char baseDirectory[] = SRCDIR "/TestData/";
    char satellite[] = "MSG1";
    char slot[] = "200406220000";
    int4 channelNumber = 2;
    IP_rectangle sourceRectangle;
    IP_rectangle destinationRectangle;
    IP_image *pDestinationImage = NULL;
    HritSegmentHeader headerData;


    pDestinationImage = IP_ImageAlloc(INT2, 3712, 3712); JOE;
    INIT_IP_RECTANGLE(sourceRectangle);
    INIT_IP_RECTANGLE(destinationRectangle);

    HritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel(baseDirectory, satellite, slot, channelNumber, &sourceRectangle, 
                                                              pDestinationImage, &destinationRectangle, &headerData);JOE;

    if (false /* functional tests */)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel\n";
    } 

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel\n";
    ERROR_HANDLER_END;
    ERROR_END;
    ERROR_CLEAR;
}


/** \brief Test HritGenerateSlotDirectoryName(char*,char*, char*)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Autumn 2010
 */

void TestHritGenerateSlotDirectoryName(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestTestHritGenerateSlotDirectoryName\n";
    #endif

    int4 nullTestFailed = true;
    int4 functionalTestFailed = true;

    /* setting up the test data */

    char baseDirectory[] = SRCDIR "/TestData/";
    char satellite[] = "MSG1";
    char slot[] = "200406220000";

    if (HritGenerateSlotDirectoryName(baseDirectory, satellite, NULL) != NULL)
    {
        nullTestFailed =false;
    }
    if (UNLIKELY(Err.ErrStat))
    {
        nullTestFailed = false;
        ERROR_CLEAR;
    }

    /* functional tests MEMORY LEAK HERE, every call here allocates but is never reclaimed*/
    functionalTestFailed =  nullTestFailed ||
                            strcmp(HritGenerateSlotDirectoryName(baseDirectory, satellite, slot),
                            SRCDIR "/TestData/MSG1_20040622/HRIT_200406220000/");JOE;

    if (functionalTestFailed)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritGenerateSlotDirectoryName\n";
    } 

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritGenerateSlotDirectoryName");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritGenerateSlotDirectoryName\n";
    ERROR_HANDLER_END;
    ERROR_END;
    ERROR_CLEAR;
}


/** \brief Test HritGeneratePrologueLocation(char*,char*, char*)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Autumn 2010
 */

void TestHritGeneratePrologueLocation(int *pReturnValue){
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritGeneratePrologueLocation\n";
    #endif

    char* functionalTestReturnChar = 0;

    int4 nullTestFailed = true;
    int4 functionalTestFailed = true;

    /* setting up the test data */

    char baseDirectory[] = SRCDIR "/TestData/";
    char satellite[] = "MSG1";
    char slot[] = "200406220000";

    if (HritGeneratePrologueLocation(baseDirectory, satellite, NULL) != NULL)
    {
        nullTestFailed =false;
    }
    if (UNLIKELY(Err.ErrStat))
    {
        nullTestFailed = false;
        ERROR_CLEAR;
    }

    functionalTestReturnChar = HritGeneratePrologueLocation(baseDirectory, satellite, slot);JOE;
    functionalTestFailed =  nullTestFailed ||
                            (functionalTestReturnChar == NULL) ||
                            strcmp(functionalTestReturnChar,
                            SRCDIR "/TestData/MSG1_20040622/HRIT_200406220000/H-000-MSG1__-MSG1________-_________-PRO______-200406220000-__");JOE;

    if (functionalTestFailed)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: HritGeneratePrologueLocation\n";
    } 

    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritGeneratePrologueLocation");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritGeneratePrologueLocation\n";
    ERROR_HANDLER_END;
        deallocate(functionalTestReturnChar); functionalTestReturnChar=NULL;
    ERROR_END;
    ERROR_CLEAR;
}


/** \brief Test HritFillPrologueStructureFromBaseDirSatelliteSlotAndChannel(char*,char*, char*, HritPrologue&)
 * \param pReturnValue pointer to the returnvalue, add 1 if a failure occurs
 * 
 * \author Stijn Nevens (RMIB)
 * \date Summer 2011
 */

void TestHritFillPrologueStructureFromBaseDirSatelliteSlotAndChannel(int *pReturnValue){
    int debint = 0;
    #ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering TestHritFillPrologueStructureFromBaseDirSatelliteSlotAndChannel\n";
    #endif

    /* seting up the tests */

    HritPrologue prologue;

    char testFilePrologue[] = SRCDIR "/TestData/MSG2_20100419/HRIT_201004191200/H-000-MSG2__-MSG2________-_________-PRO______-201004191200-__.bz2";

    /* run the program */

    HritFillPrologueStructureFromBaseDirSatelliteSlotAndChannel(SRCDIR "/TestData/","MSG1","200406220000",prologue);JOE; /* if we have an error here -> we cannot pass the test */

    /* do the functional tests */

    if (prologue.imageDescription.plannedCoverageHRV.lowerSouthLinePlanned == 1 &&
        prologue.imageDescription.level1_5ImageProduction.plannedChanProcessing[2] == 1 &&
        prologue.radiometricProcessing.level1_5ImageCalibration[3].cal_Offset == -0.1865920000000000078532735869885073043406009674072265625)
    {
        std::cout << "PASSED: \n";
    }
    else
    {
        ERROR();
        JOE;
    }


    /* Error handling */
    ERROR_HANDLER_BEGIN("TestHritFillPrologueStructureFromBaseDirSatelliteSlotAndChannel");
        *pReturnValue += 1; /* add to the returnvalue */
        std::cout << "FAILED: HritFillPrologueStructureFromBaseDirSatelliteSlotAndChannel\n";
    ERROR_HANDLER_END;
    ERROR_END;
    ERROR_CLEAR;
}

/** \brief Test compatibility with the prologue generated by MSG1 IODC (MSG-1 over the Indian Ocean)
 * \param pReturnValue pointer to the return value, add 1 if failure occurs
 *
 * \author Edward Baudrez (RMIB)
 * \date Winter 2020
 */
void TestIODCHritGeneratePrologueLocation(int *pReturnValue)
{
#ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering " << __func__ << "\n";
#endif

    char* functionalTestReturnChar = 0;

    int4 functionalTestFailed = true;

    /* setting up the test data */
    char baseDirectory[] = SRCDIR "/TestData/";
    char satellite[] = "MSG1";
    char slot[] = "201706051845";
    char product_id1[] = "MSG1_IODC";

    functionalTestReturnChar = HritGeneratePrologueLocationWithProductID1(baseDirectory, satellite, slot, product_id1); JOE;
    functionalTestFailed =  (functionalTestReturnChar == NULL) ||
                            strcmp(functionalTestReturnChar,
                            SRCDIR "/TestData/MSG1_20170605/HRIT_201706051845/H-000-MSG1__-MSG1_IODC___-_________-PRO______-201706051845-__.bz2");JOE;

    if (functionalTestFailed)
    {
        ERROR();
        JOE;
    }
    else
    {
        std::cout << "PASSED: " << __func__ << "\n";
    }

    ERROR_HANDLER_BEGIN(__func__)
    *pReturnValue += 1;
    std::cout << "FAILED: " << __func__ << "\n";
    ERROR_HANDLER_END
    deallocate(functionalTestReturnChar);
    ERROR_END
    ERROR_CLEAR;
}

/** \brief Test compatibility with filenames generated by MSG1 IODC (MSG-1 over the Indian Ocean)
 * \param pReturnValue pointer to the return value, add 1 if failure occurs
 *
 * \author Edward Baudrez (RMIB)
 * \date Winter 2020
 */
void TestIODCHritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannel(int *pReturnValue)
{
#ifdef DEBUG2
    std::cout << "***************************************************************\n";
    std::cout << "Entering " << __func__ << "\n";
#endif

    /* setting up the test data */
    char baseDirectory[] = SRCDIR "/TestData/";
    char satellite[] = "MSG1";
    char product_id1[] = "MSG1_IODC";
    char slot[] = "201706051845";
    int4 channelNumber = 2;
    IP_rectangle sourceRectangle;
    IP_rectangle destinationRectangle;
    IP_image *pDestinationImage = NULL;
    HritSegmentHeader headerData;

    pDestinationImage = IP_ImageAlloc(INT2, 3712, 3712); JOE;
    INIT_IP_RECTANGLE(sourceRectangle);
    INIT_IP_RECTANGLE(destinationRectangle);

    HritFillImageAndHeaderFromBaseDirSatelliteSlotAndChannelWithProductID1(baseDirectory, satellite, product_id1, slot,
                                                                           channelNumber, &sourceRectangle, pDestinationImage,
                                                                           &destinationRectangle, &headerData); JOE;

    std::cout << "PASSED: " << __func__ << "\n";

    ERROR_HANDLER_BEGIN(__func__)
    *pReturnValue += 1;
    std::cout << "FAILED: " << __func__ << "\n";
    ERROR_HANDLER_END
    ERROR_END
    IP_ImageFree(pDestinationImage);
    ERROR_CLEAR;
}
