#include <config.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "ErrorHandling.h"			// Util
#include "CxRITFile.h"				// DISE
#include "CxRITFileDecompressed.h"	// DISE

#ifdef XRIT_USE_SMALL_LIB
#include "rmibsmalllib.h"
#else
#include "const.h"
#include "image.h"
#include "strlist.h"
#include "comline.h"
#include "err_mng.h"
#include "unix_file.h"
#include "util.h"
#include "verbose.h"
#include "allocate.h"
#include "stringut.h"
#include "gerbtime.h"
#include "byte_swap.h"
#include "file_mng.h"
#include "data_preproc.h"
#endif

/*HELP ========================================================

   Concatenation of  HRIT segments to create
   a XPIF image. The HRIT segments given in argument should
   belong to an unique spectral band (e.g. VIS006, IR_120, ...) and
   a time slot (e.g. 200410250745). The repeat cycle prologue
   (PRO) segment for this time slot should also be given because
   it contains the calibration as well as the HRV navigation
   information.

   This version of the program supports compressed as well as
   uncompressed HRIT segments as input.

   USAGE:
     hrit2xpif   {HRIT_segments}  {HRIT pro}  [-o out.XPIF] [options]


   OPTIONS:
    -h: display this help
    -v: verbose mode
    -hrv_top: create HRV image that only contains the upper part.

   AUTHOR: Nicolas Clerbaux -- RMIB
           Alessandro Ipe -- RMIB

   ============================================================ */

char  *spectral_band_id[12] = {"VIS006___","VIS008___","IR_016___","IR_039___","WV_062___","WV_073___","IR_087___","IR_097___",\
                             "IR_108___","IR_120___","IR_134___","HRV______"};

typedef struct {
                /* LowerSouthLinePlanned, LowerNorthLinePlanned, LowerEastColumnPlanned, LowerWestColumnPlanned
                   UpperSouthLinePlanned, UpperNorthLinePlanned, UpperEastColumnPlanned, UpperWestColumnPlanned  */
                u_int4        HRV_PlannedCoverage[8];

                double      cal_slope[12];
                double      cal_offset[12];

                u_c        planned_chan_processing[12];
               } HRIT_pro ;



/* information neaded to fill the primary PXOUF header record */
typedef struct {
                int4 nav_func;
                int4 nav_loff;
                int4 nav_coff;
                int4 nav_lres;
                int4 nav_cres;


                int4 nav_goff_lon;
                int4 nav_goff_lat;

                int4 nav_split_line;
                int4 nav_split_coff;

                double nav_loff_real;
                double nav_coff_real;
                double nav_lres_real;
                double nav_cres_real;

                double      src_msg_cals;
                double      src_msg_calo;

                int4     pif_nb;  /* number of bits per pixels */

                u_c         dat_f;
                char        dat_nam[17];
                char        dat_enh[17];
                char        src_obj[17];

                int2        src_year;
                int2        src_day;
                int2        src_hour;

                u_c         planned_chan_processing;

               } XPIF_header ;



typedef struct {
               int4      num_bytes;
               u_c       *data;
             } XPIF_tar_data ;


#define NUM_LINE_SEGMENT                 464
#define HRIT_SEGMENT_HEADER_LENGTH      (163+3+NUM_LINE_SEGMENT*13)

#define INTEGER_DIVISION(a,b)             (((a)%(b)) ? ((a)/(b) + 1) : ((a)/(b)) )

HRIT_pro PRO_Read(char *file);

void xRITFileDecompress(char *filename, std::stringstream& strstream);

void HRIT_SegmentNameAnalysis(char *name, int4 *sb, int4 *seg_id, char *slot);

void HRIT_ReadImageDataSegment(std::stringstream &, IP_image *im, XPIF_tar_data *xpif_tar_data);

void IP_ImageWrite_XPIF(char *filename, IP_image *im, XPIF_header *xpif_header , XPIF_tar_data *xpif_tar_data);

void Unpack4pix10bit(unsigned char *data, unsigned short *a);

void help_hrit2xpif(void);



void xRITFileDecompress(char *filename, std::stringstream& strstream)
{
    DISE::CxRITFile compressedFile;


    // Open input file
    try
	{
        compressedFile = DISE::CxRITFile(filename);
	}
	catch (...)
	{
		std::cerr << "Opening/reading input file failed\n";
        exit(1);
	}

	// Decompress input file
	DISE::CxRITFileDecompressed decompressedFile(compressedFile);

    // Write decompressed file
    decompressedFile.Write(strstream);
}



/* extrenal variable for the sake of simplicity .... */
int4    ext_nav_flag=0, ext_nav_cfac=0, ext_nav_lfac=0, ext_nav_coff=0, ext_nav_loff=0;
int4    ext_hrv_flag=0;
int main(int argc, char **argv)
{
int4                rc = 0, k, w=0, h=0, sb=-1, sb_segment=0, pro_flag=0, n_segment, seg_id, ret, del_flag=0;
int4                year, month, day, hour, minute;
SM_strings          *list=NULL;
char                *file=NULL, *slot=NULL, slot_segment[512];
HRIT_pro            pro;
XPIF_header         xpif_header;
IP_image            *im=NULL;
XPIF_tar_data       xpif_tar_data;


	CL_Init(argc, argv);

	if(argc < 1 || CL_Opt('h'))
       {
        help_hrit2xpif();
        TERMINATE;
        }

    xpif_tar_data.num_bytes = 0;
    xpif_tar_data.data      = NULL;


   /* Get the list of HRIT segments in the command line and loop on them */
    list = CL_GetStrings(1); JOE;

    for(k=0;k<list->num;k++)
       {
        std::stringstream strstream(std::stringstream::in | std::stringstream::out | std::stringstream::binary);

        ret = FM_FileExistence(list->str[k]);
        if(ret==0)
           {
            ErrorPrintf("Missing segment : %s\n", list->str[k]);
            continue;
           }

       file = LU_DataPreProcess(list->str[k], &del_flag); JOE; ;

       HRIT_SegmentNameAnalysis(file, &sb_segment, &seg_id, slot_segment); JOE;

       if(slot==NULL)
         {
          slot = StringCopy(slot_segment);
          LU_Verbose("slot is : %s\n", slot);
         }

       ERROR_IF_M2(strcmp(slot, slot_segment),"ERROR : segment does not belong to slot %s : %s", slot, file)

       switch(seg_id)
           {
            case -1 :  LU_Verbose("Prologue segment : %s\n", file);
                       pro_flag=1;
                       pro = PRO_Read(file); JOE;
                       break;

            case -2 : LU_Verbose("Epilogue segment : %s\n", file);
                      break;

            case -3 : ErrorPrintf("don't know what top do with %s\n", file);
                      break;

            default:
                   if(sb==-1)
                      {
                       sb = sb_segment;
                       LU_Verbose("spectral band is : %s\n", spectral_band_id[sb]);
                       w = 3712;
                       h = 3712;
                       ext_hrv_flag = 0;
                       if(sb==11)
                           {
                            ext_hrv_flag = 1;
                            w =  5568;
                            h = 11136;
                           }
                       LU_Verbose("Allocating INT2 image (width = %d, height = %d)  \n", w, h);
                       im = IP_ImageAlloc(INT2, w, h); JOE;

                       n_segment = h/NUM_LINE_SEGMENT;
                       LU_Verbose("Expected segments number is %d\n", n_segment);

                       xpif_tar_data.num_bytes = n_segment * HRIT_SEGMENT_HEADER_LENGTH ;
                       xpif_tar_data.data      = (u_c *) allocate( xpif_tar_data.num_bytes );
                      }

                    ERROR_IF_M2(sb_segment!=sb,"ERROR : segment does not belong to spectral band %s : %s",  spectral_band_id[sb], file)

                    LU_Verbose("Standard segment : %s  \n", file);

                    xRITFileDecompress(file, strstream);

                    HRIT_ReadImageDataSegment(strstream, im, &xpif_tar_data );
                    break;

           }

         if(del_flag==1)
            {
             FM_FileRemove(file);
             deallocate(file); file=NULL;
             del_flag=0;
            }
       }   /* end loop on the segments */


   ERROR_IF_M(sb==-1,"No Image Data File provided (->no image generated)");

  /* XPIF header information */
   xpif_header.pif_nb         = 10;

   xpif_header.nav_func       = 42;
   xpif_header.nav_goff_lon   =  0;
   xpif_header.nav_goff_lat   =  0;
   xpif_header.nav_split_line =  0;
   xpif_header.nav_split_coff =  0;



  /* printf("Navigation is %d %d %d %d \n", ext_nav_cfac, ext_nav_lfac, ext_nav_loff, ext_nav_coff);*/

   xpif_header.nav_loff_real = (double) ext_nav_loff;
   xpif_header.nav_coff_real = (double) ext_nav_coff;
   xpif_header.nav_split_line = 0 ;
   xpif_header.nav_split_coff = 0;

   /* the big number hereafter is:  - 2^16 / delta (in degree) = - 2^16 * 10^6 * 50 / 360 */
   xpif_header.nav_lres_real = - 9102222222.2222222222222222222 / ext_nav_lfac;
   xpif_header.nav_cres_real = - 9102222222.2222222222222222222 / ext_nav_cfac;

   if(sb==11 && pro_flag)
       {
        xpif_header.nav_split_line =   11136 - (int4) pro.HRV_PlannedCoverage[1]  ;                /* LowerNorthLinePlanned*/
        xpif_header.nav_coff_real  =            (double)pro.HRV_PlannedCoverage[7] - 11136 / 2  ;   /* UpperWestColumnPlanned */
        xpif_header.nav_split_coff =  (int4)   ((double)pro.HRV_PlannedCoverage[3] - 11136 / 2  );  /* LowerWestColumnPlanned */
       }

   xpif_header.nav_loff      = (int4) xpif_header.nav_loff_real;
   xpif_header.nav_coff      = (int4) xpif_header.nav_coff_real;
   xpif_header.nav_lres      = (int4) xpif_header.nav_lres_real;
   xpif_header.nav_cres      = (int4) xpif_header.nav_cres_real;

   if(pro_flag)
     {
       xpif_header.src_msg_cals =  pro.cal_slope[sb];
       xpif_header.src_msg_calo =  pro.cal_offset[sb];
       xpif_header.planned_chan_processing = pro.planned_chan_processing[sb];
      }
   else
     {
       ErrorPrintf("No prologue provided \n");
       ErrorPrintf("   -> no calibration in the XPIF file\n");
       xpif_header.src_msg_cals =   1.0;
       xpif_header.src_msg_calo =   0.0;
       if(sb==11)  ErrorPrintf("   -> no HRV naviagation in the XPIF file\n");
      }


   /* other header information */

      xpif_header.dat_f = 20;
      switch(sb)
         {
          case  0: sprintf(xpif_header.dat_nam,"VIS_0_6_RAW     "); break;
          case  1: sprintf(xpif_header.dat_nam,"VIS_0_8_RAW     "); break;
          case  2: sprintf(xpif_header.dat_nam,"IR_1_6_RAW      "); break;
          case  3: sprintf(xpif_header.dat_nam,"IR_3_9_RAW      "); break;
          case  4: sprintf(xpif_header.dat_nam,"WV_6_2_RAW      "); break;
          case  5: sprintf(xpif_header.dat_nam,"WV_7_3_RAW      "); break;
          case  6: sprintf(xpif_header.dat_nam,"IR_8_7_RAW      "); break;
          case  7: sprintf(xpif_header.dat_nam,"IR_9_7_RAW      "); break;
          case  8: sprintf(xpif_header.dat_nam,"IR_10_8_RAW     "); break;
          case  9: sprintf(xpif_header.dat_nam,"IR_12_0_RAW     "); break;
          case 10: sprintf(xpif_header.dat_nam,"IR_13_4_RAW     "); break;
          case 11: sprintf(xpif_header.dat_nam,"HRV_RAW         "); break;
          default : ERROR_M("undefined SB");
         }

      sprintf(xpif_header.dat_enh,"DEFAULT        ");
      sprintf(xpif_header.src_obj,"MSG1________");

      year   = SM_StringExtractInt(slot, 0, 4); JOE;
      month  = SM_StringExtractInt(slot, 4, 6); JOE;
      day    = SM_StringExtractInt(slot, 6, 8); JOE;
      hour   = SM_StringExtractInt(slot, 8, 10); JOE;
      minute = SM_StringExtractInt(slot, 10, 12); JOE;

      xpif_header.src_year = (int2) year;
      xpif_header.src_day  = (int2) ShortJulianDayFromMonthAndDay( year, month, day); JOE;
      xpif_header.src_hour = (int2)( hour * 100 + minute);


    if(sb==11 && CL_Sopt("-hrv_top"))
       {
        IP_image        *tmp_im=NULL;
        IP_rectangle    rec;

        rec.ul.x = 0;
        rec.ul.y = 0;
        rec.lr.x = im->width;
        rec.lr.y = xpif_header.nav_split_line;

        LU_Verbose("Extracting the top part of the HRV image  (first %d lines)\n", xpif_header.nav_split_line);
        tmp_im = IP_ImageExtract( im, &rec);  JOE;
        IP_ImageFree(im);
        im = tmp_im;
       }

    if(CL_Opt('o'))
       {
        char *filename=NULL;
        char *filename_tmp=NULL;

        filename = CL_GetStrOpt('o'); JOE;
        LU_Verbose("Writing %s\n", filename);

		filename_tmp = StringConcat(filename, ".TEMP");

		IP_ImageWrite_XPIF(filename_tmp, im, &xpif_header, &xpif_tar_data); JOE;
        rename(filename_tmp, filename);

		deallocate( filename);
		deallocate( filename_tmp);
       }

	ERROR_HANDLER_BEGIN(argv[0]);
	rc = -1;
	ERROR_HANDLER_END;

    IP_ImageFree(im); im=NULL;

	RETURN(rc);
}




/**************************************************************

NAME
 PRO_Read -

SYNOPSIS
 HRIT_pro PRO_Read(char *file);

DESCRIPTION
  Read some fields of a HRIT "image repeat cycle prologue" file. Here
  the "PlannedCoverageHRV" record is read from the
  "ImageDescription" record and the "Level1_5ImageCalibration"
  records are read for the 12 channels in the "RadiometricProcessing"
  record. All this information is stored in the "HRIT_pro" structure.

AUTHOR
 Nicolas Clerbaux - RMIB

DATE
 26 Oct 2004

**************************************************************/
/* the defines hereafter include the 90 bytes of HRIT headers */

#define PRO_OFFSET_CALIBRATION           387155
#define PRO_OFFSET_HRV_PLANNED_COVERAGE  387037
#define PRO_PLANNED_CHAN_PROCESSING      387071

HRIT_pro PRO_Read(char *file)
{
int4        k, bflag;
FILE        *fp=NULL;
HRIT_pro    pro;

  bflag = 0;
  if(ByteSex()==LU_LITTLE_ENDIAN) bflag=1;

 fp = FM_FileOpen(file,"r"); JOE;

/* Get the HRV Planned Coverage */
   fseek(fp, PRO_OFFSET_HRV_PLANNED_COVERAGE , SEEK_SET);
   for(k=0;k<8;k++)
    {
     fread(&(pro.HRV_PlannedCoverage[k]), 4, 1, fp);
     if(bflag) MP_DataByteSwap( &(pro.HRV_PlannedCoverage[k]) , U_INT4);
    }

/* Get the Calibration */
 fseek(fp, PRO_OFFSET_CALIBRATION , SEEK_SET);
 for (k=0; k<12; k++)
    {
     fread(&(pro.cal_slope[k]), 8, 1, fp);
     fread(&(pro.cal_offset[k]),8,1,fp);
     if(bflag)
          {
           MP_DataByteSwap( &(pro.cal_slope[k]) , DOUBLE);
           MP_DataByteSwap( &(pro.cal_offset[k]) , DOUBLE);
          }
     }

/* Get the Planned Chan Processing value (0: not process, 1: process in spectral radiance, 2: process in effective radiance */
 fseek(fp, PRO_PLANNED_CHAN_PROCESSING , SEEK_SET);
 for (k=0; k<12; k++)
    {
     fread(&(pro.planned_chan_processing[k]), 1, 1, fp);
     }

   ERROR_HANDLER("PRO_Read");

  if(fp) fclose(fp);

  RETURN(pro);
}



/**************************************************************

NAME
 HRIT_ReadImageDataSegment -

SYNOPSIS
 void HRIT_ReadImageDataSegment(stringstream& strstream, IP_image *im, XPIF_tar_data *xpif_tar_data, int4 segment_id);

DESCRIPTION
 Read a HRIT "Image Data File" segment and fills the corresponding
 464 lines in the previously allocated image 'im'. If the
 structure 'xpif_tar_data != NULL' the HRIT header (which contains
 line validity information and acquisition time) is copied in the
 structure.

 Note: when the data are transcripted in the image 'im' the
 image rotation is performed.

AUTHOR
  Nioclas Clerbaux - RMIB

DATE
 26 Oct 2004

**************************************************************/

void HRIT_ReadImageDataSegment(std::stringstream& strstream, IP_image *im, XPIF_tar_data *xpif_tar_data)
{
//FILE    *fp=NULL;
u_c     nb_bit_pixel, line_side_info[13];
u_c    *line_data=NULL;
u_int2  nc, nl;
u_int4  total_header_length;
int4    k, line_num[NUM_LINE_SEGMENT], i, ii, j, w, h, bflag, line_size, ret ;
int4    nav_cfac2, nav_lfac2, nav_coff2, nav_loff2;
u_int2  v[4], segment_id;

  ERROR_IF_V(im==NULL);
  ERROR_IF_V(im->type!=INT2);
  w = im->width;
  h = im->height;
  ERROR_IF_M(w%8!=0,"image width must be a mulitple of 8");
  line_size = 10*(w/8);

  bflag = 0;
  if(ByteSex()==LU_LITTLE_ENDIAN) bflag=1;


 /* open the segment and read the headers */

  ERROR_IF_M(!strstream.good(), "Stream is corrupted");

  strstream.seekg(4, std::ios::beg);
  //fseek(fp, 4 , SEEK_SET);
  strstream.read((char *)(&total_header_length), 4);
  //fread(&total_header_length, 4, 1, fp );
  if(bflag) MP_DataByteSwap( &total_header_length , U_INT4);
  ERROR_IF_V(total_header_length!= HRIT_SEGMENT_HEADER_LENGTH );

  strstream.seekg(19, std::ios::beg);
  //fseek(fp, 19 , SEEK_SET);
  strstream.read((char *)(&nb_bit_pixel), 1);
  //fread(&nb_bit_pixel, 1, 1, fp );
  ERROR_IF_V(nb_bit_pixel!=10);

  strstream.read((char *)(&nc), 2);
  //fread(&nc, 2, 1, fp );
  if(bflag) MP_DataByteSwap( &nc , INT2);
  ERROR_IF_V(nc!=w);

  strstream.read((char *)(&nl), 2);
  //fread(&nl, 2, 1, fp );
  if(bflag) MP_DataByteSwap( &nl , INT2);
  ERROR_IF_V(nl!=NUM_LINE_SEGMENT);


 /* image segment number */
  strstream.seekg(156, std::ios::beg);
  //fseek(fp, 156 , SEEK_SET);
  strstream.read((char *)(&segment_id), 2);
  //fread( &segment_id , 2, 1, fp);
  if(bflag) MP_DataByteSwap( &segment_id , U_INT2);


 /* image NAVIGATION */
  strstream.seekg(60, std::ios::beg);
  //fseek(fp, 60 , SEEK_SET);
  strstream.read((char *)(&nav_cfac2), 4);
  //fread( &nav_cfac2 , 4, 1, fp);
  strstream.read((char *)(&nav_lfac2), 4);
  //fread( &nav_lfac2 , 4, 1, fp);
  strstream.read((char *)(&nav_coff2), 4);
  //fread( &nav_coff2 , 4, 1, fp);
  strstream.read((char *)(&nav_loff2), 4);
  //fread( &nav_loff2 , 4, 1, fp);
  if(bflag)
     {
      MP_DataByteSwap( &nav_cfac2 , INT4);
      MP_DataByteSwap( &nav_lfac2 , INT4);
      MP_DataByteSwap( &nav_coff2 , INT4);
      MP_DataByteSwap( &nav_loff2 , INT4);
     }

  if(ext_nav_flag==0)
      {
	   ext_nav_flag = 1;
       ext_nav_cfac = nav_cfac2 ;
       ext_nav_lfac = nav_lfac2 ;
       ext_nav_coff = nav_coff2 ;
       ext_nav_loff = nav_loff2 + (segment_id-1) * NUM_LINE_SEGMENT ;
      }


   ERROR_IF_V(ext_nav_cfac !=nav_cfac2);
   ERROR_IF_V(ext_nav_lfac !=nav_lfac2);
   if(ext_hrv_flag==0) ERROR_IF_V(ext_nav_coff !=nav_coff2);
   ERROR_IF_V(ext_nav_loff !=nav_loff2 + (segment_id-1) * NUM_LINE_SEGMENT );

 /* line number */
  strstream.seekg(163+3, std::ios::beg);
  //fseek(fp, 163+3 , SEEK_SET);
  for(k=0;k<NUM_LINE_SEGMENT;k++)
      {
        strstream.read((char *)line_side_info, 13);
       //fread( line_side_info , 1, 13, fp);
       memcpy( &(line_num[k]), &(line_side_info[0]), 4);
       if(bflag) MP_DataByteSwap( &(line_num[k]) , INT4);

       /* In the 'line_side_info' structure we can have line validity information (not done here)*/
      }


 if(xpif_tar_data!=NULL)
     {
      strstream.seekg(0, std::ios::beg);
      //fseek(fp, 0 , SEEK_SET);
      strstream.read((char *)xpif_tar_data->data + (segment_id-1) * total_header_length, total_header_length);
      //fread( xpif_tar_data->data + (segment_id-1) * total_header_length , 1, total_header_length, fp );
     }


 /* image line data */
  line_data = (u_c *) allocate( line_size );
  for(k=0;k<NUM_LINE_SEGMENT;k++)
       {
        j   = h - line_num[k];
        ret = strstream.readsome((char*)line_data, line_size);
       //ret = fread( line_data, 1, line_size, fp);
       ERROR_IF_M(ret!=line_size,"fread error");

        for(i=0;i<w/4;i++)
            {
             ii = w-1-i*4;

             Unpack4pix10bit(line_data + i*5, v);

             ((int2 **)im->data)[j][ii  ] = v[0];
             ((int2 **)im->data)[j][ii-1] = v[1];
             ((int2 **)im->data)[j][ii-2] = v[2];
             ((int2 **)im->data)[j][ii-3] = v[3];
             }
       }

  ERROR_HANDLER("HRIT_ReadImageDataSegment");

  deallocate(  line_data);

  ERROR_END;
}



/**************************************************************

NAME
 IP_ImageWrite_XPIF -

SYNOPSIS
 void IP_ImageWrite_XPIF(char *filename, IP_image *im, XPIF_header *xpif_header, XPIF_tar_data *xpif_tar_data);

DESCRIPTION
  Write an IP_image structure under the XPIF format. The
  XPIF format is the internal data format of the "VCS 2met!
  Processing Software".


  The navigation and calibration information that must
  be set in the XPIF primary header shoiuld be provided
  using the structure 'xpif_header'.

  If data are given in 'xpif_tar_data' they are written
  as "Textual Annotation Records"

AUTHOR
 Nicolas Clerbaux - RMIB

DATE
 26 Oct 2004

**************************************************************/

void IP_ImageWrite_XPIF(char *filename, IP_image *im, XPIF_header *xpif_header, XPIF_tar_data *xpif_tar_data)
{
u_c     header[256],  add_header[256], pif_k, pif_ext, pif_comp, pif_nb, nav_func;
u_int2  pif_n, pif_ahx;
u_int4  pif_l, pif_c;
int4    bflag=0, nav_loff, nav_coff, nav_lres, nav_cres, nav_goff_lon, nav_goff_lat;
int4    k, l, nav_split_line, nav_split_coff;
char    str[256];
double  nav_loff_real, nav_coff_real, nav_lres_real, nav_cres_real;
double  src_msg_cals, src_msg_calo;
FILE    *fp=NULL;
int2    *line=NULL;
int4    ret, nskip, j, num_tar_ext;
u_int2  ext_type, ext_stype;

int2    tmp_int2;
u_int2  tmp_u_int2;
u_int4  tmp_u_int4;
int4    tmp_int4;
double  tmp_double;
u_c     planned_chan_processing;

  ERROR_IF_V(im==NULL);
  ERROR_IF_V(im->type!=INT2);

 bflag=0;
 if(ByteSex()==LU_BIG_ENDIAN) bflag=1;

 /* fill the primary header */

  pif_k   = SizeOf(im->type) ;
  pif_n   = (im->width*pif_k)/256;
  if((im->width*pif_k)%256) pif_n++;
  pif_l   = im->height;
  pif_c   = im->width;

  num_tar_ext = 0;
  if(xpif_tar_data!=NULL)
      {
       /*num_tar_ext = xpif_tar_data->num_bytes/252;
       if( xpif_tar_data->num_bytes % 252) num_tar_ext++;*/
       num_tar_ext = INTEGER_DIVISION(  xpif_tar_data->num_bytes , 252);
      }

  if(num_tar_ext>197) num_tar_ext = 197; /* Added because the number of TARs is coded in u_c */
  pif_ext = 1 + num_tar_ext;

  pif_comp= 0;
  pif_ahx = 0;


  if(xpif_header!=NULL)
     {
      pif_nb  = (u_c) xpif_header->pif_nb;;
      nav_func= (u_c) xpif_header->nav_func;
      nav_loff= xpif_header->nav_loff;
      nav_coff= xpif_header->nav_coff;
      nav_lres= xpif_header->nav_lres;
      nav_cres= xpif_header->nav_cres;
      nav_goff_lon = xpif_header->nav_goff_lon;
      nav_goff_lat = xpif_header->nav_goff_lat;
      nav_split_line = xpif_header->nav_split_line;
      nav_split_coff = xpif_header->nav_split_coff;
      nav_loff_real = xpif_header->nav_loff_real;
      nav_coff_real = xpif_header->nav_coff_real;
      nav_lres_real = xpif_header->nav_lres_real;
      nav_cres_real = xpif_header->nav_cres_real;
      src_msg_cals  = xpif_header->src_msg_cals;
      src_msg_calo  = xpif_header->src_msg_calo;
      planned_chan_processing = xpif_header->planned_chan_processing;
    }
  else
     {
      pif_nb         = 0;
      nav_func       = 0;
      nav_loff       = 0;
      nav_coff       = 0;
      nav_lres       = 0;
      nav_cres       = 0;
      nav_goff_lon   = 0;
      nav_goff_lat   = 0;
      nav_split_line = 0;
      nav_split_coff = 0;
      nav_loff_real  = 0;
      nav_coff_real  = 0;
      nav_lres_real  = 0;
      nav_cres_real  = 0;
      src_msg_cals   = 0;
      src_msg_calo   = 0;
      planned_chan_processing = 1;
    }


  for(k=0;k<256;k++) header[k] = 0;

  header[0] = pif_k;

  tmp_u_int2 = pif_n;
  if(bflag) MP_DataByteSwap(&tmp_u_int2, U_INT2);
  memcpy(header+1, &tmp_u_int2, 2);

  tmp_u_int4 = pif_l;
  if(bflag) MP_DataByteSwap(&tmp_u_int4, U_INT4);
  memcpy(header+3, &tmp_u_int4, 4);

  tmp_u_int4 = pif_c;
  if(bflag) MP_DataByteSwap(&tmp_u_int4, U_INT4);
  memcpy(header+7, &tmp_u_int4, 4);

  memcpy(header+11, &pif_ext, 1);

  memcpy(header+12, &pif_comp, 1);

  tmp_u_int2 = pif_ahx;
  if(bflag) MP_DataByteSwap(&tmp_u_int2, U_INT2);
  memcpy(header+13, &tmp_u_int2, 2);

  memcpy(header+15, &pif_nb, 1);

  memcpy(header+16, &nav_func, 1);

  tmp_int4 = nav_loff;
  if(bflag) MP_DataByteSwap(&tmp_int4, INT4);
  memcpy(header+17, &tmp_int4, 4);

  tmp_int4 = nav_coff;
  if(bflag) MP_DataByteSwap(&tmp_int4, INT4);
  memcpy(header+21, &tmp_int4, 4);

  tmp_int4 = nav_lres;
  if(bflag) MP_DataByteSwap(&tmp_int4, INT4);
  memcpy(header+25, &tmp_int4, 4);

  tmp_int4 = nav_cres;
  if(bflag) MP_DataByteSwap(&tmp_int4, INT4);
  memcpy(header+29, &tmp_int4, 4);

  tmp_int4 = nav_goff_lon;
  if(bflag) MP_DataByteSwap(&tmp_int4, INT4);
  memcpy(header+33, &tmp_int4, 4);

  tmp_int4 = nav_goff_lat;
  if(bflag) MP_DataByteSwap(&tmp_int4, INT4);
  memcpy(header+37, &tmp_int4, 4);

  tmp_int4 = nav_split_line;
  if(bflag) MP_DataByteSwap(&tmp_int4, INT4);
  memcpy(header+41, &tmp_int4, 4);

  tmp_int4 = nav_split_coff;
  if(bflag) MP_DataByteSwap(&tmp_int4, INT4);
  memcpy(header+45, &tmp_int4, 4);

  tmp_double = nav_loff_real;
  if(bflag) MP_DataByteSwap(&tmp_double, DOUBLE);
  memcpy(header+49, &tmp_double, 8);

  tmp_double = nav_coff_real;
  if(bflag) MP_DataByteSwap(&tmp_double, DOUBLE);
  memcpy(header+57, &tmp_double, 8);

  tmp_double = nav_lres_real;
  if(bflag) MP_DataByteSwap(&tmp_double, DOUBLE);
  memcpy(header+65, &tmp_double, 8);

  tmp_double = nav_cres_real;
  if(bflag) MP_DataByteSwap(&tmp_double, DOUBLE);
  memcpy(header+73, &tmp_double, 8);

  if(xpif_header!=NULL)
     {
      header[128] = xpif_header->dat_f;
      memcpy(header+129, xpif_header->dat_nam, strlen(xpif_header->dat_nam));
      memcpy(header+145, xpif_header->dat_enh, strlen(xpif_header->dat_enh));
      memcpy(header+192, xpif_header->src_obj, strlen(xpif_header->src_obj));

      tmp_int2 = xpif_header->src_year;
      if(bflag) MP_DataByteSwap(&tmp_int2, INT2);
      memcpy(header+208, &tmp_int2, 2);

      tmp_int2 = xpif_header->src_day;
      if(bflag) MP_DataByteSwap(&tmp_int2, INT2);
      memcpy(header+210, &tmp_int2, 2);

      tmp_int2 = xpif_header->src_hour;
      if(bflag) MP_DataByteSwap(&tmp_int2, INT2);
      memcpy(header+212, &tmp_int2, 2);
     }



  tmp_double = src_msg_cals;
  if(bflag) MP_DataByteSwap(&tmp_double, DOUBLE);
  memcpy(header+214, &tmp_double, 8);

  tmp_double = src_msg_calo;
  if(bflag) MP_DataByteSwap(&tmp_double, DOUBLE);
  memcpy(header+222, &tmp_double, 8);

  memcpy(header+230, &planned_chan_processing, 1);


  sprintf(str,"USES_REAL_PARAMS");
  memcpy(header+81, str, 16);

  sprintf(str,"xPIF01.0");
  memcpy(header+248, str, strlen(str));

 /* opening the file */

  fp = FM_FileOpen(filename, "w"); JOE;

  ret = fwrite( header, 1, 256, fp);
  ERROR_IF_M(ret != 256, "fwrite error");

 /* data sync record */
  for(l=0;l<256;l++) add_header[l] = 0;

  ext_type  = 1;
  if(bflag) MP_DataByteSwap(&ext_type, U_INT2);
  memcpy(add_header+0, &ext_type, 2);

  ext_stype = 0;
  if(bflag) MP_DataByteSwap(&ext_stype, U_INT2);
  memcpy(add_header+2, &ext_stype, 2);

  ret = fwrite( add_header, 1, 256, fp);
  ERROR_IF_M(ret != 256, "fwrite error");


 /* textual annotation record */

  for(k=0;k<num_tar_ext;k++)
     {
       for(l=0;l<256;l++) add_header[l] = 0;

       ext_type  = 2;
       if(bflag) MP_DataByteSwap(&ext_type, U_INT2);
       memcpy(add_header+0, &ext_type, 2);

       ext_stype = k;
       if(bflag) MP_DataByteSwap(&ext_stype, U_INT2);
       memcpy(add_header+2, &ext_stype, 2);

       memcpy (add_header+4, xpif_tar_data->data + k*252,  MIN(252, xpif_tar_data->num_bytes - k * 252));

       ret = fwrite( add_header, 1, 256, fp);
       ERROR_IF_M(ret != 256, "fwrite error");
     }

   if(bflag)
      {
       line = (int2 *) allocate( pif_c * sizeof(int2)); JOE;
      }

   nskip = pif_n * 256 - pif_c * pif_k ;
   ERROR_IF_V(nskip < 0 || nskip >= 256);

   for (j = 0; j < (int4) pif_l; j++)
      {
       if(!bflag)
           line = (int2 *) im->data[j];
       else
          {
           memcpy(line, im->data[j], pif_k * pif_c);
           byte_swap_16(line, pif_c);
           }

       ret = fwrite(line, pif_k, pif_c, fp);
       ERROR_IF_M(ret != (int4)pif_c, "fwrite error");

       ret = fwrite(add_header, 1, nskip, fp);
       ERROR_IF_M(ret != nskip, "fwrite error");
       }


   for(k=0;k<pif_ahx;k++)
      {
       ret = fwrite( add_header, 1, 256, fp);
       ERROR_IF_V(ret!=256);
      }

  ERROR_HANDLER("IP_ImageWrite_XPIF");

  if(fp) fclose(fp);
  if(bflag) deallocate( line);

  ERROR_END;
}



void HRIT_SegmentNameAnalysis(char *file, int4 *sb, int4 *seg_id, char *slot)
{
int4    k;
char    *basename=NULL, *product_id2=NULL, *product_id3=NULL, *product_id4=NULL;

   ERROR_IF_V(file==NULL);

   basename = SM_StringExtractBasename( file);

   ERROR_IF_M1(strlen(basename)!=61,"Seems not be a valid HRIT segments %s", basename);

   product_id2 = SM_StringExtract( basename, 26, 35); /* spectral band */
   product_id3 = SM_StringExtract( basename, 36, 42); /* segment id */
   product_id4 = SM_StringExtract( basename, 46, 58); /* slot */

   if(strcmp( product_id2,"_________")==0)
     {
      *sb    = -1;
      *seg_id = -3;
      if(strcmp( product_id3,"PRO___")==0) *seg_id = -1;
      if(strcmp( product_id3,"EPI___")==0) *seg_id = -2;
      }
   else
      {
      (*sb) = -1;
      for(k=0;k<12;k++)
         if(strcmp(product_id2, spectral_band_id[k])==0) (*sb) = k;


      ERROR_IF_V( !SM_StringIsIntegerValue(product_id3));
      sscanf(product_id3,"%d", seg_id);
     }

   sprintf(slot,"%s",  product_id4);

  ERROR_HANDLER_BEGIN("HRIT_SegmentNameAnalysis");
  ErrorPrintf("file is : %s\n", file);
  ERROR_HANDLER_END;

  deallocate(basename);
  deallocate(product_id2);
  deallocate(product_id3);
  deallocate(product_id4);

  ERROR_END;
}


/* From SAF NWC library */
void Unpack4pix10bit(unsigned char *data, unsigned short *a)
{
  a[0]=(data[0]*256+data[1])/64;
  a[1]=((data[1]*256+data[2])&16368)/16;
  a[2]=((data[2]*256+data[3])&4092)/4;
  a[3]=((data[3]*256+data[4])&1023);
}



void help_hrit2xpif(void)
{
  printf("\n"
  "\n/*help ========================================================"
  "\n"
  "\n   Concatenation of  HRIT segments to create "
  "\n   a XPIF image. The HRIT segments given in argument should "
  "\n   belong to an unique spectral band (e.g. VIS006, IR_120, ...) and "
  "\n   a time slot (e.g. 200410250745). The repeat cycle prologue "
  "\n   (PRO) segment for this time slot should also be given because "
  "\n   it contains the calibration as well as the HRV navigation "
  "\n   information."
  "\n   "
  "\n   This version of the program supports compressed as well as "
  "\n   uncompressed HRIT segments as input. "
  "\n"
  "\n   USAGE:     "
  "\n     hrit2xpif   {HRIT_segments}  {HRIT pro}  [-o out.XPIF] [options]"
  "\n"
  "\n"
  "\n   OPTIONS:"
  "\n    -h: display this help"
  "\n    -v: verbose mode"
  "\n    -hrv_top: create HRV image that only contains the upper part."
  "\n"
  "\n   AUTHOR: Nicolas Clerbaux -- RMIB"
  "\n           Alessandro Ipe -- RMIB"
  "\n"
  "\n   ============================================================ */"
  "\n"  );
}


